package com.hz.pm.api.common.handler;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.hz.pm.api.user.util.LoginUserUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

import static com.hz.pm.api.common.model.constant.BaseFieldConst.*;

/**
 * <p>
 * 字段自动填充处理
 * </p>
 *
 * @author WendyYang
 * @since 17:20 2022/11/24
 */
@Slf4j
@Component
public class BaseFieldFillHandler implements MetaObjectHandler {

    private Long getUserId() {
        if (LoginUserUtil.isLogin()) {
            return LoginUserUtil.getUserId();
        }
        return 0L;
    }

    @Override
    public void insertFill(MetaObject metaObject) {
        fillVal(CREATE_BY, metaObject, getUserId());
        fillVal(UPDATE_BY, metaObject, getUserId());
        LocalDateTime now = LocalDateTime.now();
        fillVal(CREATE_ON, metaObject, now);
        fillVal(UPDATE_ON, metaObject, now);
    }

    @Override
    public void updateFill(MetaObject metaObject) {
        fillVal(UPDATE_BY, metaObject, getUserId());
        fillVal(UPDATE_ON, metaObject, LocalDateTime.now());
    }

    private void fillVal(String fieldName, MetaObject metaObject, Object val) {
        if (metaObject.hasSetter(fieldName)) {
            Class<?> setterType = metaObject.getSetterType(fieldName);
            if (setterType != null) {
                if (setterType.equals(String.class)) {
                    this.fillStrategy(metaObject, fieldName, val.toString());
                } else {
                    this.fillStrategy(metaObject, fieldName, val);
                }
            }
        }
    }

}