package com.hz.pm.api.common.statemachine.action;

import com.hz.pm.api.common.statemachine.builder.impl.AdaptStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.AdaptStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderAdaptStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderSelfTestStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

import static com.hz.pm.api.common.statemachine.action.TenderStateChangeActionUtil.getPurchaseInfo;

/**
 * <p>
 * 测试验证状态机action集合类
 * </p>
 *
 * @author WendyYang
 * @since 17:08 2024/2/24
 */
@Slf4j
@WithStateMachine(id = AdaptStateMachineBuilderImpl.MACHINE_ID)
public class AdaptStateChangeAction {

    @OnTransition(source = "WITHOUT_ADAPT_INFO", target = "ADAPT_INFO_AUDIT")
    public void SUBMIT_ADAPT_INFO(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderAdaptStatus.ADAPT_INFO_AUDIT.getCode());
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_AUDIT.getCode());
    }

    @OnTransition(source = "ADAPT_INFO_AUDIT", target = "ADAPT_INFO_PASSED")
    public void ADAPT_INFO_PASSED(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO.getCode());
        purchase.setSelfTestStatus(TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO.getCode());
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_PASSED.getCode());
    }

    @OnTransition(source = "ADAPT_INFO_AUDIT", target = "ADAPT_INFO_FAILED")
    public void ADAPT_INFO_FAILED(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderAdaptStatus.ADAPT_INFO_FAILED.getCode());
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_FAILED.getCode());
    }

    @OnTransition(source = "ADAPT_INFO_FAILED", target = "ADAPT_INFO_AUDIT")
    public void RESUBMIT_ADAPT_INFO(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderAdaptStatus.ADAPT_INFO_AUDIT.getCode());
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_AUDIT.getCode());
    }

}
