package com.hz.pm.api.common.statemachine.action;

import com.hz.pm.api.common.statemachine.builder.impl.SelfTestStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.SelfTestStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderSelfTestStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderTestValidStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

import static com.hz.pm.api.common.statemachine.action.TenderStateChangeActionUtil.getPurchaseInfo;

/**
 * <p>
 * 系统自测状态机action集合类
 * </p>
 *
 * @author WendyYang
 * @since 17:08 2024/2/24
 */
@Slf4j
@WithStateMachine(id = SelfTestStateMachineBuilderImpl.MACHINE_ID)
public class SelfTestStateChangeAction {

    @OnTransition(source = "WITHOUT_SELF_TEST_INFO", target = "SELF_TEST_INFO_AUDIT")
    public void SUBMIT_SELF_TEST_INFO(Message<SelfTestStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT.getCode());
        purchase.setSelfTestStatus(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT.getCode());
    }

    @OnTransition(source = "SELF_TEST_INFO_AUDIT", target = "SELF_TEST_INFO_PASSED")
    public void SELF_TEST_INFO_PASSED(Message<SelfTestStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderTestValidStatus.WITHOUT_TEST_VALID_INFO.getCode());
        purchase.setTestValidStatus(TenderTestValidStatus.WITHOUT_TEST_VALID_INFO.getCode());
        purchase.setSelfTestStatus(TenderSelfTestStatus.SELF_TEST_INFO_PASSED.getCode());
    }

    @OnTransition(source = "SELF_TEST_INFO_AUDIT", target = "SELF_TEST_INFO_FAILED")
    public void SELF_TEST_INFO_FAILED(Message<SelfTestStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderSelfTestStatus.SELF_TEST_INFO_FAILED.getCode());
        purchase.setSelfTestStatus(TenderSelfTestStatus.SELF_TEST_INFO_FAILED.getCode());
    }

    @OnTransition(source = "SELF_TEST_INFO_FAILED", target = "SELF_TEST_INFO_AUDIT")
    public void RESUBMIT_SELF_TEST_INFO(Message<SelfTestStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT.getCode());
        purchase.setSelfTestStatus(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT.getCode());
    }

}
