package com.hz.pm.api.common.statemachine.action;

import com.hz.pm.api.common.statemachine.builder.impl.TenderStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderAdaptStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

import static com.hz.pm.api.common.statemachine.action.TenderStateChangeActionUtil.getPurchaseInfo;

/**
 * <p>
 * 标段状态机action集合类
 * </p>
 *
 * @author WendyYang
 * @since 17:08 2024/2/24
 */
@Slf4j
@WithStateMachine(id = TenderStateMachineBuilderImpl.MACHINE_ID)
public class TenderStateChangeAction {

    /**
     * 填写合同信息
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO", target = "TO_BE_SUBMIT_OPERATION_PLAN")
    public void SUBMIT_PURCHASE_CONSTRUCTION_INFO(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setAdaptStatus(TenderAdaptStatus.WITHOUT_ADAPT_INFO.getCode());
        purchase.setStatus(TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN.getCode());
    }

    /**
     * 填写合同信息
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO", target = "TO_BE_SUBMIT_FIRST_INSPECTED_INFO")
    public void SUBMIT_PURCHASE_CONSTRUCTION_INFO_SKIP_TO_FINAL_INSPECT(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO.getCode());
    }

    /**
     * 提交单位确认
     *
     * @param message \
     */
    @OnTransition(source = "WAIT_ORG_CONFIRM", target = "TO_BE_SUBMIT_FIRST_INSPECTED_INFO")
    public void SUBMIT_PURCHASE_ORG_CONFIRM(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO.getCode());
    }

    /**
     * 填写实施计划
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_OPERATION_PLAN", target = "WITHOUT_ADAPT_INFO")
    public void SUBMIT_OPERATION_PLAN(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setAdaptStatus(TenderAdaptStatus.WITHOUT_ADAPT_INFO.getCode());
        purchase.setStatus(TenderAdaptStatus.WITHOUT_ADAPT_INFO.getCode());
    }

    /**
     * 上传初验材料
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_FIRST_INSPECTED_INFO", target = "TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY")
    public void SUBMIT_FIRST_INSPECTED_FILES(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY.getCode());
    }

    /**
     * 终验申报
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY", target = "ON_FINALLY_INSPECTED_APPLY")
    public void SUBMIT_FINALLY_INSPECTED(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.ON_FINALLY_INSPECTED_APPLY.getCode());
    }

    /**
     * 终验申报重新提交
     *
     * @param message \
     */
    @OnTransition(source = "FINALLY_INSPECTED_FAILED", target = "ON_FINALLY_INSPECTED_APPLY")
    public void RESUBMIT_FINALLY_INSPECTED(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.ON_FINALLY_INSPECTED_APPLY.getCode());
    }

    /**
     * 终验申报通过
     *
     * @param message \
     */
    @OnTransition(source = "ON_FINALLY_INSPECTED_APPLY", target = "FINALLY_INSPECTED_PASSED")
    public void FINALLY_INSPECTED_PASSED(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.FINALLY_INSPECTED_PASSED.getCode());
    }

    /**
     * 终验申报拒绝
     *
     * @param message \
     */
    @OnTransition(source = "ON_FINALLY_INSPECTED_APPLY", target = "FINALLY_INSPECTED_FAILED")
    public void FINALLY_INSPECTED_FAILED(Message<TenderStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.FINALLY_INSPECTED_FAILED.getCode());
    }

}
