package com.hz.pm.api.common.statemachine.action;

import com.hz.pm.api.common.statemachine.builder.impl.TestValidStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.TestValidStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderTestValidStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

import static com.hz.pm.api.common.statemachine.action.TenderStateChangeActionUtil.getPurchaseInfo;

/**
 * <p>
 * 测试验证状态机action集合类
 * </p>
 *
 * @author WendyYang
 * @since 17:08 2024/2/24
 */
@Slf4j
@WithStateMachine(id = TestValidStateMachineBuilderImpl.MACHINE_ID)
public class TestValidStateChangeAction {

    @OnTransition(source = "WITHOUT_TEST_VALID_INFO", target = "TEST_VALID_INFO_AUDIT")
    public void SUBMIT_TEST_VALID_INFO(Message<TestValidStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderTestValidStatus.TEST_VALID_INFO_AUDIT.getCode());
        purchase.setTestValidStatus(TenderTestValidStatus.TEST_VALID_INFO_AUDIT.getCode());
    }

    @OnTransition(source = "TEST_VALID_INFO_AUDIT", target = "TEST_VALID_INFO_PASSED")
    public void TEST_VALID_INFO_PASSED(Message<TestValidStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderMainStatus.WAIT_ORG_CONFIRM.getCode());
        purchase.setTestValidStatus(TenderTestValidStatus.TEST_VALID_INFO_PASSED.getCode());
    }

    @OnTransition(source = "TEST_VALID_INFO_AUDIT", target = "TEST_VALID_INFO_FAILED")
    public void TEST_VALID_INFO_FAILED(Message<TestValidStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderTestValidStatus.TEST_VALID_INFO_FAILED.getCode());
        purchase.setTestValidStatus(TenderTestValidStatus.TEST_VALID_INFO_FAILED.getCode());
    }

    @OnTransition(source = "TEST_VALID_INFO_FAILED", target = "TEST_VALID_INFO_AUDIT")
    public void RESUBMIT_TEST_VALID_INFO(Message<TestValidStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderTestValidStatus.TEST_VALID_INFO_AUDIT.getCode());
        purchase.setTestValidStatus(TenderTestValidStatus.TEST_VALID_INFO_AUDIT.getCode());
    }

}
