package com.hz.pm.api.common.statemachine.factory;

import com.hz.pm.api.common.statemachine.action.TenderStateChangeActionUtil;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ITenderStatus;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.statemachine.StateContext;
import org.springframework.statemachine.guard.Guard;

/**
 * <p>
 * TenderGuardFactory
 * </p>
 *
 * @author WendyYang
 * @since 17:51 2024/8/20
 */
@Configuration
public class TenderGuardFactory {

    @Bean
    public PurchaseContractRecordToOperationGuard purchaseContractRecordToOperationGuard() {
        return new PurchaseContractRecordToOperationGuard();
    }

    @Bean
    public PurchaseContractRecordToAcceptGuard purchaseContractRecordToAcceptGuard() {
        return new PurchaseContractRecordToAcceptGuard();
    }

    public static class PurchaseContractRecordToOperationGuard implements Guard<ITenderStatus, TenderStateChangeEvent> {
        @Override
        public boolean evaluate(StateContext<ITenderStatus, TenderStateChangeEvent> context) {
            Purchase purchase = TenderStateChangeActionUtil.getPurchaseInfo(context.getMessage());
            return BidTypeEnum.BUILD_APP.eq(purchase.getBidType());
        }
    }

    public static class PurchaseContractRecordToAcceptGuard implements Guard<ITenderStatus, TenderStateChangeEvent> {
        @Override
        public boolean evaluate(StateContext<ITenderStatus, TenderStateChangeEvent> context) {
            Purchase purchase = TenderStateChangeActionUtil.getPurchaseInfo(context.getMessage());
            return !BidTypeEnum.BUILD_APP.eq(purchase.getBidType());
        }
    }

}
