package com.hz.pm.api.common.statemachine.util;

import cn.hutool.log.Log;
import cn.hutool.log.LogFactory;
import com.hz.pm.api.common.statemachine.event.AbstractStateChangeEvent;
import com.ningdatech.basic.exception.BizException;
import com.wflow.exception.BusinessException;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.function.Function;

/**
 * <p>
 * AbstractStateMachineUtil
 * </p>
 *
 * @author WendyYang
 * @since 19:39 2024/4/2
 */
public interface AbstractStateMachineUtil<O, E extends Enum<E> & AbstractStateChangeEvent> {

    Log LOG = LogFactory.get(AbstractStateMachineUtil.class);

    /**
     * 获取状态变更事件类型
     *
     * @return java.lang.Class<E>
     **/
    default Class<E> eventClass() {
        Type type = getClass().getGenericInterfaces()[0];
        return (Class<E>) ((ParameterizedType) type).getActualTypeArguments()[1];
    }

    default AbstractStateMachineUtil<O, E> getLockClass() {
        LOG.info("acquire state machine lock：{}", this.getClass().getSimpleName());
        return this;
    }

    //通过审核
    default void pass(O obj) {
        try {
            synchronized (getLockClass()) {
                execute(obj, AbstractStateChangeEvent.getPassEvent(eventClass(), statusFunction().apply(obj)));
            }
        } catch (Exception e) {
            LOG.error("状态机通过失败", e);
            throw BizException.wrap("状态机通过失败");
        }
    }


    //拒绝
    default void reject(O obj) {
        try {
            synchronized (getLockClass()) {
                execute(obj, AbstractStateChangeEvent.getRejectEvent(eventClass(), statusFunction().apply(obj)));
            }
        } catch (Exception e) {
            LOG.error("状态机拒绝失败", e);
            throw new BusinessException("状态机拒绝失败");
        }
    }

    //撤回
    default void withDraw(O obj) {
        try {
            synchronized (getLockClass()) {
                execute(obj, AbstractStateChangeEvent.getWithdrawEvent(eventClass(), statusFunction().apply(obj)));
            }
        } catch (Exception e) {
            LOG.error("状态机撤回失败", e);
            throw new BusinessException("状态机撤回失败");
        }
    }

    Function<O, Integer> statusFunction();

    void execute(O o, E event) throws Exception;


}
