package com.hz.pm.api.common.statemachine.util;

import com.hz.pm.api.common.statemachine.builder.impl.TenderStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.ITenderStatus;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.function.Function;

/**
 * <p>
 * 标段状态机工具类
 * </p>
 *
 * @author WendyYang
 * @since 17:06 2024/2/24
 */
@Slf4j
@Component
public class TenderStateMachineUtil implements AbstractStateMachineUtil<Purchase, TenderStateChangeEvent> {

    public static final String PURCHASE = "purchaseInfo";

    //==================================================================================================================

    private final IPurchaseStatusChangeService purchaseStatusChangeService;
    private final StateMachinePersister<ITenderStatus, TenderStateChangeEvent, Purchase> stateMachinePersister;
    private final StateMachine<ITenderStatus, TenderStateChangeEvent> stateMachine;

    public TenderStateMachineUtil(IPurchaseStatusChangeService purchaseStatusChangeService, TenderStateMachineBuilderImpl tenderStateMachineBuilder) {
        this.purchaseStatusChangeService = purchaseStatusChangeService;
        this.stateMachinePersister = tenderStateMachineBuilder.stateMachinePersister();
        this.stateMachine = tenderStateMachineBuilder.build();
    }

    @Override
    public Function<Purchase, Integer> statusFunction() {
        return Purchase::getStatus;
    }

    @Override
    public void execute(Purchase purchase, TenderStateChangeEvent event) throws Exception {
        // 获取TO状态机
        PurchaseStatusChange change = new PurchaseStatusChange();
        change.setBeforeStatus(purchase.getStatus());
        change.setEvent(event.name());
        change.setBidId(purchase.getId());
        change.setProjectId(purchase.getProjectId());
        change.setProjectCode(ProjectIdCodeCacheUtil.get(purchase.getProjectId()));
        change.setCreateOn(LocalDateTime.now());
        Message<TenderStateChangeEvent> message = MessageBuilder.withPayload(event)
                .setHeader(PURCHASE, purchase)
                .build();
        //初始化状态机
        stateMachinePersister.restore(stateMachine, purchase);
        stateMachine.sendEvent(message);
        change.setAfterStatus(purchase.getStatus());
        purchaseStatusChangeService.save(change);
        log.info("调用状态机后的标段状态为：{} ==> {}", change.getBeforeStatus(), change.getAfterStatus());
    }

}
