package com.hz.pm.api.common.statemachine.util;

import com.hz.pm.api.common.statemachine.builder.impl.TestValidStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.TestValidStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderTestValidStatus;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.function.Function;

/**
 * <p>
 * 标段状态机工具类
 * </p>
 *
 * @author WendyYang
 * @since 17:06 2024/2/24
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class TestValidStateMachineUtil implements AbstractStateMachineUtil<Purchase, TestValidStateChangeEvent> {

    public static final String PURCHASE = "purchaseInfo";

    //==================================================================================================================

    private final TestValidStateMachineBuilderImpl testValidStateMachineBuilder;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    @Override
    public Function<Purchase, Integer> statusFunction() {
        return Purchase::getTestValidStatus;
    }

    @Override
    public void execute(Purchase purchase, TestValidStateChangeEvent event) throws Exception {
        PurchaseStatusChange change = new PurchaseStatusChange();
        change.setBeforeStatus(purchase.getTestValidStatus());
        change.setEvent(event.name());
        change.setBidId(purchase.getId());
        change.setProjectId(purchase.getProjectId());
        change.setProjectCode(ProjectIdCodeCacheUtil.get(purchase.getProjectId()));
        change.setCreateOn(LocalDateTime.now());
        StateMachine<TenderTestValidStatus, TestValidStateChangeEvent> stateMachine = testValidStateMachineBuilder.build();
        Message<TestValidStateChangeEvent> message = MessageBuilder.withPayload(event)
                .setHeader(PURCHASE, purchase)
                .build();
        //初始化状态机
        StateMachinePersister<TenderTestValidStatus, TestValidStateChangeEvent, Purchase> stateMachinePersister = testValidStateMachineBuilder.stateMachinePersister();
        stateMachinePersister.restore(stateMachine, purchase);
        stateMachine.sendEvent(message);
        change.setAfterStatus(purchase.getTestValidStatus());
        purchaseStatusChangeService.save(change);
        log.info("调用状态机后的标段状态为：{} ==> {}", change.getBeforeStatus(), change.getAfterStatus());
    }

}
