package com.hz.pm.api.common.statemachine.util;

import com.hz.pm.api.common.statemachine.builder.impl.XcfhxStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.XcfhxStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderXcfhxApplyStatus;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.function.Function;

/**
 * <p>
 * 标段状态机工具类
 * </p>
 *
 * @author WendyYang
 * @since 17:06 2024/2/24
 */
@Slf4j
@Component
public class XcfhxStateMachineUtil implements AbstractStateMachineUtil<Purchase, XcfhxStateChangeEvent> {

    public static final String PURCHASE = "purchaseInfo";

    //==================================================================================================================

    private final IPurchaseStatusChangeService purchaseStatusChangeService;
    private final StateMachinePersister<TenderXcfhxApplyStatus, XcfhxStateChangeEvent, Purchase> stateMachinePersister;
    private final StateMachine<TenderXcfhxApplyStatus, XcfhxStateChangeEvent> stateMachine;

    public XcfhxStateMachineUtil(XcfhxStateMachineBuilderImpl xcfhxStateMachineBuilder,
                                 IPurchaseStatusChangeService purchaseStatusChangeService) {
        this.purchaseStatusChangeService = purchaseStatusChangeService;
        this.stateMachine = xcfhxStateMachineBuilder.build();
        this.stateMachinePersister = xcfhxStateMachineBuilder.stateMachinePersister();
    }

    @Override
    public Function<Purchase, Integer> statusFunction() {
        return Purchase::getXcfhxApplyStatus;
    }

    @Override
    public void execute(Purchase purchase, XcfhxStateChangeEvent event) throws Exception {
        PurchaseStatusChange change = new PurchaseStatusChange();
        change.setBeforeStatus(purchase.getXcfhxApplyStatus());
        change.setEvent(event.name());
        change.setBidId(purchase.getId());
        change.setProjectId(purchase.getProjectId());
        change.setProjectCode(ProjectIdCodeCacheUtil.get(purchase.getProjectId()));
        change.setCreateOn(LocalDateTime.now());
        Message<XcfhxStateChangeEvent> message = MessageBuilder.withPayload(event)
                .setHeader(PURCHASE, purchase)
                .build();
        //初始化状态机
        stateMachinePersister.restore(stateMachine, purchase);
        stateMachine.sendEvent(message);
        change.setAfterStatus(purchase.getXcfhxApplyStatus());
        purchaseStatusChangeService.save(change);
        log.info("调用状态机后的标段状态为：{} ==> {}", change.getBeforeStatus(), change.getAfterStatus());
    }

}
