package com.hz.pm.api.common.util;

import org.apache.poi.ss.usermodel.*;

import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;

/**
 * <p>
 * ExcelExportStyle
 * </p>
 *
 * @author WendyYang
 * @since 01:39 2022/11/5
 */
public class ExcelExportStyle {

    public static HorizontalCellStyleStrategy formalStyle() {
        // 表头样式策略
        WriteCellStyle headStyle = new WriteCellStyle();
        // 是否换行
        headStyle.setWrapped(false);
        // 水平对齐方式
        headStyle.setHorizontalAlignment(HorizontalAlignment.LEFT);
        // 垂直对齐方式
        headStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        // 前景色
        headStyle.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        // 背景色
        headStyle.setFillBackgroundColor(IndexedColors.AUTOMATIC.getIndex());

        // 设置为1时，单元格将被前景色填充
        headStyle.setFillPatternType(FillPatternType.SOLID_FOREGROUND);
        // 控制单元格是否应自动调整大小以适应文本过长时的大小
        headStyle.setShrinkToFit(false);
        // 单元格边框类型
        headStyle.setBorderBottom(BorderStyle.NONE);
        headStyle.setBorderLeft(BorderStyle.NONE);
        headStyle.setBorderRight(BorderStyle.NONE);
        headStyle.setBorderTop(BorderStyle.NONE);
        // 单元格边框颜色
        headStyle.setLeftBorderColor(IndexedColors.BLACK.index);
        headStyle.setRightBorderColor(IndexedColors.BLACK.index);
        headStyle.setTopBorderColor(IndexedColors.BLACK.index);
        headStyle.setBottomBorderColor(IndexedColors.BLACK.index);
        // 字体策略
        WriteFont writeFont = new WriteFont();
        // 是否加粗/黑体
        writeFont.setBold(false);
        // 字体颜色
        writeFont.setColor(Font.COLOR_NORMAL);
        // 字体名称
        writeFont.setFontName("宋体");
        // 字体大小
        writeFont.setFontHeightInPoints((short) 11);
        // 是否使用斜体
        writeFont.setItalic(false);
        // 是否在文本中使用横线删除
        writeFont.setStrikeout(false);
        // 设置要使用的文本下划线的类型
        writeFont.setUnderline(Font.U_NONE);
        // 设置要使用的字符集
        writeFont.setCharset((int) Font.DEFAULT_CHARSET);
        headStyle.setWriteFont(writeFont);

        // 内容样式策略策略
        WriteCellStyle contentStyle = new WriteCellStyle();
        contentStyle.setFillForegroundColor(IndexedColors.WHITE.getIndex());
        contentStyle.setHorizontalAlignment(HorizontalAlignment.GENERAL);
        contentStyle.setBorderBottom(BorderStyle.NONE);
        contentStyle.setBorderLeft(BorderStyle.NONE);
        contentStyle.setBorderRight(BorderStyle.NONE);
        contentStyle.setBorderTop(BorderStyle.NONE);
        contentStyle.setFillPatternType(FillPatternType.NO_FILL);
        contentStyle.setWrapped(false);
        return new HorizontalCellStyleStrategy(headStyle, contentStyle);
    }

}
