package com.hz.pm.api.common.util;

import cn.hutool.core.collection.CollUtil;
import com.hz.pm.api.common.model.ExpertRegionModifyDiffBO;
import com.hz.pm.api.expert.model.RegionDtoMapKey;
import com.hz.pm.api.meta.model.ExpertRegionInfo;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author liuxinxin
 * @since 2022/7/28 下午4:33
 */

public class ExpertRegionInfoUtil {

    private ExpertRegionInfoUtil() {
    }

    public static ExpertRegionModifyDiffBO diff(List<ExpertRegionInfo> preRegions, List<ExpertRegionInfo> currRegions) {
        ExpertRegionModifyDiffBO diff = new ExpertRegionModifyDiffBO();
        List<ExpertRegionInfo> addList = new ArrayList<>();
        List<ExpertRegionInfo> removeList = new ArrayList<>();
        if (CollUtil.isEmpty(preRegions) && CollUtil.isEmpty(currRegions)) {
            diff.setAddList(addList);
            diff.setRemoveList(removeList);
        } else if (CollUtil.isEmpty(preRegions)) {
            diff.setAddList(currRegions);
            diff.setRemoveList(removeList);
        } else if (CollUtil.isEmpty(currRegions)) {
            diff.setAddList(addList);
            diff.setRemoveList(preRegions);
        } else {
            List<RegionDtoMapKey> preRegionMapKey = convert(preRegions);
            List<RegionDtoMapKey> currRegionMapKey = convert(currRegions);
            removeList.addAll(notContains(currRegionMapKey, preRegionMapKey));
            addList.addAll(notContains(preRegionMapKey, currRegionMapKey));
            diff.setAddList(addList);
            diff.setRemoveList(removeList);
        }
        return diff;

    }

    private static List<ExpertRegionInfo> notContains(List<RegionDtoMapKey> source, List<RegionDtoMapKey> target) {
        return target.stream().filter(w -> !source.contains(w)).map(key -> {
            ExpertRegionInfo expertRegionInfo = new ExpertRegionInfo();
            expertRegionInfo.setRegionCode(key.getRegionCode());
            expertRegionInfo.setRegionLevel(key.getRegionLevel());
            return expertRegionInfo;
        }).collect(Collectors.toList());
    }

    private static List<RegionDtoMapKey> convert(List<ExpertRegionInfo> regionInfos) {
        return regionInfos.stream().map(r -> {
            RegionDtoMapKey regionDtoMapKey = new RegionDtoMapKey();
            regionDtoMapKey.setRegionLevel(r.getRegionLevel());
            regionDtoMapKey.setRegionCode(r.getRegionCode());
            return regionDtoMapKey;
        }).collect(Collectors.toList());
    }

}
