package com.hz.pm.api.common.util;

import cn.hutool.core.codec.Base64Decoder;
import cn.hutool.core.codec.Base64Encoder;
import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * <p>
 * gzip压缩工具类
 * </p>
 *
 * @author WendyYang
 */
public class GzipUtil {

    private GzipUtil() {
    }

    /**
     * 使用gzip进行压缩
     */
    public static String zip(String content) {
        if (StrUtil.isBlank(content)) {
            return content;
        }
        try (ByteArrayOutputStream bos = new ByteArrayOutputStream();
             GZIPOutputStream gzip = new GZIPOutputStream(bos)) {
            gzip.write(content.getBytes());
            gzip.finish();
            byte[] bytes = bos.toByteArray();
            return Base64Encoder.encode(bytes);
        } catch (IOException e) {
            throw new BizException(e);
        }
    }

    /**
     * 使用gzip进行解压缩
     */
    public static String unzip(String content) {
        if (StrUtil.isBlank(content)) {
            return content;
        }
        try (ByteArrayInputStream bis = new ByteArrayInputStream(Base64Decoder.decode(content));
             ByteArrayOutputStream bos = new ByteArrayOutputStream();
             GZIPInputStream gis = new GZIPInputStream(bis)) {
            byte[] buffer = new byte[1024 * 16];
            int offset;
            while ((offset = gis.read(buffer)) != -1) {
                bos.write(buffer, 0, offset);
            }
            return bos.toString();
        } catch (IOException e) {
            throw new BizException(e);
        }
    }

}
