package com.hz.pm.api.common.util;

import cn.hutool.core.map.MapUtil;
import org.slf4j.MDC;

import java.util.Map;
import java.util.concurrent.Callable;

/**
 * <p>
 * MDCThreadPoolUtil
 * </p>
 *
 * @author WendyYang
 * @since 2023/10/27
 **/
public class MDCThreadUtil {

    private MDCThreadUtil() {
    }

    public static <T> Callable<T> wrap(final Callable<T> callable) {
        Map<String, String> context = MDC.getCopyOfContextMap();
        return () -> {
            if (MapUtil.isNotEmpty(context)) {
                MDC.setContextMap(context);
            }
            try {
                return callable.call();
            } finally {
                MDC.clear();
            }
        };
    }

    public static Runnable wrap(final Runnable runnable) {
        Map<String, String> context = MDC.getCopyOfContextMap();
        return () -> {
            if (MapUtil.isNotEmpty(context)) {
                MDC.setContextMap(context);
            }
            try {
                runnable.run();
            } finally {
                MDC.clear();
            }
        };
    }

}