package com.hz.pm.api.common.util;

import cn.hutool.extra.spring.SpringUtil;
import com.hz.pm.api.common.config.ThreadPoolUtilProperties;
import com.hz.pm.api.common.config.ThreadPoolUtilProperties.ThreadPoolProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;

import java.util.concurrent.ThreadPoolExecutor;

/**
 * <p>
 * ThreadPoolUtil
 * </p>
 *
 * @author WendyYang
 * @since 2023/10/27
 **/
@Slf4j
public class ThreadPoolUtil {

    private ThreadPoolUtil() {
    }

    public static final ThreadPoolTaskExecutor REQUEST;
    public static final ThreadPoolTaskScheduler SCHEDULER;

    static {
        ThreadPoolUtilProperties properties = SpringUtil.getBean(ThreadPoolUtilProperties.class);
        ThreadPoolProperties requestProps = properties.getRequest();
        REQUEST = new MDCThreadPoolTaskExecutor();
        REQUEST.setCorePoolSize(requestProps.getCorePoolSize());
        REQUEST.setMaxPoolSize(requestProps.getMaxPoolSize());
        REQUEST.setQueueCapacity(requestProps.getQueueCapacity());
        REQUEST.setThreadPriority(Thread.NORM_PRIORITY);
        REQUEST.setAllowCoreThreadTimeOut(requestProps.getAllowCoreThreadTimeOut());
        REQUEST.setWaitForTasksToCompleteOnShutdown(requestProps.getWaitForTasksToCompleteOnShutdown());
        REQUEST.setKeepAliveSeconds(requestProps.getKeepAliveSeconds());
        REQUEST.setThreadNamePrefix(requestProps.getThreadNamePrefix());
        REQUEST.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        REQUEST.initialize();
        ThreadPoolProperties schedulerProps = properties.getScheduler();
        SCHEDULER = new MDCThreadPoolTaskScheduler();
        SCHEDULER.setThreadNamePrefix(schedulerProps.getThreadNamePrefix());
        SCHEDULER.setPoolSize(schedulerProps.getCorePoolSize());
        SCHEDULER.setThreadPriority(Thread.NORM_PRIORITY - 1);
        SCHEDULER.setWaitForTasksToCompleteOnShutdown(schedulerProps.getWaitForTasksToCompleteOnShutdown());
        SCHEDULER.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        SCHEDULER.initialize();
    }

}
