package com.hz.pm.api.dashboard.manage;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.model.entity.DataDTO;
import com.hz.pm.api.dashboard.constant.DashboardConstant;
import com.hz.pm.api.dashboard.model.vo.CostStatisticsVO;
import com.hz.pm.api.meeting.entity.domain.Meeting;
import com.hz.pm.api.meeting.entity.domain.MeetingExpert;
import com.hz.pm.api.meeting.service.IMeetingExpertService;
import com.hz.pm.api.meeting.service.IMeetingService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Classname ProjectCostStatisticsManage
 * @Description
 * @Date 2023/12/05 17:44
 * @Author PoffyZhang
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class ProjectCostStatisticsManage {

    private final IMeetingExpertService meetingExpertService;
    private final IMeetingService meetingService;

    //评审费  每人500 (0.05万)
    private static final BigDecimal REVIEW_AMOUNT_EXPERT = DashboardConstant.Expert.REVIEW_AOUMT_EXPERT;

    private List<Integer> getRecentTwoYears() {
        int currYear = LocalDate.now().getYear();
        return Arrays.asList(currYear - 1, currYear);
    }

    public CostStatisticsVO statistics() {
        CostStatisticsVO res = new CostStatisticsVO();
        List<Meeting> meetings = meetingService.list(Wrappers.lambdaQuery(Meeting.class)
                .select(Meeting::getStartTime, Meeting::getId));

        //算出所有的 同意的专家
        List<MeetingExpert> meetingExperts;

        if (meetings.isEmpty()) {
            meetingExperts = Collections.emptyList();
        } else {
            List<Long> meetingIds = CollUtils.fieldList(meetings, Meeting::getId);
            meetingExperts = meetingExpertService.listAgreedExperts(meetingIds);
        }

        //计算累积
        res.setReviewAmount(REVIEW_AMOUNT_EXPERT.multiply(BigDecimal.valueOf(meetingExperts.size())));
        res.setPricingAmount(BigDecimal.ZERO);

        //近两年
        List<DataDTO> twoYearsReviews = Lists.newArrayList();
        List<DataDTO> twoYearsPricing = Lists.newArrayList();
        List<DataDTO> twoYearsAvg = Lists.newArrayList();

        DataDTO reviewAvg = new DataDTO();
        reviewAvg.setName("评审费");
        DataDTO pricingAvg = new DataDTO();
        pricingAvg.setName("核价费");
        BigDecimal reviewTwoYearTotal = BigDecimal.ZERO;

        Map<Integer, List<Meeting>> meetingMapYear = CollUtils.group(meetings, w -> w.getStartTime().getYear());
        Map<Long, List<MeetingExpert>> expertMapMeetingId = CollUtils.group(meetingExperts, MeetingExpert::getMeetingId);

        for (Integer thisYear : getRecentTwoYears()) {
            //查出 年份的 会议数据
            List<Meeting> yearMeetings = meetingMapYear.getOrDefault(thisYear, Collections.emptyList());
            List<MeetingExpert> yearMeetingExperts = yearMeetings.stream()
                    .map(w -> expertMapMeetingId.get(w.getId()))
                    .filter(Objects::nonNull)
                    .flatMap(Collection::stream)
                    .collect(Collectors.toList());
            //算出所有的 同意的专家
            BigDecimal yearExpertCnt = BigDecimal.valueOf(yearMeetingExperts.size());
            BigDecimal yearExpertAmount = REVIEW_AMOUNT_EXPERT.multiply(yearExpertCnt);
            twoYearsReviews.add(DataDTO.of(thisYear.toString(), yearExpertAmount));
            twoYearsPricing.add(DataDTO.of(thisYear.toString(), BigDecimal.ZERO));

            reviewTwoYearTotal = reviewTwoYearTotal.add(yearExpertAmount);
        }
        reviewAvg.setAmount(reviewTwoYearTotal.divide(BigDecimal.valueOf(2), BigDecimal.ROUND_CEILING,
                RoundingMode.HALF_UP));
        twoYearsAvg.add(reviewAvg);
        twoYearsAvg.add(pricingAvg);
        res.setTwoYearsReviews(twoYearsReviews);
        res.setTwoYearsPricing(twoYearsPricing);
        res.setTwoYearsAvg(twoYearsAvg);
        return res;
    }
}
