package com.hz.pm.api.datascope.provider;

import cn.hutool.core.collection.CollUtil;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.SpringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 * 数据权限获取
 * </p>
 *
 * @author WendyYang
 * @since 2022/1/9 23:28
 */
public class DataScopeContext {

    private DataScopeContext() {
    }

    private static final String WARN_MSG = "请先创建数据权限 [%s] 的实现类";

    private static final Map<String, DataScopeProvider> DSP_MAP;

    static {
        DSP_MAP = new HashMap<>(16);
        Map<String, DataScopeProvider> beansMap = SpringUtils.getBeansOfType(DataScopeProvider.class);
        if (CollUtil.isNotEmpty(beansMap)) {
            for (DataScopeProvider value : beansMap.values()) {
                DSP_MAP.put(value.findRole().name(), value);
            }
        }
    }

    public static Optional<DataScopeDTO> getDataScope(String roleCode) {
        DataScopeProvider dataScopeProvider = DSP_MAP.get(roleCode);
        if (dataScopeProvider == null) {
            throw BizException.wrap(WARN_MSG, roleCode);
        }
        return dataScopeProvider.findDataFieldProperty();
    }

    public static Optional<DataScopeDTO> getDataScopeHasUserId(String roleCode, Long userId) {
        DataScopeProvider dataScopeProvider = DSP_MAP.get(roleCode);
        if (dataScopeProvider == null) {
            throw BizException.wrap(WARN_MSG, roleCode);
        }
        return dataScopeProvider.findDataFieldProperty(userId);
    }

}
