package com.hz.pm.api.expert.service;

import com.hz.pm.api.expert.model.entity.ExpertReview;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author WendyYang
 * @since 2023-02-15
 */
public interface IExpertReviewService extends IService<ExpertReview> {

    /**
     * 根据项目ID和专家ID获取评审记录
     *
     * @param projectId 项目ID
     * @param expertId  专家ID
     * @param meetingId 会议ID
     * @return 评审记录
     * @author WendyYang
     **/
    List<ExpertReview> listByProjectIdAndExpertId(Long projectId, Long expertId, Long meetingId);

    /**
     * 获取最终评审结果
     *
     * @param meetingId 会议ID
     * @param projectId 项目ID
     * @return {@link ExpertReview}
     * @author WendyYang
     **/
    ExpertReview getFinalReview(Long meetingId, Long projectId);

    /**
     * 获取最终评审结果
     *
     * @param meetingId 会议ID
     * @return {@link ExpertReview}
     * @author WendyYang
     **/
    Map<Long, ExpertReview> listFinalReviewMap(Long meetingId);

    /**
     * 获取最终评审结果
     *
     * @param meetingId 会议ID
     * @return {@link ExpertReview}
     * @author WendyYang
     **/
    List<ExpertReview> listFinalReview(Long meetingId);

    /**
     * 查某个项目 某一个 阶段的 最终会议
     * @param meetingType
     * @return
     */
    ExpertReview getFinalReview(String projectCode,Integer meetingType);
}
