package com.hz.pm.api.external;

import cn.hutool.http.useragent.UserAgentUtil;
import com.hz.pm.api.meeting.entity.config.WebProps;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.service.FileService;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * <p>
 * FilePreviewClient
 * </p>
 *
 * @author WendyYang
 * @since 09:27 2024/3/18
 */
@Component
@RequiredArgsConstructor
public class FilePreviewClient {

    private static final Logger log = LoggerFactory.getLogger(FilePreviewClient.class);

    @Value("${mh.file-preview.public-host:}")
    private String mhFilePreviewPublicUrl;

    @Value("${mh.file-preview.private-host:}")
    private String mhFilePreviewPrivateUrl;

    private final FileService fileService;

    private static final String FILE_DOWN_NO_AUTH_URL = "/api/v1/file/preview/anonymous/down/";

    //==================================================================================================================

    private static final String PREVIEW_PATH_FMT = "%s/public/risen/core/resrc/view/pdf_view.html.do?strMap.remote_url=%s&strMap.ext=%s";

    public String getFilePreviewUrl(Long fileId, HttpServletRequest request) {
        String serverName = request.getServerName();
        log.info("serverName:{}", serverName);
        if (serverName.contains("weixin") || serverName.contains("10.54.38.13")) {
            // 浙政钉访问需要转换为互联网可访问的地址
            String fileDownUrl = WebProps.zzdApiUrl + FILE_DOWN_NO_AUTH_URL + fileId;
            File file = fileService.getById(fileId);
            return String.format(PREVIEW_PATH_FMT, mhFilePreviewPublicUrl, fileDownUrl, file.getSuffix());
        } else {
            String fileDownUrl = WebProps.apiUrl + FILE_DOWN_NO_AUTH_URL + fileId;
            File file = fileService.getById(fileId);
            return String.format(PREVIEW_PATH_FMT, mhFilePreviewPrivateUrl, fileDownUrl, file.getSuffix());
        }
    }

    public void downloadFile(Long fileId, HttpServletResponse response) {
        try {
            fileService.download(fileId, response);
        } catch (Exception e) {
            log.info("文件下载失败：{}", fileId, e);
            throw BizException.wrap("文件下载失败：%s", fileId);
        }
    }

}
