package com.hz.pm.api.external;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.hz.pm.api.external.model.dto.CallRetDTO;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;

import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * MobileCallClient
 * </p>
 *
 * @author WendyYang
 * @since 17:20 2024/4/25
 */
@Slf4j
public class MobileCallClient {

    @Data
    public static class SendCallDTO {

        private String requestId;

    }

    @Data
    public static class ReplyCallDTO {

        private Integer duration;

        private String remark;

        private Integer dialStatus;

        private String remoteNumber;

        private String dtmf;

        /**
         * 提交时间
         */
        private Date sendDate;

        /**
         * 接通时间
         */
        private Date answerDate;

        /**
         * 拨号时间
         */
        private Date dialDate;

    }

    //==================================================================================================================

    private static final Object LOCK = new Object();

    private static final String CALL_HOST_KEY = "mobile-call.host";
    private static final String CALL_OPEN_KEY = "mobile-call.open";

    //==================================================================================================================

    private static volatile MobileCallClient client;

    private static volatile boolean mobileCallOpen = Boolean.TRUE;

    public static MobileCallClient getClient() {
        if (mobileCallOpen && client == null) {
            synchronized (LOCK) {
                if (client == null) {
                    Environment environment = SpringUtil.getBean(Environment.class);
                    mobileCallOpen = environment.getProperty(CALL_OPEN_KEY, Boolean.class, Boolean.TRUE);
                    if (mobileCallOpen) {
                        String callHost = environment.getProperty(CALL_HOST_KEY);
                        Assert.notBlank(callHost, "语音服务地址不能为空");
                        client = new MobileCallClient(callHost);
                    }
                }
            }
        }
        Assert.isTrue(mobileCallOpen, "语音服务未开启");
        return client;
    }

    private final String callHost;

    private MobileCallClient(String callHost) {
        this.callHost = callHost;
    }


    private static final String MOBILE_CALL_URL = "/send_batch_voice_sms";
    private static final String MOBILE_REPLY_URL = "/query_voice_sms";

    public CallRetDTO<SendCallDTO> sendCall(Collection<String> phones, String content) {
        String url = callHost + MOBILE_CALL_URL;
        JSONObject bodyObj = new JSONObject();
        bodyObj.set("remote_numbers", phones);
        bodyObj.set("content", content);
        String retStr = HttpUtil.post(url, bodyObj.toString());
        return JSONUtil.toBean(retStr, new TypeReference<CallRetDTO<SendCallDTO>>() {
        }, false);
    }

    public CallRetDTO<List<ReplyCallDTO>> replyCall(String requestId) {
        String url = callHost + MOBILE_REPLY_URL + "?request_id=" + requestId;
        String retStr = HttpUtil.get(url);
        return JSONUtil.toBean(retStr, new TypeReference<CallRetDTO<List<ReplyCallDTO>>>() {
        }, false);
    }

}
