package com.hz.pm.api.external.model.enumeration;


import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * CallDialStatus
 * </p>
 *
 * @author WendyYang
 * @since 10:27 2024/4/30
 */
@Getter
@AllArgsConstructor
public enum CallDialStatus {

    /**
     * 未拨号
     */
    NOT_DIALED(0, "未拨号"),

    /**
     * 拨号中
     */
    DIALING(1, "拨号中"),

    /**
     * 通话中
     */
    IN_CALL(2, "通话中"),

    /**
     * 通话结束（已接通）
     */
    CALL_ENDED_CONNECTED(3, "通话结束（已接通）"),

    /**
     * 通话结束（未接通）
     */
    CALL_ENDED_DISCONNECTED(4, "通话结束（未接通）");

    private final Integer status;
    private final String remark;

    /**
     * 根据整型值获取对应的DialStatus枚举
     *
     * @param status 整型状态值
     * @return 对应的DialStatus枚举，如果状态值不匹配则返回null
     */
    public static Optional<CallDialStatus> get(Integer status) {
        return Arrays.stream(values()).filter(w -> w.getStatus().equals(status)).findFirst();
    }

    public static CallDialStatus getNoNull(Integer status) {
        return get(status).orElseThrow(() -> new IllegalArgumentException("无效的拨号状态"));
    }

}
