package com.hz.pm.api.filemanage.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.CommonConst;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.expert.manage.ExpertReviewManage;
import com.hz.pm.api.expert.model.vo.ExpertReviewDetailVO;
import com.hz.pm.api.expert.model.vo.ProjectReviewDetailVO;
import com.hz.pm.api.filemanage.model.entity.ProjectApplyBorrow;
import com.hz.pm.api.filemanage.model.param.ProjectFileListParam;
import com.hz.pm.api.filemanage.model.vo.ProjectApplyBorrowVO;
import com.hz.pm.api.filemanage.model.vo.ProjectFileListVO;
import com.hz.pm.api.filemanage.model.vo.ProjectFileVO;
import com.hz.pm.api.filemanage.service.INdProjectApplyBorrowService;
import com.hz.pm.api.projectdeclared.manage.ConstructionManage;
import com.hz.pm.api.projectdeclared.manage.DeclaredProjectHelper;
import com.hz.pm.api.projectdeclared.manage.FinalAcceptanceManage;
import com.hz.pm.api.projectdeclared.manage.PurchaseManage;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.vo.ContractVO;
import com.hz.pm.api.projectdeclared.model.vo.FinalAcceptanceVO;
import com.hz.pm.api.projectdeclared.model.vo.PreInsVO;
import com.hz.pm.api.projectdeclared.model.vo.PurchaseVO;
import com.hz.pm.api.projectdeclared.service.IContractService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectApplication;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.vo.ProjectDetailVO;
import com.hz.pm.api.projectlib.service.IProjectApplicationService;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.staging.enums.MsgTypeEnum;
import com.hz.pm.api.sys.manage.NoticeManage;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.todocenter.constant.WorkNoticeConst;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.service.FileService;
import com.ningdatech.file.utils.StrPool;
import com.wflow.bean.entity.WflowModels;
import com.wflow.contants.HisProInsEndActId;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.process.FileBasicInfo;
import com.wflow.workflow.bean.process.ProcessComment;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.HistoryService;
import org.flowable.engine.TaskService;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.engine.task.Comment;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @Classname ProjectFileManage
 * @Description
 * @Date 2023/6/6 16:04
 * @Author PoffyZhang
 */
@Component
@AllArgsConstructor
@Slf4j
public class ProjectFileManage {

    private final IProjectService projectService;
    private final ProjectLibManage projectLibManage;
    private final UserInfoHelper userInfoHelper;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final ProcessInstanceService processInstanceService;
    private final NoticeManage noticeManage;
    private final IProjectInstService projectInstService;
    private final HistoryService historyService;
    private final INdProjectApplyBorrowService projectApplyBorrowService;
    private final IProjectApplicationService projectApplicationService;
    private final TaskService taskService;
    private final IPurchaseService purchaseService;
    private final IContractService contractService;
    private final ExpertReviewManage expertReviewManage;
    private final FileService fileService;
    private final PurchaseManage purchaseManage;
    private final ConstructionManage constructionManage;
    private final FinalAcceptanceManage finalAcceptanceManage;
    private final MhUnitCache mhUnitCache;

    public PageVo<ProjectFileListVO> list(ProjectFileListParam req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .like(StrUtils.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName())
                .like(StrUtils.isNotBlank(req.getBuildOrgName()), Project::getBuildOrgName, req.getBuildOrgName())
                .eq(Project::getStatus, ProjectStatus.ACCEPTED.getCode())
                .orderByDesc(Project::getUpdateOn);
        Page<Project> page = projectService.page(req.page(), query);
        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }
        // 获取本单位及下属单位的单位code
        List<Long> subOrgSet = getSubOrgList(user.getMhUnitId());

        List<ProjectFileListVO> vos = page.getRecords().stream().map(p -> {
            ProjectFileListVO vo = new ProjectFileListVO();
            vo.setId(p.getId());
            // 用户只可查阅自己及下属单位的项目档案和已经申请借阅通过的项目档案
            if (!NumberUtil.isNumber(p.getBuildOrgCode())
                    || subOrgSet.contains(Long.parseLong(p.getBuildOrgCode()))
                    || checkCanRead(subOrgSet, p)) {
                vo.setCanRead(Boolean.TRUE);
            }
            vo.setBuildOrgName(p.getBuildOrgName());
            vo.setPojectCode(p.getProjectCode());
            vo.setProjectName(p.getProjectName());
            // 判断当前项目,当前登录用户是否已经开启了申请借阅并且没有审批完成
            Wrapper<ProjectApplyBorrow> pabQuery = Wrappers.lambdaQuery(ProjectApplyBorrow.class)
                    .select(ProjectApplyBorrow::getInstanceId)
                    .eq(ProjectApplyBorrow::getApplyBorrowUserId, user.getUserIdStr())
                    .eq(ProjectApplyBorrow::getProjectId, p.getId())
                    .orderByDesc(ProjectApplyBorrow::getCreateOn)
                    .last(BizConst.LIMIT_1);
            ProjectApplyBorrow projectApplyBorrow = projectApplyBorrowService.getOne(pabQuery);
            if (Objects.nonNull(projectApplyBorrow)) {
                String instCode = projectApplyBorrow.getInstanceId();
                // 获取流程实例
                HistoricProcessInstance newInstance = historyService.createHistoricProcessInstanceQuery()
                        .processInstanceId(instCode)
                        .singleResult();
                if (newInstance == null) {
                    vo.setCanApplyBorrow(Boolean.FALSE);
                    return vo;
                }
                ProjectInst projectInst = projectInstService.getOne(Wrappers.lambdaQuery(ProjectInst.class)
                        .eq(ProjectInst::getInstCode, instCode));
                // 推送省局联审的项目流程实例为EMPTY
                if (Objects.nonNull(projectInst)) {
                    Integer instType = projectInst.getInstType();
                    // 从申请借阅信息表中查询当前项目是否有当前登录用户发起的申请借阅审批流程
                    ProjectApplyBorrow applyBorrow = projectApplyBorrowService.getOne(Wrappers
                            .lambdaQuery(ProjectApplyBorrow.class)
                            .eq(ProjectApplyBorrow::getApplyBorrowUserId, user.getUserIdStr())
                            .eq(ProjectApplyBorrow::getProjectId, p.getId())
                            .eq(ProjectApplyBorrow::getInstanceId, instCode));
                    // 延期申请流程还未审核结束
                    if (Objects.nonNull(applyBorrow) && InstTypeEnum.APPLY_BORROW.eq(instType)
                            && Objects.isNull(newInstance.getEndActivityId())) {
                        vo.setCanApplyBorrow(Boolean.FALSE);
                    } else if (InstTypeEnum.APPLY_BORROW.eq(instType) &&
                            HisProInsEndActId.REJECT.equals(newInstance.getEndActivityId())) {
                        // 如果是延期申请审核被驳回,设置项目可以申请延期申报
                        vo.setCanApplyBorrow(Boolean.TRUE);
                    }
                }
            }
            return vo;
        }).collect(Collectors.toList());
        return PageVo.of(vos, page.getTotal());
    }

    private boolean checkCanRead(List<Long> subOrgSet, Project project) {
        // 从申请借阅信息表中查出本单位及下属单位审批通过的项目
        List<String> subOrgSetStr = CollUtils.convert(subOrgSet, String::valueOf);
        HashSet<Long> borrowProjectIdSet = new HashSet<>();
        List<Long> applyBorrowProjectIdList = projectApplyBorrowService
                .list(Wrappers.lambdaQuery(ProjectApplyBorrow.class)
                        .select(ProjectApplyBorrow::getProjectId)
                        .in(ProjectApplyBorrow::getApplyBorrowOrgCode, subOrgSetStr)
                        .eq(ProjectApplyBorrow::getIsSuccess, Boolean.TRUE))
                .stream().map(ProjectApplyBorrow::getProjectId)
                .filter(borrowProjectIdSet::add)
                .collect(Collectors.toList());
        return applyBorrowProjectIdList.contains(project.getId());
    }

    private List<Long> getSubOrgList(Long mhUnitId) {
        return mhUnitCache.getViewChildIdsRecursion(mhUnitId);
    }

    public ProjectFileVO file(Long projectId) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        // 判断该项目是否为本单位或下属单位的项目或者已经申请借阅成功的项目
        Project project = projectService.getById(projectId);
        List<Long> subOrgList = getSubOrgList(user.getMhUnitId());

        if (!checkCanRead(subOrgList, project)) {
            throw new BizException("您没有权限查看此项目档案!");
        }
        ProjectFileVO projectFileVo = new ProjectFileVO();
        String projectCode = project.getProjectCode();
        ProjectDetailVO projectDetail = projectLibManage.getProjectDetail(project);
        List<ProcessDetailVO> processDetail = projectLibManage.processScheduleDetail(project);
        ProjectReviewDetailVO projectReviewDetail = expertReviewManage.projectExpertReviewDetail(projectCode);
        List<PurchaseVO> purchaseVos = purchaseManage.detailByProjectId(projectId);
        List<ContractVO> contractVos = constructionManage.detailContractByProjectId(projectId);
        List<PreInsVO> preInsVo = constructionManage.detailPreInsByProjectId(projectId);
        FinalAcceptanceVO finalAcceptanceVo = finalAcceptanceManage.detailByProject(project);

        projectFileVo.setProjectDetailVo(projectDetail);
        projectFileVo.setProcessDetailVos(processDetail);
        projectFileVo.setProjectReviewDetailVo(projectReviewDetail);
        projectFileVo.setPurchaseVos(purchaseVos);
        projectFileVo.setContractVos(contractVos);
        projectFileVo.setPreInsVo(preInsVo);
        projectFileVo.setFinalAcceptanceVo(finalAcceptanceVo);

        return projectFileVo;
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized String startApplyBorrowProcess(Long projectId) {
        Project project = projectService.getNewestNoNull(projectId);

        // 首先要判断 项目当前状态 是不是 已归档
        if (!ProjectStatus.ACCEPTED.eq(project.getStatus())) {
            throw BizException.wrap("提交失败 该项目不是 已归档");
        }

        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        WflowModels model = processModelManage.getWflowModelsNoNull(ProjectProcessType.APPLY_BORROW,
                user.getMhUnitId());

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectHelper.buildUser(user));
        params.setProcessUsers(Collections.emptyMap());
        // 放入条件判断的项目字段
        // 把条件值给放入工作流
        declaredProjectHelper.buildCondition(params, project);
        // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMapUserOrgCode(project);
        String instanceId =
                processInstanceService.newStartProcess(model.getProcessDefId(), model.getFormId(), params, orgModelMap);
        log.info("申请借阅申报成功 【{}】", instanceId);

        // 保存项目借阅实例相关
        projectLibManage.saveProjectByApplyBorrow(project, user, instanceId);

        // 发送给第一个审批人消息
        noticeManage.sendFirstUser(project, model.getFormName(), instanceId, WorkNoticeConst.PASS_MSG_TEMPLATE,
                MsgTypeEnum.PROJECT_REVIEW);

        return instanceId;
    }

    public void downloadFile(Long projectId, HttpServletResponse response) {
        Project project = projectService.getById(projectId);
        // 获取项目各阶段上传文件ID
        List<Long> fileIdList = Lists.newArrayList();
        getProjectAppendix(project, fileIdList);
        if (fileIdList.isEmpty()) {
            throw BizException.wrap("暂无档案可供下载");
        }
        List<File> files = fileService.listByIds(fileIdList);
        if (files.isEmpty()) {
            throw BizException.wrap("暂无档案可供下载");
        }
        String zipFileName = "项目档案";
        // 设置response的header
        response.setContentType("application/zip");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + ExcelDownUtil.encodeName(zipFileName) + ".zip");

        try {
            // 创建ZipOutputStream
            ZipOutputStream zos = new ZipOutputStream(response.getOutputStream());
            // 获取文件输入流
            for (File file : files) {
                String originalFileName = file.getOriginalFileName();
                String name = originalFileName.substring(0, originalFileName.indexOf(StrPool.DOT));
                String fileName = name + StrPool.UNDERSCORE + file.getId()
                        + originalFileName.substring(originalFileName.indexOf(StrPool.DOT));
                InputStream fileInputStream = fileService.getFileInputStream(file);
                // 文件
                ZipEntry zipEntry = new ZipEntry(fileName);
                zos.putNextEntry(zipEntry);
                byte[] bytes = new byte[1024];
                int length;
                while ((length = fileInputStream.read(bytes)) >= 0) {
                    zos.write(bytes, 0, length);
                }
                fileInputStream.close();
                zos.closeEntry();
            }
            zos.close();
        } catch (IOException e) {
            log.error("项目档案下载失败：", e);
            throw BizException.wrap("项目档案下载失败");
        }
    }

    private void getProjectAppendix(Project project, List<Long> fileIdList) {
        Long projectId = project.getId();
        // 项目申报阶段
        // 获取立项依据
        String buildBasis = project.getBuildBasis();
        List<JSONObject> buildBasisFileArray = JSON.parseArray(buildBasis, JSONObject.class);
        for (JSONObject jsonObject : buildBasisFileArray) {
            String fileInfo = jsonObject.getString(CommonConst.FILE_LIST);
            List<Long> buildBasisFileIdList = getFileIdList(fileInfo);
            fileIdList.addAll(buildBasisFileIdList);
        }
        // 获取附件-初步方案
        String preliminaryPlanFile = project.getPreliminaryPlanFile();
        List<Long> prePlanFileIdList = getFileIdList(preliminaryPlanFile);
        fileIdList.addAll(prePlanFileIdList);
        // 获取附件-佐证材料
        String supMaterialsFile = project.getSupportingMaterialsFile();
        List<Long> supMaterialFileIdList = getFileIdList(supMaterialsFile);
        fileIdList.addAll(supMaterialFileIdList);
        // 获取附件-项目总投资测算明细
        String calTotalInvestFile = project.getCalculationTotalInvestmentFile();
        List<Long> calTotalInvestFileIdList = getFileIdList(calTotalInvestFile);
        fileIdList.addAll(calTotalInvestFileIdList);
        // 获取附件-申报单位主要职责(单位三定方案)
        String mainResAppFile = project.getMainResponsibilitiesApplicantFile();
        List<Long> mainResAppFileIdList = getFileIdList(mainResAppFile);
        fileIdList.addAll(mainResAppFileIdList);
        // 获取附件-项目申报书
        String projectApplicationForm = project.getProjectApplicationForm();
        List<Long> projectAppFormFileIdList = getFileIdList(projectApplicationForm);
        fileIdList.addAll(projectAppFormFileIdList);
        // 获取项目应用信息中的文件
        List<ProjectApplication> applicationList = projectApplicationService
                .list(Wrappers.lambdaQuery(ProjectApplication.class).eq(ProjectApplication::getProjectId, projectId));
        for (ProjectApplication projectApplication : applicationList) {
            // 获取试点文件
            String experimentsFile = projectApplication.getExperimentsFile();
            List<Long> experimentsFileIdList = getFileIdList(experimentsFile);
            fileIdList.addAll(experimentsFileIdList);
            // 获取应用总投资测算明细
            String appEstimateFile = projectApplication.getApplicationEstimateFile();
            List<Long> appEstimateFileIdList = getFileIdList(appEstimateFile);
            fileIdList.addAll(appEstimateFileIdList);
        }
        // 流程审核意见文件
        // 获取项目关联的流程实例
        List<ProjectInst> projectInstList =
                projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class).eq(ProjectInst::getProjectId, projectId));
        // 从历史评论表中获取对应流程实例的评论附件信息
        for (ProjectInst projectInst : projectInstList) {
            String instanceId = projectInst.getInstCode();
            List<Comment> commentList = taskService.getProcessInstanceComments(instanceId);
            for (Comment comment : commentList) {
                ProcessComment processComment = JSONObject.parseObject(comment.getFullMessage(), ProcessComment.class);
                List<FileBasicInfo> attachments = processComment.getAttachments();
                if (CollUtil.isNotEmpty(attachments)) {
                    List<Long> idList = attachments.stream().map(FileBasicInfo::getFileId).collect(Collectors.toList());
                    fileIdList.addAll(idList);
                }
            }
        }

        // 预审申报阶段
        // 预审申报上级条线主管单位审核意见
        String higherLineSuperOrgReviewComments = project.getHigherLineSuperOrgReviewComments();
        if (StringUtils.isNotBlank(higherLineSuperOrgReviewComments)) {
            List<Long> highSuperOrgFileIdList = getFileIdList(higherLineSuperOrgReviewComments);
            fileIdList.addAll(highSuperOrgFileIdList);
        }

        // 方案申报阶段
        // 获取建设方案文件
        String constructionPlanFile = project.getConstructionPlanFile();
        String str = StrPool.LEFT_SQ_BRACKET + StrPool.RIGHT_SQ_BRACKET;
        if (!str.equals(constructionPlanFile)) {
            JSONArray array = JSON.parseArray(constructionPlanFile);
            array.forEach(o -> {
                JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(o));
                Long constructFileId = jsonObject.getLong(CommonConst.FILE_ID_STR);
                fileIdList.add(constructFileId);
            });
        }

        // 立项批复阶段
        // 获取批复文件
        String approvedFile = project.getApprovedFile();
        if (Objects.nonNull(approvedFile)) {
            fileIdList.add(Long.valueOf(approvedFile));
        }
        // 获取建设方案文件
        String approvedConstructionPlanFile = project.getApprovedConstructionPlanFile();
        if (Objects.nonNull(approvedConstructionPlanFile)) {
            fileIdList.add(Long.valueOf(approvedConstructionPlanFile));
        }

        // 采购结果备案阶段
        Purchase purchase = purchaseService
                .getOne(Wrappers.lambdaQuery(Purchase.class).eq(Purchase::getProjectId, projectId).last(BizConst.LIMIT_1));
        // 获取投标文件
        if (Objects.nonNull(purchase)) {
            String biddingDoc = purchase.getBiddingDoc();
            List<Long> biddingFileIdList = getFileIdList(biddingDoc);
            fileIdList.addAll(biddingFileIdList);
            // 获取招标文件
            String bidDoc = purchase.getBidDoc();
            List<Long> bidFileIdList = getFileIdList(bidDoc);
            fileIdList.addAll(bidFileIdList);
            // 获取中标通知书
            String acceptanceLetter = purchase.getAcceptanceLetter();
            List<Long> acceptFileIdList = getFileIdList(acceptanceLetter);
            fileIdList.addAll(acceptFileIdList);
        }

        // 合同备案阶段
        Contract contract = contractService
                .getOne(Wrappers.lambdaQuery(Contract.class).eq(Contract::getProjectId, projectId).last(BizConst.LIMIT_1));
        // 获取合同附件
        if (Objects.nonNull(contract)) {
            String attachment = contract.getAttachment();
            List<Long> contractFileIdList = getFileIdList(attachment);
            fileIdList.addAll(contractFileIdList);
        }

        // 获取终验材料
        String finalAcceptanceMaterials = project.getFinalAcceptanceMaterials();
        getInspectionMaterials(fileIdList, finalAcceptanceMaterials);

        // 专家评审记录-如果有初步方案评审会议、部门联审评审会议、建设方案评审会议、验收评审会议对应的附件
        // 获取项目的所有评审意见
        String projectCode = project.getProjectCode();
        ProjectReviewDetailVO projectReviewDetailVO = expertReviewManage.projectExpertReviewDetail(projectCode);
        Collection<ProjectReviewDetailVO.ReviewDetailByTypeVO> reviews = projectReviewDetailVO.getReviews();
        // 获取最终评审意见附件
        if (CollUtil.isNotEmpty(reviews)) {
            List<Long> finalReviewFileIdList = reviews.stream().map(r -> {
                ExpertReviewDetailVO finalReview = r.getFinalReview();
                return finalReview.getAttachFileId();
            }).collect(Collectors.toList());
            fileIdList.addAll(finalReviewFileIdList);
            // 获取组员评审意见
            for (ProjectReviewDetailVO.ReviewDetailByTypeVO review : reviews) {
                List<ExpertReviewDetailVO> teamMemberReviews = review.getTeamMemberReviews();
                for (ExpertReviewDetailVO teamMemberReview : teamMemberReviews) {
                    Long attachFileId = teamMemberReview.getAttachFileId();
                    fileIdList.add(attachFileId);
                }
            }
        }
    }

    private void getInspectionMaterials(List<Long> fileIdList, String preliminaryInspectionMaterials) {
        List<String> fileStrList = JSON.parseArray(preliminaryInspectionMaterials, String.class);
        if (CollUtil.isNotEmpty(fileStrList)) {
            for (String fileStr : fileStrList) {
                JSONObject object = JSON.parseObject(fileStr, JSONObject.class);
                String dataStr = object.getString("data");
                List<String> array = JSON.parseArray(dataStr, String.class);
                for (String file : array) {
                    JSONObject fileObject = JSON.parseObject(file, JSONObject.class);
                    String filesStr = fileObject.getString("files");
                    List<JSONObject> jsonObjects = JSON.parseArray(filesStr, JSONObject.class);
                    if (CollUtil.isEmpty(jsonObjects)) {
                        // 如果没有上传附件,跳过
                        continue;
                    }
                    for (JSONObject json : jsonObjects) {
                        String response = json.getString("response");
                        JSONObject responseJson = JSON.parseObject(response, JSONObject.class);
                        String fileData = responseJson.getString("data");
                        JSONObject fileInfoObject = JSON.parseObject(fileData, JSONObject.class);
                        Long fileId = fileInfoObject.getLong("id");
                        fileIdList.add(fileId);
                    }
                }
            }
        }
    }

    private List<Long> getFileIdList(String buildBasis) {
        List<JSONObject> prePlanFileArray = JSON.parseArray(buildBasis, JSONObject.class);
        return CollUtils.fieldList(prePlanFileArray, w -> w.getLong(CommonConst.ID_STR));
    }

    /**
     * 查看 借阅的详情
     *
     * @param instanceId
     * @return
     */
    public ProjectApplyBorrowVO wantReadDetail(String instanceId) {
        ProjectApplyBorrow apply = projectApplyBorrowService.getOne(Wrappers.lambdaQuery(ProjectApplyBorrow.class)
                .eq(ProjectApplyBorrow::getInstanceId, instanceId)
                .last(BizConst.LIMIT_1));

        if (Objects.isNull(apply)) {
            return null;
        }

        ProjectApplyBorrowVO vo = BeanUtil.copyProperties(apply, ProjectApplyBorrowVO.class);
        if (StrUtils.isNotBlank(vo.getApplyBorrowUserId())) {
            UserFullInfoDTO user = userInfoHelper.getUserFullInfo(Long.valueOf(vo.getApplyBorrowUserId()));
            if (Objects.nonNull(user)) {
                vo.setApplyBorrowUserName(user.getUsername());
            }
        }
        if (StrUtils.isNotBlank(vo.getApplyBorrowOrgCode())) {
            UnitDTO mhUnit = mhUnitCache.getById(Long.parseLong(vo.getApplyBorrowOrgCode()));
            vo.setApplyBorrowOrgName(mhUnit.getName());
        }
        if (Objects.nonNull(vo.getProjectId())) {
            Project project = projectService.getById(vo.getProjectId());
            if (Objects.nonNull(project)) {
                vo.setProjectName(project.getProjectName());
            }
        }
        return vo;
    }

}
