package com.hz.pm.api.meeting.controller;

import com.hz.pm.api.external.sms.MhSmsClient;
import com.hz.pm.api.external.sms.dto.SmsDTO;
import com.hz.pm.api.external.sms.vo.SmsReceipt;
import com.hz.pm.api.external.sms.vo.SmsReplyDTO;
import com.hz.pm.api.external.sms.vo.SmsSendDTO;
import com.hz.pm.api.meeting.entity.req.MeetingCalenderReq;
import com.hz.pm.api.meeting.entity.vo.*;
import com.hz.pm.api.meeting.manage.DashboardManage;
import com.hz.pm.api.sms.constant.VoiceSmsTemplateConst;
import com.ningdatech.basic.model.PagePo;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * DashboardController
 * </p>
 *
 * @author WendyYang
 * @since 22:19 2022/8/24
 */
@Api(tags = "工作台相关接口")
@RestController
@AllArgsConstructor
@RequestMapping("/api/v1/dashboard")
public class ExpertDashboardController {

    private final DashboardManage dashboardManage;

    @ApiOperation("短信发送")
    @GetMapping("/sendSms")
    public SmsDTO<SmsSendDTO> test(String phone) {
        return MhSmsClient.getClient().smsSend(VoiceSmsTemplateConst.EXPERT_INVITE_SMS, Collections.singletonList(phone));
    }

    @ApiOperation("短信回复")
    @GetMapping("/aplySms")
    public SmsReplyDTO smsReply(String uuid) {
        return MhSmsClient.getClient().smsReply(uuid);
    }

    @ApiOperation("短信回执")
    @GetMapping("/status")
    public SmsReceipt smsReceipt(String uuid) {
        return MhSmsClient.getClient().smsReceipt(uuid);
    }

    @ApiOperation("会议日历")
    @GetMapping("/meetingCalender")
    @WebLog(value = "会议日历")
    public List<MeetingCalenderItemVO> meetingCalender(@Valid MeetingCalenderReq po) {
        return dashboardManage.meetingCalender(po);
    }

    @ApiOperation("待办：专家待替换列表")
    @GetMapping("/todo/expertReplace")
    public PageVo<ExpertReplaceTodoListItemVO> expertReplaceToDo(PagePo po) {
        return dashboardManage.expertReplaceTodoList(po);
    }

    @ApiOperation("待办：专家待确认列表")
    @GetMapping("/todo/expertConfirm")
    public PageVo<MeetingConfirmToDoListItemVO> expertConfirmToDo(PagePo po) {
        return dashboardManage.expertConfirmToDo(po);
    }

    @ApiOperation("会议数量统计")
    @GetMapping("/meetingCountSummary")
    @WebLog(value = "会议数量统计")
    public MeetingCountSummaryVO meetingCountSummary() {
        return dashboardManage.meetingCountSummary();
    }

    @ApiOperation("专家工作台：会议数量统计")
    @GetMapping("/meetingCountByExpert")
    @WebLog(value = "专家工作台：会议数量统计")
    public MeetingCountByExpertVO meetingCountByExpert() {
        return dashboardManage.meetingCountByExpert();
    }

}
