package com.hz.pm.api.performance.model.vo;

import com.alibaba.fastjson.annotation.JSONField;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * @Classname PerformanceAppraisalProjectVO
 * @Description 绩效评价对象
 * @Date 2023/6/19 14:02
 * @Author PoffyZhang
 */
@Data
@ApiModel(value = "绩效评价项目对象", description = "绩效评价项目对象")
public class PerformanceAppraisalProjectVO implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("项目ID")
    private Long projectId;

    @ApiModelProperty("项目编码")
    private String projectCode;

    @ApiModelProperty("项目名称")
    private String projectName;

    @ApiModelProperty("申报单位")
    private String buildOrgName;

    @ApiModelProperty("项目类型")
    private String projectType;

    @ApiModelProperty("项目类型名")
    private String projectTypeName;

    @ApiModelProperty("是否复评")
    private Boolean isReAppraisal;

    @ApiModelProperty("申报金额")
    private BigDecimal declareAmount;

    @ApiModelProperty("预算年度")
    private Integer projectYear;

    @ApiModelProperty("是否可自评")
    private Boolean canSelfAppraisal = Boolean.FALSE;

    @ApiModelProperty("是否可复评")
    private Boolean canReAppraisal = Boolean.FALSE;

    @ApiModelProperty("是否可核查")
    private Boolean canVerify = Boolean.FALSE;

    @ApiModelProperty("是否可填写整改意见")
    private Boolean canRectify = Boolean.FALSE;

    @ApiModelProperty("自评开始时间")
    private LocalDateTime selfAppraisalStart;

    @ApiModelProperty("自评结束时间")
    private LocalDateTime selfAppraisalEnd;

    @ApiModelProperty("自评总分")
    private BigDecimal selfAppraisalTotalScore;

    @ApiModelProperty("自评打分时间")
    @JSONField(format = "yyyy-MM-dd HH:mm")
    private LocalDateTime selfAppraisalScoreTime;

    @ApiModelProperty("复评总分")
    private BigDecimal reAppraisalTotalScore;

    @ApiModelProperty("复评打分时间")
    @JSONField(format = "yyyy-MM-dd HH:mm")
    private LocalDateTime reAppraisalScoreTime;

    @ApiModelProperty("核查总分")
    private BigDecimal verifyTotalScore;

    @ApiModelProperty("核查打分时间")
    @JSONField(format = "yyyy-MM-dd HH:mm")
    private LocalDateTime verifyScoreTime;

    @ApiModelProperty("项目是否需要整改")
    private Boolean isRectify;

    @ApiModelProperty("整改意见")
    private String rectifyOpinion;

    @ApiModelProperty("整改提交人")
    private UserFullInfoDTO rectifySubmitEmployee;

    @ApiModelProperty("整改提交时间")
    private LocalDateTime rectifySubmitTime;

    @ApiModelProperty("评价计划ID")
    private Long appraisalId;

    @ApiModelProperty("所在分组ID")
    private Long groupId;

    @ApiModelProperty("整改状态 1 整改情况待提交、2 整改核查不通过、3 整改待审核、4 整改审核通过")
    private Integer rectifyStatus;

    @ApiModelProperty("整改状态名称")
    private String rectifyStatusName;

    @ApiModelProperty("整改结果说明")
    private String rectifyResult;

    @ApiModelProperty("整改结果附件")
    private String rectifyResultAppendix;

    @ApiModelProperty("整改结果提交人员工code")
    private String rectifyResSubEmpCode;

    @ApiModelProperty("整改结果提交时间")
    private LocalDateTime rectifyResSubTime;

    @ApiModelProperty("整改审核结果 1 通过、2 不通过")
    private Integer rectifyAuditResult;

    @ApiModelProperty("整改审核意见")
    private String rectifyAuditOpinion;

    @ApiModelProperty("整改审核附件")
    private String rectifyAuditAppendix;

}
