package com.hz.pm.api.projectdeclared.controller;

import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.projectdeclared.manage.FinalAcceptanceManage;
import com.hz.pm.api.projectdeclared.model.req.SubmitFinallyInspectedReq;
import com.hz.pm.api.projectdeclared.model.vo.FinalAcceptProgressStatisticsVO;
import com.hz.pm.api.projectdeclared.model.vo.FinalAcceptanceVO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;


/**
 * @Classname FinalAcceptanceController
 * @Description 申报项目-终验
 * @Date 2023/6/2 09:29
 * @Author PoffyZhang
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/api/v1/declared/final-acc")
@Api(value = "FinalAcceptanceController", tags = "申报管理-终验")
@RequiredArgsConstructor
public class FinalAcceptanceController {

    private final FinalAcceptanceManage finalAcceptanceManage;

    @GetMapping("/progressStatistics")
    @ApiOperation(value = "项目终验进度统计", notes = "项目终验进度统计")
    public FinalAcceptProgressStatisticsVO progressStatistics(@ModelAttribute ProjectListReq req) {
        return finalAcceptanceManage.finalAcceptProgressStatistics(req);
    }

    @ApiOperation(value = "待终验申报的项目列表", notes = "待终验申报的项目列表")
    @GetMapping("/project-list")
    public PageVo<ProjectLibListItemVO> projectlist(@ModelAttribute ProjectListReq req) {
        return finalAcceptanceManage.projectLibList(req);
    }

    @GetMapping("/export")
    @ApiOperation("待终验申报的项目导出")
    @WebLog("待终验申报的项目导出")
    public void exportList(ProjectListReq req, HttpServletResponse response) {
        ExcelDownUtil.downXls(response, req, finalAcceptanceManage::exportList);
    }

    @ApiOperation(value = "待终验申报的项目列表", notes = "待终验申报的项目列表")
    @GetMapping("/detail/{projectId}")
    public FinalAcceptanceVO detailByProjectId(@PathVariable Long projectId) {
        return finalAcceptanceManage.detailByProjectId(projectId);
    }

    @ApiOperation(value = "终验申报", notes = "终验申报")
    @WebLog("终验申报")
    @PostMapping("/start-process")
    public String startTheProcess(@Validated @RequestBody SubmitFinallyInspectedReq req) {
        String instanceId = finalAcceptanceManage.startProcess(req);
        return "终验申报 【" + instanceId + "】 成功";
    }

    @ApiOperation(value = "终验申报重新提交", notes = "终验申报重新提交")
    @WebLog("终验申报重新提交")
    @PostMapping("/restart-process")
    public String restartTheProcess(@Validated @RequestBody SubmitFinallyInspectedReq req) {
        String instanceId = finalAcceptanceManage.startProcess(req);
        return "终验申报 【" + instanceId + "】 成功";
    }

}
