package com.hz.pm.api.projectdeclared.controller;

import com.hz.pm.api.meeting.entity.dto.ProjectReviewResultDTO;
import com.hz.pm.api.projectdeclared.manage.ProjectReviewManage;
import com.hz.pm.api.projectdeclared.model.req.ProjectReviewApplyReq;
import com.hz.pm.api.projectdeclared.model.vo.ReviewProgressStatisticsVO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * ProjectReviewController
 * </p>
 *
 * @author WendyYang
 * @since 10:52 2024/4/17
 */
@Api(tags = "项目评审控制器")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/projectReview")
public class ProjectReviewController {

    private final ProjectReviewManage projectReviewManage;

    @ApiOperation("提交项目评审申请")
    @PostMapping(value = {"/submit", "resubmit"})
    @WebLog("提交项目评审申请")
    public void projectReviewApply(@RequestBody ProjectReviewApplyReq req) {
        projectReviewManage.projectReviewApply(req);
    }

    @ApiOperation("项目评审详情")
    @GetMapping("/detail/{projectId}")
    public ProjectReviewResultDTO projectReviewDetail(@PathVariable Long projectId) {
        return projectReviewManage.projectReviewDetail(projectId);
    }

    @ApiOperation("项目评审列表")
    @GetMapping("/pageReviewProject")
    public PageVo<ProjectLibListItemVO> pageReviewProject(ProjectListReq req) {
        return projectReviewManage.pageReviewProject(req);
    }

    @GetMapping("/progressStatistics")
    @ApiOperation("评审项目进度统计")
    public ReviewProgressStatisticsVO progressStatistics(ProjectListReq req) {
        return projectReviewManage.reviewProjectProgressStatistics(req);
    }

}
