package com.hz.pm.api.projectdeclared.converter;

import com.google.common.collect.Lists;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeNewEnum;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.Objects;

/**
 * @Classname ApplicationConverter
 * @Description
 * @Date 2023/3/13 17:38
 * @Author PoffyZhang
 */
@Slf4j
public class ApplicationConverter {

    private ApplicationConverter() {

    }

    private static Integer convertProjectType(String projectType) {
        if (StringUtils.isBlank(projectType)) {
            return null;
        }

        if (Lists.newArrayList(ProjectTypeNewEnum.FIRST_BUILD.getCode(),
                ProjectTypeNewEnum.SJ_BUILD.getCode(), ProjectTypeNewEnum.SZ_BUILD.getCode()).contains(projectType)) {
            return ProjectTypeEnum.RECONSTRUCTION.getCode();
        } else if (Lists.newArrayList(ProjectTypeNewEnum.NEW_OPERA.getCode(),
                ProjectTypeNewEnum.XU_OPERA.getCode()).contains(projectType)) {
            return ProjectTypeEnum.COMPREHENSIVE_INTEGRATION.getCode();
        }
        return null;
    }

    private static String checkAddBudget(BigDecimal declareHaveAmount, BigDecimal declareGovOwnFinanceAmount,
                                         BigDecimal declareGovSuperiorFinanceAmount, BigDecimal declareBankLendingAmount,
                                         BigDecimal otherAmount) {
        StringBuilder sb = new StringBuilder();

        if (Objects.nonNull(declareHaveAmount) && declareHaveAmount.compareTo(BigDecimal.ZERO) > 0) {
            sb.append("自有资金,");
        }
        if (Objects.nonNull(declareGovOwnFinanceAmount) && declareGovOwnFinanceAmount.compareTo(BigDecimal.ZERO) > 0) {
            sb.append("政府投资-本级财政资金,");
        }
        if (Objects.nonNull(declareGovSuperiorFinanceAmount) && declareGovSuperiorFinanceAmount.compareTo(BigDecimal.ZERO) > 0) {
            sb.append("政府投资-上级财政资金,");
        }
        if (Objects.nonNull(declareBankLendingAmount) && declareBankLendingAmount.compareTo(BigDecimal.ZERO) > 0) {
            sb.append("银行贷款,");
        }
        if (Objects.nonNull(otherAmount) && otherAmount.compareTo(BigDecimal.ZERO) > 0) {
            sb.append("其他资金");
        }
        return sb.toString();
    }


    /**
     * 去除掉 oss链接的超时时间
     *
     * @param url
     * @return
     */
    private static String removeExpire(String url) {
        if (StringUtils.isNotBlank(url)) {
            String s = "Expires";
            return url.replaceAll("&?" + s + "=[^&]*&", StringUtils.EMPTY);
        }
        return StringUtils.EMPTY;
    }

}
