package com.hz.pm.api.projectdeclared.helper;

import cn.hutool.core.io.FileUtil;
import cn.hutool.json.JSONUtil;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.external.MhFileClient;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.ApiResponse;
import com.ningdatech.file.controller.FileController;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;

/**
 * @Classname MhXchxFileHelper
 * @Description
 * @Date 2024/7/3 10:41
 * @Author PoffyZhang
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class MhXchxFileHelper {

    private final MhFileClient mhFileClient;

    private final FileController fileController;

    public String getXchxFile(String reportFile,String fileName) {
        if (StrUtils.isBlank(reportFile)) {
            throw BizException.wrap("信创报告文件不能为空");
        }
        ApiResponse<FileResultVO> retFileInfo;
        File tmpFile = null;
        FileInputStream fis = null;
        try {
            tmpFile = mhFileClient.downloadToTmpFile(reportFile);
            if(StringUtils.isNotBlank(fileName)){
                fileName = fileName + FileUtil.getSuffix(tmpFile);
            }else{
                fileName = tmpFile.getName();
            }
            fis = new FileInputStream(tmpFile);
            String mimeType = FileUtil.getMimeType(tmpFile.getPath());
            MockMultipartFile multipartFile = new MockMultipartFile(fileName, fileName, mimeType, fis);
            retFileInfo = fileController.upload(multipartFile, "default");
        } catch (IOException e) {
            log.error("信创报告上传失败", e);
            throw BizException.wrap("信创符合性测评报告上传失败");
        } finally {
            if (tmpFile != null) {
                tmpFile.deleteOnExit();
            }
            if (fis != null) {
                IOUtils.closeQuietly(fis);
            }
        }
        return JSONUtil.toJsonStr(retFileInfo);
    }

    public void exportReport(String reportFile, HttpServletResponse response) throws UnsupportedEncodingException {
        if (StrUtils.isBlank(reportFile)) {
            throw BizException.wrap("信创报告文件不能为空");
        }

        File file = mhFileClient.downloadToTmpFile(reportFile);
        // 设置响应的内容类型和头信息
        response.setContentType("application/pdf");
        response.setHeader("Content-Disposition", "attachment;filename=\"" + URLEncoder.encode(file.getName(),"utf-8") + "\"");

        // 使用ServletOutputStream写文件内容
        try (InputStream in = Files.newInputStream(file.toPath());
             ServletOutputStream out = response.getOutputStream()) {
            byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = in.read(buffer)) != -1) {
                out.write(buffer, 0, bytesRead);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }
}
