package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.common.statemachine.util.TenderStateMachineUtil;
import com.hz.pm.api.common.util.BizDateUtil;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.common.util.ExcelExportStyle;
import com.hz.pm.api.projectdeclared.model.dto.DeclaredProjectExportDTO;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectdeclared.model.req.SubmitFinallyInspectedReq;
import com.hz.pm.api.projectdeclared.model.vo.FinalAcceptProgressStatisticsVO;
import com.hz.pm.api.projectdeclared.model.vo.FinalAcceptanceVO;
import com.hz.pm.api.projectdeclared.service.IContractService;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeNewEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.TenderListInfoVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.BiFunction;
import java.util.stream.Collectors;

import static com.hz.pm.api.user.helper.MhUnitQueryAuthHelper.UnitQueryState;

/**
 * @Classname FinalAcceptanceManage
 * @Description 终验
 * @Date 2023/6/2 14:48
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class FinalAcceptanceManage {

    private final IProjectService projectService;
    private final UserInfoHelper userInfoHelper;
    private final IPurchaseService purchaseService;
    private final IPurchaseInstService purchaseInstService;
    private final ProcessModelManage processModelManage;
    private final ProcessInstanceService processInstanceService;
    private final DeclaredProjectHelper declaredProjectManage;
    private final IContractService contractService;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final TenderStateMachineUtil tenderStateMachineUtil;
    private final IProjectInstService projectInstService;
    private final MhUnitQueryAuthHelper mhUnitQueryAuthHelper;

    //==================================================================================================================

    private static final List<Integer> FINAL_INSPECTED_TENDER_STATUS;

    static {
        FINAL_INSPECTED_TENDER_STATUS = new ArrayList<>();
        FINAL_INSPECTED_TENDER_STATUS.add(TenderMainStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY.getCode());
        FINAL_INSPECTED_TENDER_STATUS.add(TenderMainStatus.ON_FINALLY_INSPECTED_APPLY.getCode());
        FINAL_INSPECTED_TENDER_STATUS.add(TenderMainStatus.FINALLY_INSPECTED_PASSED.getCode());
        FINAL_INSPECTED_TENDER_STATUS.add(TenderMainStatus.FINALLY_INSPECTED_FAILED.getCode());
    }

    /**
     * 待采购的-项目列表
     *
     * @param req \
     * @return \
     */
    public FinalAcceptProgressStatisticsVO finalAcceptProgressStatistics(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long buildOrgCode = req.clearBuildOrgCode();
        UnitQueryState queryState = mhUnitQueryAuthHelper.listCanViewUnitIds(buildOrgCode, user);
        if (!queryState.isState()) {
            return null;
        }
        LambdaQueryWrapper<Purchase> purchaseQuery = Wrappers.lambdaQuery(Purchase.class)
                .select(Purchase::getProjectId, Purchase::getStatus)
                .in(Purchase::getStatus, FINAL_INSPECTED_TENDER_STATUS);
        List<Purchase> purchases = purchaseService.list(purchaseQuery);
        if (purchases.isEmpty()) {
            return null;
        }
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        if (CollUtil.isNotEmpty(queryState.getUnitIds())) {
            query.in(Project::getBuildOrgCode, CollUtils.convert(queryState.getUnitIds(), String::valueOf));
        }
        query.in(Project::getId, CollUtils.fieldList(purchases, Purchase::getProjectId))
                .select(Project::getId)
                .eq(Project::getNewest, Boolean.TRUE);
        List<Project> projects = projectService.list(query);
        List<Long> projectIds = CollUtils.fieldList(projects, Project::getId);
        Map<Long, List<Integer>> purchaseStatusMap = purchases.stream()
                .filter(purchase -> projectIds.contains(purchase.getProjectId()))
                .collect(Collectors.groupingBy(Purchase::getProjectId,
                        Collectors.mapping(Purchase::getStatus, Collectors.toList())));
        BiFunction<Map<Long, List<Integer>>, TenderMainStatus, Integer> counter = (statusMap, status) -> {
            Set<Map.Entry<Long, List<Integer>>> entries = statusMap.entrySet();
            return CollUtil.count(entries, entry -> entry.getValue().contains(status.getCode()));
        };
        return FinalAcceptProgressStatisticsVO.builder()
                .totalCount(projects.size())
                .auditCount(counter.apply(purchaseStatusMap, TenderMainStatus.ON_FINALLY_INSPECTED_APPLY))
                .todoCount(counter.apply(purchaseStatusMap, TenderMainStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY))
                .failedCount(counter.apply(purchaseStatusMap, TenderMainStatus.FINALLY_INSPECTED_FAILED))
                .passedCount(counter.apply(purchaseStatusMap, TenderMainStatus.FINALLY_INSPECTED_PASSED))
                .build();
    }

    /**
     * 待采购的-项目列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> projectLibList(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long buildOrgCode = req.clearBuildOrgCode();
        UnitQueryState queryState = mhUnitQueryAuthHelper.listCanViewUnitIds(buildOrgCode, user);
        if (!queryState.isState()) {
            return null;
        }
        LambdaQueryWrapper<Purchase> purchaseQuery = Wrappers.lambdaQuery(Purchase.class)
                .select(Purchase::getStatus, Purchase::getProjectId, Purchase::getId, Purchase::getBidName);
        if (req.getTabStatus() == null) {
            purchaseQuery.in(Purchase::getStatus, FINAL_INSPECTED_TENDER_STATUS);
        } else {
            purchaseQuery.eq(Purchase::getStatus, req.getTabStatus());
        }
        List<Purchase> purchases = purchaseService.list(purchaseQuery);
        if (purchases.isEmpty()) {
            return PageVo.empty();
        }
        List<Long> projectIds = CollUtils.fieldList(purchases, Purchase::getProjectId);
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req)
                .in(Project::getId, projectIds)
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByAsc(Project::getApprovalDate);
        if (CollUtil.isNotEmpty(queryState.getUnitIds())) {
            query.in(Project::getBuildOrgCode, CollUtils.convert(queryState.getUnitIds(), String::valueOf));
        }
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        Map<Long, List<Purchase>> purchaseMap = CollUtils.group(purchases, Purchase::getProjectId);
        Map<Long, Contract> contractMap = contractService.listByProjectIds(projectIds);
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            item.setReviewAmount(w.getReviewAmount());
            item.setApprovedAmount(w.getApprovalAmount());
            item.setApprovalDate(w.getApprovalDate());
            item.setBuildCycle(NumberUtil.parseInt(w.getBuildCycle(), null));
            item.setPlanAcceptanceTime(w.getPlanAcceptanceTime());
            BizUtils.notNull(purchaseMap.get(w.getId()), tmp -> {
                List<TenderListInfoVO> tenders = CollUtils.convert(tmp, x -> {
                    TenderListInfoVO tender = new TenderListInfoVO();
                    tender.setBidName(x.getBidName());
                    tender.setBidId(x.getId());
                    tender.setBidStatus(x.getStatus());
                    tender.setBidStatusName(TenderMainStatus.getDesc(x.getStatus()));
                    Contract contract = contractMap.get(x.getId());
                    if (contract != null) {
                        tender.setConstructionAmount(contract.getTotalAmount());
                    }
                    return tender;
                });
                item.setTenders(tenders);
            });
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    public void exportList(HttpServletResponse response, ProjectListReq param) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        //待终验
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitId());
        query.eq(Project::getStatus, ProjectStatus.ON_PILOT_RUNNING.getCode());
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        query.isNotNull(Project::getApprovalDate);
        query.isNotNull(Project::getBuildCycle);
        query.orderByAsc(Project::getApprovalDate);
        List<Project> records = projectService.list(query);

        AtomicInteger serialNumber = new AtomicInteger(0);
        List<DeclaredProjectExportDTO> collect = Lists.newArrayList();
        if (CollUtil.isNotEmpty(records)) {
            records.forEach(r -> {
                DeclaredProjectExportDTO item = new DeclaredProjectExportDTO();
                BeanUtils.copyProperties(r, item);
                item.setProjectTypeName(ProjectTypeNewEnum.getDesc(r.getProjectType()));
                item.setStatusName(ProjectStatus.getDesc(r.getStatus()));
                String createOnStr = NdDateUtils.format(r.getCreateOn(), "yyyy-MM-dd HH:mm");
                item.setCreateOn(createOnStr);
                item.setSerialNumber(serialNumber.incrementAndGet());
                item.setPlanAcceptanceTime(BizDateUtil.toDateStr(r.getPlanAcceptanceTime()));
                collect.add(item);
            });
        }

        String fileName = "待终验申请项目列表";
        ExcelDownUtil.setFileName(fileName, response);
        //数据导出处理函数
        try {
            EasyExcel.write(response.getOutputStream(), DeclaredProjectExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(collect);
        } catch (Exception e) {
            throw new BizException(e);
        }
    }


    public FinalAcceptanceVO detailByProjectId(Long projectId) {
        FinalAcceptanceVO vo = new FinalAcceptanceVO();
        Project project = projectService.getById(projectId);
        vo.setProjectId(projectId);
        if (Objects.nonNull(project)) {
            vo.setFinalAcceptanceMaterials(project.getFinalAcceptanceMaterials());
            vo.setApps(Collections.emptyList());
        }
        return vo;
    }

    public FinalAcceptanceVO detailByProject(Project project) {
        FinalAcceptanceVO vo = new FinalAcceptanceVO();
        vo.setProjectId(project.getId());
        vo.setFinalAcceptanceMaterials(project.getFinalAcceptanceMaterials());
        vo.setApps(Collections.emptyList());
        return vo;
    }

    /**
     * 开启流程
     *
     * @param req \
     * @return \
     */
    @Transactional(rollbackFor = Exception.class)
    public synchronized String startProcess(SubmitFinallyInspectedReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectProcessType instType = ProjectProcessType.ACCEPTANCE_DECLARATION_APPROVAL_PROCESS;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());

        List<Purchase> purchases = purchaseService.listByProjectId(req.getProjectId());
        Map<Long, Purchase> purchaseMap = CollUtils.listToMap(purchases, Purchase::getId);
        Purchase purchase = purchaseMap.get(req.getBidId());
        Assert.notNull(purchase, "当前标段不存在");
        if (!TenderMainStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY.eq(purchase.getStatus())
                && !TenderMainStatus.FINALLY_INSPECTED_FAILED.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段不支持终验申报");
        }
        Project project = projectService.getNewProject(req.getProjectId());
        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectManage.buildUser(user));
        params.setProcessUsers(Collections.emptyMap());
        params.setFormData(Collections.emptyMap());
        // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
        Map<String, OrgInfoDTO> orgMap = declaredProjectManage.buildOrgModelMap(user, project);
        String instanceId = processInstanceService.startProcessLs(model, params, orgMap);

        log.info("终验方案项目申报成功 【{}】", instanceId);
        tenderStateMachineUtil.pass(purchase);
        purchase.setFinalInspectionDate(req.getInspectionDate());
        purchase.setFinalInspectionMaterials(req.getInspectionMaterials());
        purchase.setHasSecurityEval(req.getHasSecurityEval());
        purchase.setSecurityEvalDate(req.getSecurityEvalDate());
        purchase.setSecurityEvalMaterials(req.getSecurityEvalMaterials());
        purchase.setXcfhxReportFiles(req.getXcfhxReportFiles());
        purchase.setMatchXcfhx(req.getMatchXcfhx());
        purchase.setXcfhxEvalFinishDate(req.getXcfhxEvalFinishDate());
        purchaseService.updateById(purchase);
        PurchaseInst purchaseInst = new PurchaseInst();
        purchaseInst.setBidId(req.getBidId());
        purchaseInst.setInstCode(instanceId);
        purchaseInst.setProjectId(req.getProjectId());
        purchaseInst.setInstType(instType);
        purchaseInstService.save(purchaseInst);
        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstType(instType);
        projectInst.setInstCode(instanceId);
        projectInst.setProjectId(req.getProjectId());
        projectInstService.save(projectInst);

        if (ProjectStatus.ON_PILOT_RUNNING.eq(project.getStatus())) {
            // 判断是否所有标段已经进行过终验申报 无论是否通过
            LambdaQueryWrapper<PurchaseInst> purchaseInstQuery = Wrappers.lambdaQuery(PurchaseInst.class)
                    .select(PurchaseInst::getBidId)
                    .eq(PurchaseInst::getInstType, ProjectProcessType.ACCEPTANCE_DECLARATION_APPROVAL_PROCESS.getCode())
                    .in(PurchaseInst::getBidId, purchaseMap.keySet());
            List<PurchaseInst> currPurchaseInstList = purchaseInstService.list(purchaseInstQuery);
            Set<Long> finalInspectedBidIds = CollUtils.fieldSet(currPurchaseInstList, PurchaseInst::getBidId);
            int needFinalAcceptCount = CollUtil.count(purchases, w -> BidTypeEnum.BUILD_APP.eq(w.getBidType()));
            if (finalInspectedBidIds.size() == needFinalAcceptCount) {
                projectStateMachineUtil.pass(project);
                projectService.updateById(project);
            }
        }
        //发送给第一个审批人消息
        return instanceId;
    }

}
