package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.statemachine.util.TenderStateMachineUtil;
import com.hz.pm.api.projectdeclared.model.dto.OperationReq;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.vo.OperationVO;
import com.hz.pm.api.projectdeclared.service.IOperationService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * @Classname OperationManage
 * @Description
 * @Date 2023/7/31 14:48
 * @Author PoffyZhang
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class OperationManage {

    private final IOperationService operationService;
    private final IPurchaseService purchaseService;
    private final IProjectService projectService;
    private final TenderStateMachineUtil tenderStateMachineUtil;

    /**
     * 获取实施详情
     *
     * @param projectId \
     * @return \
     */
    public List<OperationVO> detail(Long projectId) {
        Project project = projectService.getNewestNoNull(projectId);
        List<Operation> operations = operationService.list(Wrappers.lambdaQuery(Operation.class)
                .eq(Operation::getProjectCode, project.getProjectCode()));

        return CollUtils.convert(operations, w -> BeanUtil.copyProperties(w, OperationVO.class));
    }

    /**
     * 获取标段实施详情
     */
    public OperationVO detailByBidId(Long bidId) {
        LambdaQueryWrapper<Operation> query = Wrappers.lambdaQuery(Operation.class)
                .eq(Operation::getBidId, bidId);
        Operation operation = operationService.getOne(query);
        return BeanUtil.copyProperties(operation, OperationVO.class);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized String pushOperation(OperationReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        List<Purchase> purchases = purchaseService.listByProjectId(req.getProjectId());
        Map<Long, Purchase> purchaseMap = CollUtils.listToMap(purchases, Purchase::getId);
        Purchase purchase = purchaseMap.get(req.getBidId());
        if (!TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段不支持填写实施计划");
        }
        Operation operation = operationService.getByBidId(req.getBidId())
                .orElseGet(Operation::new);
        Project project = projectService.getNewProject(req.getProjectId());
        LocalDateTime now = LocalDateTime.now();
        BeanUtil.copyProperties(req, operation);
        if (operation.getId() == null) {
            operation.setCreateOn(now);
            operation.setCreateBy(user.getUsername());
        }
        operation.setProjectCode(project.getProjectCode());
        operation.setUpdateOn(now);
        operation.setUpdateBy(user.getUsername());
        operationService.saveOrUpdate(operation);
        // 修改标段状态
        tenderStateMachineUtil.pass(purchase);
        purchaseService.updateById(purchase);
        return operation.getProjectCode();
    }
}
