package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.*;
import com.hz.pm.api.common.util.*;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.MhFileClient;
import com.hz.pm.api.external.model.dto.MhPurchaseIntentionDTO;
import com.hz.pm.api.external.model.dto.MhPurchaseNoticeDTO;
import com.hz.pm.api.external.model.enumeration.MhUnitStripEnum;
import com.hz.pm.api.projectdeclared.helper.MhXchxFileHelper;
import com.hz.pm.api.projectdeclared.model.dto.DeclaredProjectExportDTO;
import com.hz.pm.api.projectdeclared.model.dto.PurchaseSaveDTO;
import com.hz.pm.api.projectdeclared.model.dto.XcfhxApplyEditDTO;
import com.hz.pm.api.projectdeclared.model.dto.XcfhxApplySaveDTO;
import com.hz.pm.api.projectdeclared.model.entity.*;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectdeclared.model.enumerization.PurchaseScopeEnum;
import com.hz.pm.api.projectdeclared.model.enumerization.PurchaseSourceTypeEnum;
import com.hz.pm.api.projectdeclared.model.req.*;
import com.hz.pm.api.projectdeclared.model.req.SubmitPurchaseNoticeReq.PurchaseNoticeTenderDTO;
import com.hz.pm.api.projectdeclared.model.vo.*;
import com.hz.pm.api.projectdeclared.service.*;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeNewEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.*;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.TenderListInfoVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper.UnitQueryState;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import com.ningdatech.file.service.FileService;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * PurchaseManage
 * </p>
 *
 * @author WendyYang
 * @since 11:15 2024/4/2
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class PurchaseManage {

    private final MhUnitCache mhUnitCache;
    private final IProjectService projectService;
    private final UserInfoHelper userInfoHelper;
    private final IContractService contractService;
    private final IPurchaseService purchaseService;
    private final TenderStateMachineUtil tenderStateMachineUtil;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final MhApiClient mhApiClient;
    private final MhFileClient mhFileClient;
    private final FileService fileService;
    private final IOperationService operationService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectManage;
    private final ProcessInstanceService processInstanceService;
    private final TestValidStateMachineUtil testValidStateMachineUtil;
    private final SelfTestStateMachineUtil selfTestStateMachineUtil;
    private final AdaptStateMachineUtil adaptStateMachineUtil;
    private final IPurchaseInstService purchaseInstService;
    private final IProjectInstService projectInstService;
    private final EnvironmentUtil environmentUtil;
    private final IXinchuangService xinchuangService;
    private final MhXchxFileHelper mhXchxFileHelper;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;
    private final MhUnitQueryAuthHelper mhUnitQueryAuthHelper;

    public PurchaseProgressStatVO purchaseProgressStatistics(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long buildOrgCode = req.clearBuildOrgCode();
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        //数据权限
        if (!permission(query, user, buildOrgCode)) {
            return null;
        }
        //待采购状态
        query.notIn(Project::getStage, ProjectStatus.STOPPED.getCode(), ProjectStatus.CHANGE.getCode())
                .eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode())
                .eq(Project::getNewest, Boolean.TRUE)
                .exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.DECLARED_RECORD_PASS)
                .select(Project::getId);
        List<Project> projects = projectService.list(query);
        if (projects.isEmpty()) {
            return null;
        }
        List<Long> projectIds = CollUtils.fieldList(projects, Project::getId);
        List<Purchase> purchases = purchaseService.list(Wrappers.lambdaQuery(Purchase.class)
                .in(Purchase::getProjectId, projectIds)
                .select(Purchase::getProjectId, Purchase::getStatus));
        Map<Long, Collection<Purchase>> projectPurchaseMap = CollUtils.listToMap(projects, Project::getId,
                w -> CollUtil.filterNew(purchases, x -> x.getProjectId().equals(w.getId())));
        PurchaseProgressStatVO stat = new PurchaseProgressStatVO();
        stat.setOnPurchaseCount(CollUtil.count(projectPurchaseMap.values(),
                w -> CollUtil.anyMatch(w, x -> TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.eq(x.getStatus()))));
        stat.setToBePurchaseCount(CollUtil.count(projectPurchaseMap.values(), Collection::isEmpty));
        stat.setPurchaseFinishedCount(CollUtil.count(projectPurchaseMap.values(),
                w -> !w.isEmpty() && CollUtil.allMatch(w, x -> !TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.eq(x.getStatus()))));
        stat.setTotalCount(projects.size());
        return stat;
    }

    /**
     * 待采购的-项目列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> projectLibList(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Assert.notNull(user, "获取登录用户失败!");
        Long buildOrgCode = req.clearBuildOrgCode();
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        //数据权限
        if (!permission(query, user, buildOrgCode)) {
            return PageVo.empty();
        }
        //待采购状态
        query.notIn(Project::getStage, ProjectStatus.STOPPED.getCode(), ProjectStatus.CHANGE.getCode());
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByAsc(Project::getApprovalDate);
        ProjectManageUtil.projectBaseQuery(query);
        if (req.getTabStatus() != null) {
            if (TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.eq(req.getTabStatus())) {
                query.exists(ExistsSqlConst.PROJECT_EXISTS_PURCHASE +
                        " and np.status = {0} ", req.getTabStatus());
            } else if (TenderMainStatus.PURCHASE_FINISHED.eq(req.getTabStatus())) {
                query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0} ", ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
            } else if (ProjectStatus.TO_BE_PURCHASED.eq(req.getTabStatus())) {
                query.eq(Project::getStatus, ProjectStatus.TO_BE_PURCHASED.getCode());
            } else {
                return PageVo.empty();
            }
        } else {
            query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                    " and npsc.event = {0}", ProjectStateChangeEvent.DECLARED_RECORD_PASS);
        }
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        Map<Long, List<Purchase>> purchaseMap;
        Map<Long, BigDecimal> contractAmountMap;
        if (!ProjectStatus.TO_BE_PURCHASED.eq(req.getTabStatus())) {
            List<Long> projectIds = CollUtils.fieldList(page.getRecords(), Project::getId);
            LambdaQueryWrapper<Purchase> purchaseQuery = Wrappers.lambdaQuery(Purchase.class)
                    .in(Purchase::getProjectId, projectIds);
            if (req.getTabStatus() != null) {
                if (TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.eq(req.getTabStatus())) {
                    purchaseQuery.eq(Purchase::getStatus, req.getTabStatus());
                } else if (TenderMainStatus.PURCHASE_FINISHED.eq(req.getTabStatus())) {
                    purchaseQuery.exists(ExistsSqlConst.PURCHASE_EXISTS_STATUS_CHANGE +
                            " and npsc.event = {0} ", TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO);
                }
            }
            List<Purchase> purchases = purchaseService.list(purchaseQuery);
            purchaseMap = CollUtils.group(purchases, Purchase::getProjectId);
            if (purchaseMap.isEmpty()) {
                contractAmountMap = Collections.emptyMap();
            } else {
                contractAmountMap = contractService.listContractAmountByProjectIds(projectIds);
            }
        } else {
            purchaseMap = Collections.emptyMap();
            contractAmountMap = Collections.emptyMap();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setApprovedAmount(w.getApprovalAmount());
            item.setReviewAmount(w.getReviewAmount());
            item.setApprovalDate(w.getApprovalDate());
            List<Purchase> currPurchases = purchaseMap.get(w.getId());
            if (currPurchases != null) {
                List<TenderListInfoVO> tenders = new ArrayList<>();
                for (Purchase x : currPurchases) {
                    TenderMainStatus tenderMainStatus;
                    if (!TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.eq(x.getStatus())) {
                        tenderMainStatus = TenderMainStatus.PURCHASE_FINISHED;
                    } else {
                        tenderMainStatus = TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO;
                    }
                    TenderListInfoVO tender = new TenderListInfoVO();
                    tender.setBidName(x.getBidName());
                    tender.setBidId(x.getId());
                    tender.setBidStatus(tenderMainStatus.getCode());
                    tender.setBidStatusName(tenderMainStatus.getDesc());
                    tender.setConstructionAmount(contractAmountMap.get(x.getId()));
                    tenders.add(tender);
                    item.setTenders(tenders);
                }
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    /**
     * 根据角色 分配权限
     *
     * @param query
     * @param user
     */
    private boolean permission(LambdaQueryWrapper<Project> query, UserInfoDetails user, Long buildOrgCode) {
        UnitQueryState state = mhUnitQueryAuthHelper.listCanViewUnitIds(buildOrgCode, user);
        if (!state.isState()) {
            return false;
        }
        if (CollUtil.isNotEmpty(state.getUnitIds())) {
            query.in(Project::getBuildOrgCode, CollUtils.convert(state.getUnitIds(), String::valueOf));
        }
        return true;
    }

    public void exportList(HttpServletResponse response, ProjectListReq param) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        VUtils.isTrue(Objects.isNull(user)).throwMessage("获取登录用户失败!");
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        //待采购状态
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitId());
        query.eq(Project::getStatus, ProjectStatus.TO_BE_PURCHASED.getCode());
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByAsc(Project::getApprovalDate);
        List<Project> records = projectService.list(query);
        Assert.notEmpty(records, "暂无数据可导出");
        List<DeclaredProjectExportDTO> data = Lists.newArrayList();

        AtomicInteger serialNumber = new AtomicInteger(0);
        records.forEach(r -> {
            DeclaredProjectExportDTO dpe = new DeclaredProjectExportDTO();
            BeanUtils.copyProperties(r, dpe);
            dpe.setProjectTypeName(ProjectTypeNewEnum.getDesc(r.getProjectType()));
            dpe.setStatusName(ProjectStatus.getDesc(r.getStatus()));
            String createOnStr = NdDateUtils.format(r.getCreateOn(), "yyyy-MM-dd HH:mm");
            dpe.setCreateOn(createOnStr);
            dpe.setSerialNumber(serialNumber.incrementAndGet());
            dpe.setApprovedAmount(r.getApprovalAmount());
            dpe.setApprovalDate(r.getApprovalDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            data.add(dpe);
        });

        String fileName = "采购结果待备案项目列表";
        ExcelDownUtil.setFileName(fileName, response);
        //数据导出处理函数
        try {
            EasyExcel.write(response.getOutputStream(), DeclaredProjectExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(data);
        } catch (IOException e) {
            log.error("导出异常", e);
            throw BizException.wrap("导出失败，请重试");
        }
    }

    /**
     * 获取采购结果备案详情
     *
     * @param projectId
     * @return
     */
    public List<PurchaseVO> detailByProjectId(Long projectId) {
        List<Purchase> purchases = purchaseService.listByProjectId(projectId);
        return BeanUtil.copyToList(purchases, PurchaseVO.class);
    }

    /**
     * 填写采购结果
     *
     * @param dto
     * @return
     */
    public String submitResult(PurchaseSaveDTO dto) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        Long projectId = dto.getProjectId();
        VUtils.isTrue(Objects.isNull(projectId)).throwMessage("提交失败 缺少项目ID!");
        Project project = projectService.getNewestNoNull(dto.getProjectId());

        //首先要判断 项目当前状态 是不是 采购结果备案
        VUtils.isTrue(!ProjectStatus.TO_BE_PURCHASED.eq(project.getStatus())
                        || !ProjectStatus.PROJECT_APPROVED.eq(project.getStage()))
                .throwMessage("提交失败 该项目不是 待采购备案状态或者已立项阶段");

        List<Purchase> purchases = purchaseService.listByProjectId(projectId);
        Map<Long, Purchase> purchaseMap = CollUtils.listToMap(purchases, Purchase::getId);
        List<Purchase> purchaseEntities = CollUtils.convert(dto.getTenders(), w -> {
            Purchase purchase = BeanUtil.copyProperties(w, Purchase.class);
            purchase.setProjectId(projectId);
            if (!purchaseMap.containsKey(w.getId())) {
                purchase.setCreateBy(user.getUserIdStr());
            }
            return purchase;
        });

        if (purchaseService.saveOrUpdateBatch(purchaseEntities)) {
            projectStateMachineUtil.pass(project);
            project.setUpdateOn(LocalDateTime.now());

            // 获取总的成交时间及金额
            LocalDateTime lastTransactionTime = null;
            BigDecimal transactionAmountTotal = BigDecimal.ZERO;
            for (Purchase purchase : purchaseEntities) {
                if (lastTransactionTime == null || purchase.getTransactionTime().isAfter(lastTransactionTime)) {
                    lastTransactionTime = purchase.getTransactionTime();
                }
                transactionAmountTotal = transactionAmountTotal.add(purchase.getTransactionAmount());
            }

            project.setTransactionAmount(transactionAmountTotal);
            project.setTransactionTime(lastTransactionTime);
            projectService.updateById(project);

            return "填写成功";
        }

        return "保存失败";
    }

    @Value("${mh.purchase-notice.open:true}")
    private Boolean mhPurchaseNoticeOpen;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitPurchaseNotice(SubmitPurchaseNoticeReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long projectId = req.getProjectId();
        Project project = projectService.getNewestNoNull(projectId);

        //首先要判断 项目当前状态 是不是 采购结果备案
        VUtils.isTrue(!ProjectStatus.TO_BE_PURCHASED.eq(project.getStatus())
                        || !ProjectStatus.PROJECT_APPROVED.eq(project.getStage()))
                .throwMessage("提交失败,该项目不是待采购备案状态或者已立项阶段");
        List<MhPurchaseNoticeDTO> notices = new ArrayList<>();
        List<Purchase> purchases = req.getTenders().stream().map(w -> {
            Purchase purchase = new Purchase();
            purchase.setPurchaseMethod(w.getPurchaseMethod());
            purchase.setBidAddress(w.getBidAddress());
            purchase.setBidName(w.getBidName());
            purchase.setBidBudget(w.getBidBudget());
            purchase.setBidDoc(w.getBidDoc());
            purchase.setBidStartTime(w.getBidStartTime());
            purchase.setBidEndTime(w.getBidEndTime());
            purchase.setId(w.getBidId());
            purchase.setProjectId(projectId);
            purchase.setPurchaseConnectAddress(w.getPurchaseConnectAddress());
            purchase.setPurchasePhone(w.getPurchasePhone());
            purchase.setPurchaseRemark(w.getPurchaseRemark());
            purchase.setPurchaseScope(w.getPurchaseScope());
            purchase.setPurchaseUnit(w.getPurchaseUnit());
            purchase.setPurchaseUnitId(w.getPurchaseUnitId());
            purchase.setPurchaseContact(w.getPurchaseContact());
            purchase.setBidType(w.getBidType());
            purchase.setPurchaseDate(w.getPurchaseDate());
            purchase.setBidPayAmount(w.getBidPayAmount());
            purchase.setPublishMhNotice(w.getPublishMhNotice());
            if (purchase.getId() == null) {
                purchase.setCreateBy(user.getUserIdStr());
            }
            purchase.setStatus(TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.getCode());
            purchase.setXcfhxApplyStatus(TenderXcfhxApplyStatus.TO_BE_SUBMIT_XCFHX_APPLY.getCode());
            if (Boolean.TRUE.equals(mhPurchaseNoticeOpen) && Boolean.TRUE.equals(w.getPublishMhNotice())) {
                notices.add(buildPurchaseNotice(w, user));
            }
            return purchase;
        }).collect(Collectors.toList());
        purchaseService.saveOrUpdateBatch(purchases);
        projectStateMachineUtil.pass(project);
        projectService.updateById(project);
        // 发布采购公告
        if (!notices.isEmpty()) {
            mhApiClient.publishPurchaseNotice(notices);
        }
    }

    private MhPurchaseNoticeDTO buildPurchaseNotice(PurchaseNoticeTenderDTO w, UserInfoDetails user) {
        if (environmentUtil.isDevEnv()) {
            return null;
        }
        Date now = new Date();
        List<FileResultVO> bidFiles = JSONUtil.toList(w.getBidDoc(), FileResultVO.class);
        List<File> files = fileService.listByIds(CollUtils.fieldSet(bidFiles, FileResultVO::getId));
        String fileIds = files.stream().map(file -> {
            String fileUrl = fileService.findUrlByFile(file);
            String filePath = FileUtil.getTmpDirPath() + file.getOriginalFileName();
            java.io.File tmpFile = new java.io.File(filePath);
            HttpUtil.downloadFile(fileUrl, tmpFile);
            String fileId = mhFileClient.uploadFile(tmpFile);
            tmpFile.deleteOnExit();
            return fileId;
        }).collect(Collectors.joining(","));
        PurchaseScopeEnum scope = PurchaseScopeEnum.getByCode(w.getPurchaseScope())
                .orElseThrow(() -> BizException.wrap("采购范围无效"));
        return MhPurchaseNoticeDTO.builder()
                .bidOpenTime(DateUtil.date(w.getBidStartTime()).toJdkDate())
                .endTime(DateUtil.date(w.getBidEndTime()).toJdkDate())
                .buyDesc(w.getPurchaseRemark())
                .budgetMoney(w.getBidBudget().multiply(BigDecimal.valueOf(10000)))
                .buyItem(w.getBidName())
                .buyFile(fileIds)
                .buyScope(scope.getMhCode())
                .buyWay(w.getPurchaseMethod())
                .buyUnitName(w.getPurchaseUnit())
                .createUnit(w.getPurchaseUnitId().intValue())
                .createTime(now)
                .updateTime(now)
                .createUser(user.getMhUserId())
                .updateUser(user.getMhUserId())
                // 招标采购
                .type("ZBGG")
                .detailAddress(w.getPurchaseConnectAddress())
                .linkAddress(w.getBidAddress())
                .linkUser(w.getPurchaseContact())
                .linkPhone(w.getPurchasePhone())
                .postStatus("Y")
                .build();
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitPurchaseRecord(SubmitPurchaseContractRecordReq req) {
        Purchase purchase = purchaseService.getById(req.getBidId());
        if (!TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段无法进行采购&合同信息备案");
        }
        // 保存采购信息
        purchase.setId(req.getBidId());
        purchase.setAcceptanceLetter(req.getAcceptanceLetter());
        purchase.setBidBuildContent(req.getBidBuildContent());
        purchase.setBiddingDoc(req.getBiddingDoc());
        purchase.setSupplier(req.getSupplier());
        purchase.setSupplierSocialCreditCode(req.getSupplierSocialCreditCode());
        tenderStateMachineUtil.pass(purchase);
        purchaseService.saveOrUpdate(purchase);

        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long projectId = req.getProjectId();
        Project project = projectService.getNewestNoNull(projectId);
        LocalDateTime now = LocalDateTime.now();

        // 保存合同信息
        Contract contract = contractService.getByBidId(req.getBidId())
                .orElseGet(() -> {
                    Contract newContract = new Contract();
                    newContract.setCreateBy(user.getUserIdStr());
                    newContract.setCreateOn(now);
                    return newContract;
                });
        contract.setContractTime(req.getContractTime());
        contract.setAttachment(req.getAttachment());
        contract.setName(req.getName());
        contract.setProjectId(projectId);
        contract.setProjectCode(project.getProjectCode());
        contract.setBidId(req.getBidId());
        contract.setUpdateBy(user.getUserIdStr());
        contract.setUpdateOn(now);
        contract.setTotalAmount(req.getTotalAmount());
        contract.setConstructionNo(req.getConstructionNo());
        contract.setSupplierContact(req.getSupplierContact());
        contract.setSupplierContactInfo(req.getSupplierContactInfo());
        contractService.saveOrUpdate(contract);
        // 判断是否所有标项都完成了采购合同备案
        Wrapper<Purchase> select = Wrappers.lambdaQuery(Purchase.class)
                .select(Purchase::getId, Purchase::getSupplier)
                .eq(Purchase::getProjectId, projectId);
        List<Purchase> purchases = purchaseService.list(select);
        if (CollUtil.allMatch(purchases, w -> StrUtil.isNotBlank(w.getSupplier()))) {
            projectStateMachineUtil.pass(project);
        }
        // 修改项目合同金额
        Wrapper<Contract> query = Wrappers.lambdaQuery(Contract.class)
                .eq(Contract::getProjectId, projectId);
        List<Contract> allContracts = contractService.list(query);
        BigDecimal totalAmount = DecimalUtil.sum(allContracts, Contract::getTotalAmount);
        project.setUpdateOn(now);
        project.setContractAmount(totalAmount);
        projectService.updateById(project);
    }

    public AdaptionProgressStatVO adaptionProgressStatistics(PurchaseAdaptionListReq req) {
        LambdaQueryWrapper<Purchase> query = Wrappers.lambdaQuery(Purchase.class)
                .select(Purchase::getStatus, Purchase::getId)
                .eq(Purchase::getBidType, BidTypeEnum.BUILD_APP.getCode())
                .exists(ExistsSqlConst.PURCHASE_EXISTS_STATUS_CHANGE
                        + " and npsc.event = {0}", TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .notExists(ExistsSqlConst.PURCHASE_EXISTS_PROJECT
                                + "and np.stage in ({0}, {1})", ProjectStatus.STOPPED.getCode(),
                        ProjectStatus.CHANGE.getCode())
                .orderByDesc(Purchase::getCreateOn);
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        if (!purchaseService.buildPurchaseQueryPermission(query, user)) {
            return null;
        }
        if (req.hasProjectQuery()) {
            List<Long> projectIds = filterProjectIdsForPurchase(req);
            if (projectIds.isEmpty()) {
                return null;
            }
            query.in(Purchase::getProjectId, projectIds);
        }
        query.like(StrUtil.isNotBlank(req.getBidName()), Purchase::getBidName, req.getBidName())
                .le(req.getCreateOnMax() != null, Purchase::getCreateOn, req.getCreateOnMax())
                .ge(req.getCreateOnMin() != null, Purchase::getCreateOn, req.getCreateOnMin());
        List<Purchase> purchases = purchaseService.list(query);
        return adaptionProgressStatistics(purchases);
    }

    public AdaptionProgressStatVO adaptionProgressStatistics(List<Purchase> purchases) {
        Map<ITenderStatus, Long> statusCountMap = CollUtils.groupCount(purchases,
                w -> {
                    ITenderStatus status = TenderMainStatus.getNoNull(w.getStatus());
                    if (TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN.eq(status)) {
                        return TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN;
                    } else if (TenderMainStatus.WAIT_ORG_CONFIRM.eq(status)) {
                        return TenderMainStatus.WAIT_ORG_CONFIRM;
                    } else if (TenderAdaptStatus.WITHOUT_ADAPT_INFO.eq(status)
                            || TenderAdaptStatus.ADAPT_INFO_AUDIT.eq(status)
                            || TenderAdaptStatus.ADAPT_INFO_FAILED.eq(status)) {
                        return TenderAdaptStatus.WITHOUT_ADAPT_INFO;
                    } else if (TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO.eq(status)
                            || TenderSelfTestStatus.SELF_TEST_INFO_AUDIT.eq(status)
                            || TenderSelfTestStatus.SELF_TEST_INFO_FAILED.eq(status)) {
                        return TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO;
                    } else if (TenderTestValidStatus.WITHOUT_TEST_VALID_INFO.eq(status)
                            || TenderTestValidStatus.TEST_VALID_INFO_AUDIT.eq(status)
                            || TenderTestValidStatus.TEST_VALID_INFO_FAILED.eq(status)) {
                        return TenderTestValidStatus.WITHOUT_TEST_VALID_INFO;
                    } else {
                        return status;
                    }
                });
        return AdaptionProgressStatVO.builder()
                .withoutAdaptionInfo(statusCountMap.getOrDefault(TenderAdaptStatus.WITHOUT_ADAPT_INFO, 0L))
                .withoutOperationCount(statusCountMap.getOrDefault(TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN, 0L))
                .withoutOrgConfirmCount(statusCountMap.getOrDefault(TenderMainStatus.WAIT_ORG_CONFIRM, 0L))
                .withoutSelfTestCount(statusCountMap.getOrDefault(TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO, 0L))
                .withoutTestValidCount(statusCountMap.getOrDefault(TenderTestValidStatus.WITHOUT_TEST_VALID_INFO, 0L))
                .totalCount(purchases.size())
                .build();
    }

    private PageVo<PurchaseAdaptionListVO> queryData(PurchaseAdaptionListReq req) {
        LambdaQueryWrapper<Purchase> query = Wrappers.lambdaQuery(Purchase.class)
                .eq(Purchase::getBidType, BidTypeEnum.BUILD_APP.getCode())
                .exists(ExistsSqlConst.PURCHASE_EXISTS_STATUS_CHANGE
                        + " and npsc.event = {0}", TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .notExists(ExistsSqlConst.PURCHASE_EXISTS_PROJECT
                                + "and np.stage in ({0}, {1})", ProjectStatus.STOPPED.getCode(),
                        ProjectStatus.CHANGE.getCode())
                .orderByDesc(Purchase::getCreateOn);
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        if (!purchaseService.buildPurchaseQueryPermission(query, user)) {
            return PageVo.empty();
        }
        if (req.hasProjectQuery()) {
            List<Long> projectIds = filterProjectIdsForPurchase(req);
            if (projectIds.isEmpty()) {
                return PageVo.empty();
            }
            query.in(Purchase::getProjectId, projectIds);
        }
        query.like(StrUtil.isNotBlank(req.getBidName()), Purchase::getBidName, req.getBidName())
                .le(req.getCreateOnMax() != null, Purchase::getCreateOn, req.getCreateOnMax())
                .ge(req.getCreateOnMin() != null, Purchase::getCreateOn, req.getCreateOnMin());
        if (req.getStatus() != null) {
            ITenderStatus tenderStatus = TenderMainStatus.getNoNull(req.getStatus());
            if (tenderStatus.eq(TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN)) {
                query.eq(Purchase::getStatus, TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN.getCode());
            } else if (tenderStatus.eq(TenderAdaptStatus.WITHOUT_ADAPT_INFO)) {
                query.in(Purchase::getStatus, TenderAdaptStatus.WITHOUT_ADAPT_INFO.getCode(),
                        TenderAdaptStatus.ADAPT_INFO_FAILED.getCode(),
                        TenderAdaptStatus.ADAPT_INFO_AUDIT.getCode());
            } else if (tenderStatus.eq(TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO)) {
                query.in(Purchase::getStatus, TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO.getCode(),
                        TenderSelfTestStatus.SELF_TEST_INFO_FAILED.getCode(),
                        TenderSelfTestStatus.SELF_TEST_INFO_AUDIT.getCode());
            } else if (tenderStatus.eq(TenderTestValidStatus.WITHOUT_TEST_VALID_INFO)) {
                query.in(Purchase::getStatus, TenderTestValidStatus.WITHOUT_TEST_VALID_INFO.getCode(),
                        TenderTestValidStatus.TEST_VALID_INFO_FAILED.getCode(),
                        TenderTestValidStatus.TEST_VALID_INFO_AUDIT.getCode());
            } else if (tenderStatus.eq(TenderMainStatus.WAIT_ORG_CONFIRM)) {
                query.eq(Purchase::getStatus, TenderMainStatus.WAIT_ORG_CONFIRM.getCode());
            } else {
                return PageVo.empty();
            }
        }
        Page<Purchase> page = purchaseService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<Purchase> records = page.getRecords();
        List<Long> projectIds = CollUtils.fieldList(records, Purchase::getProjectId);
        Wrapper<Project> projectQuery = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getProjectCode, Project::getProjectName, Project::getStatus,
                        Project::getBuildOrgName, Project::getBuildOrgCode, Project::getUnitStrip)
                .in(Project::getId, projectIds);
        List<Project> projects = projectService.list(projectQuery);
        Map<Long, Project> projectMap = CollUtils.listToMap(projects, Project::getId);
        Map<Long, Contract> contractMap = contractService.listByProjectIds(projectIds);
        List<Long> finishOrgConfirmBidIds = new ArrayList<>();
        if (req.getStatus() == null) {
            List<Long> bidIds = CollUtils.fieldList(records, Purchase::getId);
            Wrapper<PurchaseStatusChange> pscQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                    .select(PurchaseStatusChange::getBidId)
                    .eq(PurchaseStatusChange::getEvent, TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM)
                    .in(PurchaseStatusChange::getBidId, bidIds);
            List<PurchaseStatusChange> changes = purchaseStatusChangeService.list(pscQuery);
            changes.forEach(c -> finishOrgConfirmBidIds.add(c.getBidId()));
        }
        List<PurchaseAdaptionListVO> data = records.stream().map(w -> {
            PurchaseAdaptionListVO adaption = new PurchaseAdaptionListVO();
            adaption.setBidId(w.getId());
            if (finishOrgConfirmBidIds.contains(w.getId())) {
                adaption.setStatus(TenderMainStatus.ORG_CONFIRM_PASSED.getCode());
            } else {
                adaption.setStatus(w.getStatus());
            }
            adaption.setBizName(w.getBidName());
            adaption.setProjectId(w.getProjectId());
            Project project = projectMap.get(w.getProjectId());
            adaption.setProjectName(project.getProjectName());
            adaption.setBuildOrg(project.getBuildOrgName());
            adaption.setProjectStatus(project.getStatus());
            adaption.setProjectCode(project.getProjectCode());
            adaption.setUnitStripName(MhUnitStripEnum.getVal(project.getUnitStrip()));
            Contract contract = contractMap.get(w.getId());
            if (contract != null) {
                adaption.setContractAmount(contract.getTotalAmount());
            }
            return adaption;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    private List<Long> filterProjectIdsForPurchase(PurchaseAdaptionListReq req) {
        LambdaQueryWrapper<Project> projQuery = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .eq(Project::getNewest, Boolean.TRUE)
                .like(StrUtil.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName())
                .like(StrUtil.isNotBlank(req.getBuildOrgName()), Project::getBuildOrgName, req.getBuildOrgName());
        if (req.getBuildOrgCode() != null) {
            projQuery.in(Project::getBuildOrgCode, mhUnitCache.getViewChildIdsRecursion(req.getBuildOrgCode()));
        }
        List<Project> projects = projectService.list(projQuery);
        return CollUtils.fieldList(projects, Project::getId);
    }


    public PageVo<PurchaseAdaptionListVO> adaptList(PurchaseAdaptionListReq req) {
        return queryData(req);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitTestValidInfo(TestValidInfoReq req) {
        List<Purchase> purchases = purchaseService.listByIds(req.getBidIds());
        purchases.forEach(purchase -> {
            if (!TenderTestValidStatus.WITHOUT_TEST_VALID_INFO.eq(purchase.getTestValidStatus())
                    && !TenderTestValidStatus.TEST_VALID_INFO_FAILED.eq(purchase.getTestValidStatus())) {
                throw BizException.wrap("该状态下不支持提交测试验证信息");
            }
        });
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectProcessType instType = ProjectProcessType.TEST_VALID;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());
        LocalDateTime now = LocalDateTime.now();
        for (Purchase purchase : purchases) {
            Long projectId = purchase.getProjectId();
            Project project = projectService.getNewProject(projectId);
            ProcessStartParamsVo instParam = new ProcessStartParamsVo();
            instParam.setUser(declaredProjectManage.buildUser(user));
            instParam.setProcessUsers(Collections.emptyMap());
            instParam.setFormData(Collections.emptyMap());
            // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
            Map<String, OrgInfoDTO> orgMap = declaredProjectManage.buildOrgModelMap(user, project);
            String instanceId = processInstanceService.startProcessLs(model, instParam, orgMap);
            testValidStateMachineUtil.pass(purchase);
            purchaseService.updateById(purchase);
            Operation operation = operationService.getByBidId(purchase.getId())
                    .orElseGet(() -> {
                        Operation newOperation = new Operation();
                        newOperation.setBidId(purchase.getId());
                        newOperation.setCreateBy(user.getUsername());
                        newOperation.setCreateOn(now);
                        newOperation.setProjectId(projectId);
                        newOperation.setProjectCode(project.getProjectCode());
                        return newOperation;
                    });
            operation.setUpdateBy(user.getUsername());
            operation.setUpdateOn(now);
            operation.setActualTestValidTime(req.getActualTime());
            operation.setTestValidFiles(req.getFiles());
            operation.setTestValidRemark(req.getRemark());
            operationService.saveOrUpdate(operation);
            saveProjectPurchaseInstCode(purchase.getId(), instanceId, project.getId(), instType);
        }
    }

    private void saveProjectPurchaseInstCode(Long bidId,
                                             String instanceId,
                                             Long projectId,
                                             ProjectProcessType instType) {
        // 保存标段实列关系
        PurchaseInst purchaseInst = new PurchaseInst();
        purchaseInst.setBidId(bidId);
        purchaseInst.setInstCode(instanceId);
        purchaseInst.setProjectId(projectId);
        purchaseInst.setInstType(instType);
        purchaseInstService.save(purchaseInst);
        // 保存项目实列关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstType(instType);
        projectInst.setInstCode(instanceId);
        projectInst.setProjectId(projectId);
        projectInstService.save(projectInst);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitSelfTestInfo(TestValidInfoReq req) {
        List<Purchase> purchases = purchaseService.listByIds(req.getBidIds());
        for (Purchase purchase : purchases) {
            if (!TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO.eq(purchase.getSelfTestStatus())
                    && !TenderSelfTestStatus.SELF_TEST_INFO_FAILED.eq(purchase.getSelfTestStatus())) {
                throw BizException.wrap("该状态下不支持提交系统自测信息");
            }
        }
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectProcessType instType = ProjectProcessType.SELF_TEST;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());

        LocalDateTime now = LocalDateTime.now();
        for (Purchase purchase : purchases) {
            Long projectId = purchase.getProjectId();
            Project project = projectService.getNewProject(projectId);
            ProcessStartParamsVo instParam = new ProcessStartParamsVo();
            instParam.setUser(declaredProjectManage.buildUser(user));
            instParam.setProcessUsers(Collections.emptyMap());
            instParam.setFormData(Collections.emptyMap());
            // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
            Map<String, OrgInfoDTO> orgMap = declaredProjectManage.buildOrgModelMap(user, project);
            String instanceId = processInstanceService.startProcessLs(model, instParam, orgMap);
            selfTestStateMachineUtil.pass(purchase);
            purchaseService.updateById(purchase);
            Operation operation = operationService.getByBidId(purchase.getId())
                    .orElseGet(() -> {
                        Operation newOperation = new Operation();
                        newOperation.setBidId(purchase.getId());
                        newOperation.setCreateBy(user.getUsername());
                        newOperation.setCreateOn(now);
                        newOperation.setProjectId(projectId);
                        newOperation.setProjectCode(project.getProjectCode());
                        return newOperation;
                    });
            operation.setActualSelfTestTime(req.getActualTime());
            operation.setSelfTestFiles(req.getFiles());
            operation.setSelfTestRemark(req.getRemark());
            operation.setUpdateBy(user.getUsername());
            operation.setUpdateOn(now);
            operationService.saveOrUpdate(operation);
            saveProjectPurchaseInstCode(purchase.getId(), instanceId, project.getId(), instType);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitAdaptionInfo(TestValidInfoReq req) {
        List<Purchase> purchases = purchaseService.listByIds(req.getBidIds());
        for (Purchase purchase : purchases) {
            if (!TenderAdaptStatus.WITHOUT_ADAPT_INFO.eq(purchase.getAdaptStatus())
                    && !TenderAdaptStatus.ADAPT_INFO_FAILED.eq(purchase.getAdaptStatus())) {
                throw BizException.wrap("该状态下不支持提交适配改造信息");
            }
        }
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectProcessType instType = ProjectProcessType.ADAPTION;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());

        LocalDateTime now = LocalDateTime.now();
        for (Purchase purchase : purchases) {
            Long projectId = purchase.getProjectId();
            Project project = projectService.getNewProject(projectId);
            ProcessStartParamsVo instParam = new ProcessStartParamsVo();
            instParam.setUser(declaredProjectManage.buildUser(user));
            instParam.setProcessUsers(Collections.emptyMap());
            instParam.setFormData(Collections.emptyMap());
            // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
            Map<String, OrgInfoDTO> orgMap = declaredProjectManage.buildOrgModelMap(user, project);
            String instanceId = processInstanceService.startProcessLs(model, instParam, orgMap);
            adaptStateMachineUtil.pass(purchase);
            purchaseService.updateById(purchase);
            Operation operation = operationService.getByBidId(purchase.getId())
                    .orElseGet(() -> {
                        Operation newOperation = new Operation();
                        newOperation.setBidId(purchase.getId());
                        newOperation.setCreateBy(user.getUsername());
                        newOperation.setCreateOn(now);
                        newOperation.setProjectId(projectId);
                        newOperation.setProjectCode(project.getProjectCode());
                        return newOperation;
                    });
            operation.setActualAdaptionTime(req.getActualTime());
            operation.setAdaptionFiles(req.getFiles());
            operation.setAdaptionRemark(req.getRemark());
            operation.setUpdateBy(user.getUsername());
            operation.setUpdateOn(now);
            operationService.saveOrUpdate(operation);
            saveProjectPurchaseInstCode(purchase.getId(), instanceId, project.getId(), instType);
        }
    }

    public PageVo<XcfhxApplyListVO> pageXcfhxApply(XcfhxListReq req) {
        Integer pageNumber = req.getPageNumber();
        Integer pageSize = req.getPageSize();
        //手动添加的
        List<XcfhxApplyListVO> manuals = xinchuangService.listXcfhxApplyListVO(req);
        //系统和大数据推送的
        List<XcfhxApplyListVO> others = purchaseService.listXcfhxApplyListVO(req);

        //合并
        List<XcfhxApplyListVO> merge = Stream.of(manuals, others)
                .flatMap(List::stream)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        return PageVo.of(merge.stream().sorted(Comparator.comparing(XcfhxApplyListVO::getCreateOn).reversed())
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize).collect(Collectors.toList()), merge.size());
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    public XinchuangVO detailXcfhxApply(Long id) {
        Xinchuang xinchuang = xinchuangService.getById(id);
        if (Objects.isNull(xinchuang)) {
            return null;
        }
        return BeanUtil.copyProperties(xinchuang, XinchuangVO.class);
    }

    public List<PurchaseNoticeTenderDTO> listPurchaseIntention(Long unitId) {
        List<MhPurchaseIntentionDTO> intentions = mhApiClient.listPurchaseNotice(unitId);
        if (intentions.isEmpty()) {
            return Collections.emptyList();
        }
        return intentions.stream().map(w -> {
            PurchaseScopeEnum scope = PurchaseScopeEnum.getByMhCode(w.getBuyScope())
                    .orElseThrow(() -> BizException.wrap("采购范围无效"));
            PurchaseNoticeTenderDTO item = new PurchaseNoticeTenderDTO();
            item.setBidName(w.getBuyItem());
            item.setBidBudget(BigDecimal.valueOf(w.getBudgetMoney())
                    .divide(BigDecimal.valueOf(10000), RoundingMode.HALF_UP));
            item.setPurchaseMethod(w.getBuyWay());
            item.setPurchaseContact(w.getLinkUser());
            item.setPurchasePhone(w.getLinkPhone());
            item.setPurchaseConnectAddress(w.getDetailAddress());
            BizUtils.notNull(w.getBidOpenTime(), t -> item.setBidStartTime(LocalDateTimeUtil.of(t)));
            BizUtils.notNull(w.getEndTime(), t -> item.setBidEndTime(LocalDateTimeUtil.of(t)));
            item.setPurchaseRemark(w.getBuyDesc());
            item.setPurchaseScope(scope.getCode());
            return item;
        }).collect(Collectors.toList());
    }

    /**
     * 新增信创系统
     *
     * @param dto
     * @return
     */
    public Long addSystem(XcfhxApplySaveDTO dto) {
        Xinchuang xinchuang = BeanUtil.copyProperties(dto, Xinchuang.class);
        xinchuang.setSourceType(PurchaseSourceTypeEnum.MANUAL_ADDITION.getCode());
        if (Boolean.TRUE.equals(dto.getMatchXcfhx())
                && StrUtils.isNotBlank(dto.getMhXcfhxReportFile())) {
            String fileName = xinchuang.getBidName() + "-信创符合性测评报告.";
            xinchuang.setXcfhxReportFiles(mhXchxFileHelper.getXchxFile(dto.getMhXcfhxReportFile(), fileName));
        }

        if (xinchuangService.save(xinchuang)) {
            return xinchuang.getId();
        }
        throw BizException.wrap("新增失败");
    }

    public Long updateSystem(XcfhxApplyEditDTO dto) {
        Xinchuang old = xinchuangService.getById(dto.getId());
        VUtils.isTrue(Objects.isNull(old)).throwMessage("数据不存在");
        Xinchuang xinchuang = BeanUtil.copyProperties(dto, Xinchuang.class);
        if (Boolean.TRUE.equals(dto.getMatchXcfhx())
                && StrUtils.isNotBlank(dto.getMhXcfhxReportFile()) &&
                !dto.getMhXcfhxReportFile().equals(old.getMhXcfhxReportFile())) {
            String fileName = xinchuang.getBidName() + "-信创符合性测评报告.";
            xinchuang.setXcfhxReportFiles(mhXchxFileHelper.getXchxFile(dto.getMhXcfhxReportFile(), fileName));
        }
        if (xinchuangService.updateById(xinchuang)) {
            return xinchuang.getId();
        }
        throw BizException.wrap("编辑失败");
    }

    public String xcfhxReportFiles(String code) {
        return mhXchxFileHelper.getXchxFile(code, null);
    }

    public void exportXcfhFile(String code, HttpServletResponse response) throws UnsupportedEncodingException {
        mhXchxFileHelper.exportReport(code, response);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void purchaseOrgConfirm(PurchaseOrgConfirmReq req) {
        Purchase purchase = purchaseService.getById(req.getBidId());
        if (!TenderMainStatus.WAIT_ORG_CONFIRM.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段不支持单位确认");
        }
        tenderStateMachineUtil.pass(purchase);
        purchaseService.updateById(purchase);
        LambdaQueryWrapper<Purchase> purchaseQuery = Wrappers.lambdaQuery(Purchase.class)
                .select(Purchase::getStatus, Purchase::getBidType)
                .eq(Purchase::getProjectId, purchase.getProjectId());
        List<Purchase> purchases = purchaseService.list(purchaseQuery);
        boolean changeProjectStatus = CollUtil.allMatch(purchases, w -> {
            if (!BidTypeEnum.BUILD_APP.eq(w.getBidType())) {
                return true;
            }
            return TenderMainStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO.eq(w.getStatus());
        });
        if (changeProjectStatus) {
            Project project = projectService.getNewestNoNull(purchase.getProjectId());
            projectStateMachineUtil.pass(project);
            projectService.updateById(project);
        }
    }

}
