package com.hz.pm.api.projectdeclared.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.ningdatech.basic.util.CollUtils;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author zpf
 * @since 2023-05-29
 */
public interface IContractService extends IService<Contract> {

    default Map<Long, Contract> listByBidIds(Collection<Long> bidIds) {
        LambdaQueryWrapper<Contract> query = Wrappers.lambdaQuery(Contract.class)
                .in(Contract::getBidId, bidIds);
        return CollUtils.listToMap(list(query), Contract::getBidId);
    }

    default Optional<Contract> getByBidId(Long bidId) {
        LambdaQueryWrapper<Contract> query = Wrappers.lambdaQuery(Contract.class)
                .eq(Contract::getBidId, bidId)
                .last(BizConst.LIMIT_1);
        return Optional.ofNullable(getOne(query));
    }

    default Map<Long, Contract> listByProjectIds(Collection<Long> projectIds) {
        LambdaQueryWrapper<Contract> query = Wrappers.lambdaQuery(Contract.class)
                .in(Contract::getProjectId, projectIds);
        return CollUtils.listToMap(list(query), Contract::getBidId);
    }

    default Map<Long, BigDecimal> listContractAmountByProjectIds(Collection<Long> projectIds) {
        LambdaQueryWrapper<Contract> query = Wrappers.lambdaQuery(Contract.class)
                .select(Contract::getBidId, Contract::getTotalAmount)
                .in(Contract::getProjectId, projectIds);
        return CollUtils.listToMap(list(query), Contract::getBidId, Contract::getTotalAmount);
    }

}
