package com.hz.pm.api.projectdeclared.utils;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.RegionConst;
import com.hz.pm.api.gov.model.dto.*;
import com.hz.pm.api.gov.model.entity.GovBizProjectBaseinfo;
import com.hz.pm.api.gov.model.entity.GovOperationProjectBaseinfo;
import com.hz.pm.api.gov.service.IGovBizProjectBaseinfoService;
import com.hz.pm.api.gov.service.IGovOperationProjectBaseinfoService;
import com.hz.pm.api.projectdeclared.contant.ProjectCodeCont;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectCollection;
import com.hz.pm.api.projectlib.service.IProjectService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.util.Objects;

/**
 * 生成项目编号工具类
 *
 * @author CMM
 * @since 2023/04/04 14:13
 */
@Component
@RequiredArgsConstructor
public class ProjectCodeGenUtil {

    private final IProjectService projectService;
    private final IGovBizProjectBaseinfoService baseinfoService;
    private final IGovOperationProjectBaseinfoService operationProjectBaseinfoService;

    public String generateProjectCode(ProjectDTO project) {
        String areaCode = RegionConst.RC_HZ + ProjectCodeCont.REGION_LAST_3;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(project.getProjectYear()) ? String.valueOf(project.getProjectYear())
                : String.valueOf(LocalDate.now().getYear());
        // 16-20 项目序号00001
        String currentCode = areaCode + year;
        // 16-20 项目序号00001
        //要查询 两边的项目序号
        Long max = getMaxProjectCode(currentCode);
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeCont.SHUZI_5, max);
    }

    public String generateProjectCode(Integer projectYear) {
        String areaCode = RegionConst.RC_HZ + ProjectCodeCont.REGION_LAST_3;
        // 获取建设年度 (10-13)
        String year = projectYear != null ? String.valueOf(projectYear) : String.valueOf(LocalDate.now().getYear());
        // 16-20 项目序号00001
        String currentCode = areaCode + year;
        // 16-20 项目序号00001
        //要查询 两边的项目序号
        Long max = getMaxProjectCode(currentCode);
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeCont.SHUZI_5, max);
    }

    public String generateProjectCode(ProjectCollection project) {
        // 获取所属行政区划代码(9位)
        String areaCode = RegionConst.RC_HZ + ProjectCodeCont.REGION_LAST_3;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(project.getProjectYear()) ? String.valueOf(project.getProjectYear())
                : String.valueOf(LocalDate.now().getYear());
        // 16-20 项目序号00001
        String currentCode = areaCode + year;
        // 16-20 项目序号00001
        //要查询 最大的项目编号
        Long max = getMaxProjectCode(currentCode);
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeCont.SHUZI_5, max);
    }

    public String generateProjectCode(GovBizProjectSaveDTO dto) {
        // 获取所属行政区划代码(9位)
        GovBizProjectBaseinfoDTO baseinfo = dto.getBaseinfo();
        Assert.notNull(baseinfo, "没有项目基本信息 无法生成项目编号!");
        GovBizProjectApplyDTO apply = dto.getApply();
        Assert.notNull(apply, "没有项目申报信息 无法生成项目编号!");

        String areaCode = RegionConst.RC_HZ + BizConst.NINE_AREA_CODE_LAST;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(apply.getBaseProjSetYear()) ? apply.getBaseProjSetYear()
                : String.valueOf(LocalDate.now().getYear());
        String currentCode = areaCode + year;
        // 16-20 项目序号00001
        //要查询 最大的项目编号
        Long max = getMaxProjectCode(currentCode);
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeCont.SHUZI_5, max);
    }

    public String generateProjectCode(GovOperationProjectSaveDTO dto) {
        // 获取所属行政区划代码(9位)
        GovOperationProjectBaseinfoDTO baseinfo = dto.getBaseinfo();
        Assert.notNull(baseinfo, "没有项目基本信息 无法生成项目编号!");
        GovBizProjectApplyDTO apply = dto.getApply();
        Assert.notNull(apply, "没有项目申报信息 无法生成项目编号!");

        String areaCode = RegionConst.RC_HZ + BizConst.NINE_AREA_CODE_LAST;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(apply.getBaseProjSetYear()) ? apply.getBaseProjSetYear()
                : String.valueOf(LocalDate.now().getYear());
        String currentCode = areaCode + year;
        // 16-20 项目序号00001
        //要查询 最大的项目编号
        Long max = getMaxProjectCode(currentCode);
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeCont.SHUZI_5, max);
    }

    private synchronized Long getMaxProjectCode(String currentCode) {
        long max = 0L;
        GovBizProjectBaseinfo baseMax = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(GovBizProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovBizProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if (Objects.nonNull(baseMax) && StringUtils.isNotBlank(baseMax.getBaseProjId())) {
            max = Long.parseLong(StrUtil.removePrefix(baseMax.getBaseProjId(), currentCode));
        }
        GovOperationProjectBaseinfo operationBaseMax = operationProjectBaseinfoService.getOne(Wrappers.lambdaQuery(GovOperationProjectBaseinfo.class)
                .like(GovOperationProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovOperationProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if (Objects.nonNull(operationBaseMax) && StringUtils.isNotBlank(operationBaseMax.getBaseProjId())) {
            long operationMax = Long.parseLong(StrUtil.removePrefix(operationBaseMax.getBaseProjId(), currentCode));
            max = Math.max(max, operationMax);
        }
        Project projectMax = projectService.getOne(Wrappers.lambdaQuery(Project.class)
                .like(Project::getProjectCode, currentCode)
                .orderByDesc(Project::getProjectCode)
                .last(BizConst.LIMIT_1));
        if (Objects.nonNull(projectMax) && StringUtils.isNotBlank(projectMax.getProjectCode())) {
            long projectMaxCode = Long.parseLong(StrUtil.removePrefix(projectMax.getProjectCode(), currentCode));
            max = Math.max(max, projectMaxCode);
        }
        max = max + 1;
        return max;
    }

}
