package com.hz.pm.api.projectlib.controller;

import com.hz.pm.api.projectlib.manage.MhSystemReplaceManage;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.dto.MhSystemStoppedRecordDTO;
import com.hz.pm.api.projectlib.model.req.MhSystemReplaceReq;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * MhSystemReplaceController
 * </p>
 *
 * @author WendyYang
 * @since 15:10 2024/8/16
 */
@Api(tags = "系统替代情况底账")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/mh/systemReplace")
public class MhSystemReplaceController {

    private final MhSystemReplaceManage mhSystemReplaceManage;

    @GetMapping("/page")
    @ApiOperation("列表查询")
    public PageVo<MhSystemReplaceInfoVO> page(MhSystemReplaceReq req) {
        return mhSystemReplaceManage.page(req);
    }

    @GetMapping("/option")
    @ApiOperation("绑定改造情况下拉")
    public List<MhSystemReplaceInfoVO> option(MhSystemReplaceReq req) {
        return mhSystemReplaceManage.option(req);
    }

    @GetMapping("/sync")
    @ApiOperation("同步系统改造计划数据")
    @WebLog("同步系统改造计划数据")
    @PreAuthorize("hasAuthority('SUPER_ADMIN')")
    public void syncMhSystemReplaceInfo() {
        mhSystemReplaceManage.syncMhSystemReplaceInfo();
    }

    @PostMapping("/submitStoppedRecord")
    @ApiOperation("提交停用备案")
    @WebLog("提交停用备案")
    public void submitStoppedRecord(@RequestBody @Valid MhSystemStoppedRecordDTO req) {
        mhSystemReplaceManage.submitStoppedRecord(req);
    }

}
