package com.hz.pm.api.projectlib.controller;


import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.projectlib.manage.ProjectRenewalFundManage;
import com.hz.pm.api.projectlib.model.dto.ProjectRenewalFundDeclarationDTO;
import com.hz.pm.api.projectlib.model.req.ProjectRenewalAuditReq;
import com.hz.pm.api.projectlib.model.req.ProjectRenewalListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectRenewalFundDeclarationVO;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;

/**
 * <p>
 * 续建项目资金申请表 前端控制器
 * </p>
 *
 * @author WendyYang
 * @since 2023-02-15
 */
@RestController
@RequestMapping("/api/v1/project/renewal")
@AllArgsConstructor
@Api(tags = "续建项目资金库")
public class ProjectRenewalFundDeclarationController {
    private final ProjectRenewalFundManage projectRenewalFundManage;

    private final UserInfoHelper userInfoHelper;

    @GetMapping("/list")
    @ApiOperation("项目续建资金库")
    public PageVo<ProjectRenewalFundDeclarationVO> list(ProjectRenewalListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        return projectRenewalFundManage.list(req, user);
    }

    @GetMapping("/{id}")
    @ApiOperation("项目续建资金详情")
    @WebLog("项目续建资金详情")
    public ProjectRenewalFundDeclarationVO detail(@PathVariable Long id) {
        return projectRenewalFundManage.detail(id);
    }

    @PostMapping("/declared")
    @ApiOperation("项目续建资金申请")
    @WebLog("项目续建资金申请")
    public Long declared(@Validated @RequestBody ProjectRenewalFundDeclarationDTO dto) {
        return projectRenewalFundManage.declared(dto);
    }

    @PostMapping("/audit")
    @ApiOperation("续建项目审核")
    @WebLog("续建项目审核")
    public Long audit(@Validated @RequestBody ProjectRenewalAuditReq param) {
        return projectRenewalFundManage.audit(param);
    }

    @DeleteMapping("/delete/{projectRenewalId}")
    @ApiOperation("续建项目删除")
    @WebLog("续建项目删除")
    public Long delete(@PathVariable("projectRenewalId") Long projectRenewalId) {
        return projectRenewalFundManage.delete(projectRenewalId);
    }

    @GetMapping("/export")
    @ApiOperation("项目库续建项目列表导出")
    @WebLog("项目库续建项目列表导出")
    public void exportList(ProjectRenewalListReq req, HttpServletResponse response) {
        ExcelDownUtil.downXls(response, req, projectRenewalFundManage::exportList);
    }

    @GetMapping("/exportRenewalDeclare")
    @ApiOperation("申报管理续建项目资金申报列表导出")
    @WebLog("申报管理续建项目资金申报列表导出")
    public void exportRenewalDeclareList(ProjectRenewalListReq req, HttpServletResponse response) {
        ExcelDownUtil.downXls(response, req, projectRenewalFundManage::exportRenewalDeclareList);
    }

}
