package com.hz.pm.api.projectlib.controller;

import com.hz.pm.api.projectlib.manage.ProjectChangeManage;
import com.hz.pm.api.projectlib.manage.ProjectStoppedManage;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.SubmitProjectChangeApplyReq;
import com.hz.pm.api.projectlib.model.req.SubmitProjectStoppedApplyReq;
import com.hz.pm.api.projectlib.model.vo.ProjectChangeDetailVO;
import com.hz.pm.api.projectlib.model.vo.ProjectChangeListVO;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.ProjectStoppedDetailVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * ProjectStoppedController
 * </p>
 *
 * @author WendyYang
 * @since 11:03 2024/8/5
 */
@RestController
@Api(tags = "项目终止变更控制器")
@RequestMapping("/api/v1/project")
@RequiredArgsConstructor
public class ProjectStoppedChangeController {

    private final ProjectStoppedManage projectStoppedManage;
    private final ProjectChangeManage projectChangeManage;

    @GetMapping("/stopped/detail/{projectId}")
    @ApiOperation("项目终止详情")
    @WebLog("项目终止详情")
    public ProjectStoppedDetailVO stoppedDetail(@PathVariable("projectId") Long projectId) {
        return projectStoppedManage.detail(projectId);
    }

    @PostMapping("/stopped/submitApply")
    @ApiOperation("提交项目终止申请")
    @WebLog("提交项目终止申请")
    public void submitStoppedApply(@RequestBody @Valid SubmitProjectStoppedApplyReq req) {
        projectStoppedManage.submitProjectStoppedApply(req);
    }

    @PostMapping("/change/submitApply")
    @ApiOperation("提交项目变更申请")
    @WebLog("提交项目变更申请")
    public void submitChangeApply(@RequestBody @Valid SubmitProjectChangeApplyReq req) {
        projectChangeManage.submitProjectChangeApply(req);
    }

    @GetMapping("/stopped/page")
    @ApiOperation("项目终止列表")
    @WebLog("项目终止列表")
    public PageVo<ProjectLibListItemVO> pageStopped(ProjectListReq req) {
        return projectStoppedManage.page(req);
    }

    @GetMapping("/stopped/options")
    @ApiOperation("项目终止下拉筛选")
    @WebLog("项目终止下拉筛选")
    public List<ProjectLibListItemVO> stoppedOptions() {
        return projectStoppedManage.options();
    }

    @GetMapping("/change/page")
    @ApiOperation("项目变更列表")
    @WebLog("项目变更列表")
    public PageVo<ProjectChangeListVO> pageChange(ProjectListReq req) {
        return projectChangeManage.page(req);
    }

    @GetMapping("/change/detail/{id}")
    @ApiOperation("项目变更详情")
    @WebLog("项目变更详情")
    public ProjectChangeDetailVO changeDetail(@PathVariable("id") Long id) {
        return projectChangeManage.changeDetail(id);
    }

    @GetMapping("/change/detail/{projectId}/{instCode}")
    @ApiOperation("项目变更详情")
    @WebLog("项目变更详情")
    public ProjectChangeDetailVO changeDetail(@PathVariable("projectId") Long projectId,
                                              @PathVariable("instCode") String instCode) {
        return projectChangeManage.changeDetail(projectId, instCode);
    }

    @GetMapping("/change/options")
    @ApiOperation("项目变更下拉筛选")
    @WebLog("项目变更下拉筛选")
    public List<ProjectLibListItemVO> changeOptions() {
        return projectChangeManage.options();
    }

}
