package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.todocenter.util.BuildUserUtils;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 流程执行链
 *
 * @author CMM
 * @since 2023/02/26 21:36
 */
@Component
@RequiredArgsConstructor
public class ProcessProgressHelper {

    /**
     * 具体处理者的集合
     */
    private final List<AbstractProcessHandle> processHandles;
    private final BuildUserUtils buildUserUtils;

    public List<ProcessDetailVO> handle(Project project) {
        HandleParamContext handleParamContext = new HandleParamContext();
        handleParamContext.setProject(project);
        handleParamContext.setProcessDetails(new ArrayList<>());
        if (!processHandles.isEmpty()) {
            processHandles.sort(Comparator.comparing(AbstractProcessHandle::getOrder));
            for (AbstractProcessHandle processBusinessHandle : processHandles) {
                processBusinessHandle.handle(handleParamContext);
            }
            List<ProcessDetailVO> details = handleParamContext.getProcessDetails();
            List<ProgressNode> prgressList = details.stream()
                    .filter(w -> !w.getStepStatus().equals(StepStatusEnum.NOT_START))
                    .map(w -> {
                        List<ProgressNode> progresses = new ArrayList<>();
                        ProcessProgressVo progress = w.getProcessProgressVo();
                        if (progress != null) {
                            progresses.addAll(progress.getProgressInfo());
                        } else if (w.getProcessInfoList() != null) {
                            for (ProcessInfoVO processInfo : w.getProcessInfoList()) {
                                progresses.addAll(processInfo.getProcessProgressVo().getProgressInfo());
                            }
                        }
                        return progresses;
                    }).flatMap(Collection::stream).collect(Collectors.toList());
            if (!prgressList.isEmpty()) {
                buildUserUtils.buildUserByProcessInfo(prgressList);
            }
            return details;
        }
        return Collections.emptyList();
    }

}
