package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * 项目终验处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class FinalInspectionHandle extends AbstractProcessHandle {

    private final ProcessInstanceService processInstanceService;

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.FINAL_INSPECTED;
    }

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        if (!ctx.getPreStepStatus().equals(StepStatusEnum.COMPLETED)) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            List<Purchase> purchases = ctx.getPurchases();
            Map<Long, Purchase> purchaseMap = purchases.stream()
                    .collect(Collectors.toMap(Purchase::getId, Function.identity()));
            StepStatusEnum stepStatus;
            if (purchaseMap.isEmpty()) {
                stepStatus = StepStatusEnum.NOT_START;
            } else {
                List<ProcessInfoVO> processes = new ArrayList<>();
                Map<Long, PurchaseInst> instMap = getPurchaseInstMap(ctx.getPurchaseInstList(),
                        purchaseMap.keySet(), InstTypeEnum.PROJECT_FINAL_INSPECTION);
                instMap.forEach((bidId, inst) -> {
                    if (detail.getFinishTime() == null || detail.getFinishTime().isBefore(inst.getCreateOn())) {
                        detail.setFinishTime(inst.getCreateOn());
                    }
                    ProcessProgressVo instDetail = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
                    ProcessInfoVO tenderProgress = new ProcessInfoVO();
                    tenderProgress.setProcessName(purchaseMap.get(bidId).getBidName());
                    tenderProgress.setProcessProgressVo(instDetail);
                    processes.add(tenderProgress);
                });
                if (instMap.size() != purchases.size()) {
                    stepStatus = StepStatusEnum.NOT_START;
                } else {
                    stepStatus = getPurchaseStatus(processes);
                }
                if (stepStatus != StepStatusEnum.NOT_START) {
                    detail.setProcessInfoList(processes);
                }
            }
            detail.setStepStatus(stepStatus);
        }
        ctx.getProcessDetails().add(detail);
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
