package com.hz.pm.api.projectlib.handle.stage;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

/**
 * 初验处理
 *
 * @author CMM
 * @since 2023/02/27 10:09
 */
@Component
@RequiredArgsConstructor
public class FirstInspectedHandle extends AbstractProcessHandle {

    private final IProjectStatusChangeService projectStatusChangeService;

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.FIRST_INSPECTED;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        if (!ctx.getPreStepStatus().equals(StepStatusEnum.COMPLETED)) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            Wrapper<ProjectStatusChange> query = Wrappers.lambdaQuery(ProjectStatusChange.class)
                    .eq(ProjectStatusChange::getProjectCode, ctx.getProject().getProjectCode())
                    .eq(ProjectStatusChange::getEvent, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES)
                    .orderByDesc(ProjectStatusChange::getCreateOn)
                    .last(BizConst.LIMIT_1);
            ProjectStatusChange change = projectStatusChangeService.getOne(query);
            if (change != null) {
                detail.setFinishTime(change.getCreateOn());
                detail.setStepStatus(StepStatusEnum.COMPLETED);
                ctx.setPreStepStatus(StepStatusEnum.COMPLETED);
            } else {
                detail.setStepStatus(StepStatusEnum.NOT_START);
                ctx.setPreStepStatus(StepStatusEnum.NOT_START);
            }
        }
    }

}
