package com.hz.pm.api.projectlib.handle.stage;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

/**
 * <p>
 * PilotRunningHandle
 * </p>
 *
 * @author WendyYang
 * @since 10:34 2024/4/9
 */
@Component
@RequiredArgsConstructor
public class PilotRunningHandle extends AbstractProcessHandle {

    private final IProjectStatusChangeService projectStatusChangeService;

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        if (!ctx.getPreStepStatus().equals(StepStatusEnum.COMPLETED)) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            Wrapper<ProjectStatusChange> query = Wrappers.lambdaQuery(ProjectStatusChange.class)
                    .eq(ProjectStatusChange::getProjectCode, ctx.getProject().getProjectCode())
                    .in(ProjectStatusChange::getEvent,
                            ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES,
                            ProjectStateChangeEvent.FINAL_ACCEPTANCE_APPLICATION)
                    .orderByDesc(ProjectStatusChange::getCreateOn)
                    .last(BizConst.LIMIT_1);
            ProjectStatusChange change = projectStatusChangeService.getOne(query);
            if (change != null) {
                if (ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES.name().equals(change.getEvent())) {
                    detail.setStepStatus(StepStatusEnum.ON_GOING);
                } else {
                    detail.setStepStatus(StepStatusEnum.COMPLETED);
                }
                detail.setFinishTime(change.getCreateOn());
            } else {
                detail.setStepStatus(StepStatusEnum.NOT_START);
            }
            ctx.setPreStepStatus(detail.getStepStatus());
        }
    }

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.PILOT_RUNNING;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }
}
