package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.ProcessStatusEnum;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Comparator;
import java.util.Map;
import java.util.Optional;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.*;

/**
 * 立项批复处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ProjectApprovalHandle extends AbstractProcessHandle {

    private final ProcessInstanceService processInstanceService;

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_APPROVAL;
    }

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        if (!StepStatusEnum.COMPLETED.equals(ctx.getPreStepStatus())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
            return;
        }
        // 项目状态为待立项批复之前的状态
        Map<ProjectStateChangeEvent, ProjectStatusChange> eventMap = ctx.getProjectStatusChanges();
        if (!eventMap.containsKey(DECLARED_RECORD_SUBMIT)) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            ProjectInst inst = ctx.getProjectInstMap().get(InstTypeEnum.DECLARED_RECORD);
            if (inst == null) {
                detail.setStepStatus(StepStatusEnum.NOT_START);
            } else {
                Optional<ProjectStatusChange> event = eventMap.values().stream()
                        .filter(w -> DECLARED_RECORD_SUBMIT.eq(w.getEvent())
                                || DECLARED_RECORD_PASS.eq(w.getEvent())
                                || DECLARED_RECORD_RESUBMIT.eq(w.getEvent())
                                || DECLARED_RECORD_FAILED.eq(w.getEvent())
                        ).max(Comparator.comparing(ProjectStatusChange::getCreateOn));
                if (event.isPresent()) {
                    ProcessProgressVo progress = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
                    buildStepStatus(detail, progress, event.get());
                } else {
                    detail.setStepStatus(StepStatusEnum.NOT_START);
                }
            }
        }
        ctx.setPreStepStatus(detail.getStepStatus());
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
