package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Comparator;
import java.util.Map;
import java.util.Optional;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.*;

/**
 * 项目申报处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ProjectReviewHandle extends AbstractProcessHandle {

    private final ProcessInstanceService processInstanceService;

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_REVIEW;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        if (!StepStatusEnum.COMPLETED.equals(ctx.getPreStepStatus())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
            return;
        }
        // 项目状态为待立项批复之前的状态
        Map<ProjectStateChangeEvent, ProjectStatusChange> eventMap = ctx.getProjectStatusChanges();
        if (!eventMap.containsKey(PROJECT_REVIEW_SUBMIT)) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            Optional<ProjectStatusChange> event = eventMap.values().stream()
                    .filter(w -> PROJECT_REVIEW_PASS.eq(w.getEvent())
                            || PROJECT_REVIEW_RESUBMIT.eq(w.getEvent())
                            || PROJECT_REVIEW_FAILED.eq(w.getEvent())
                            || PROJECT_REVIEW_SUBMIT.eq(w.getEvent())
                    ).max(Comparator.comparing(ProjectStatusChange::getCreateOn));
            if (event.isPresent()) {
                ProjectStatusChange change = event.get();
                ProjectInst inst = ctx.getProjectInstMap().get(InstTypeEnum.PROJECT_REVIEW);
                if (PROJECT_REVIEW_PASS.eq(change.getEvent()) && inst == null) {
                    detail.setStepStatus(StepStatusEnum.COMPLETED);
                    detail.setFinishTime(change.getCreateOn());
                } else if (inst != null) {
                    ProcessProgressVo progress = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
                    buildStepStatus(detail, progress, change);
                } else {
                    detail.setStepStatus(StepStatusEnum.NOT_START);
                }
            } else {
                detail.setStepStatus(StepStatusEnum.NOT_START);
            }
        }
        ctx.setPreStepStatus(detail.getStepStatus());
    }

}
