package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.hz.pm.api.projectlib.model.enumeration.status.TenderSelfTestStatus.*;

/**
 * <p>
 * SelfTestHandle
 * </p>
 *
 * @author WendyYang
 * @since 09:29 2024/4/9
 */
@Component
@RequiredArgsConstructor
public class ProjectSelfTestHandle extends AbstractProcessHandle {

    private final ProcessInstanceService processInstanceService;

    @Override
    public void handle(HandleParamContext ctx) {
        if (!ctx.getPreStepStatus().equals(StepStatusEnum.COMPLETED)) {
            return;
        }
        List<Purchase> purchases = ctx.getPurchases();
        Map<Long, Purchase> purchaseMap = purchases.stream()
                .filter(w -> SELF_TEST_INFO_PASSED.eq(w.getSelfTestStatus())
                        || SELF_TEST_INFO_FAILED.eq(w.getSelfTestStatus())
                        || SELF_TEST_INFO_AUDIT.eq(w.getSelfTestStatus()))
                .collect(Collectors.toMap(Purchase::getId, Function.identity()));
        if (purchaseMap.isEmpty()) {
            return;
        }
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        detail.setProcessInfoList(new ArrayList<>());

        Map<Long, PurchaseInst> instMap = getPurchaseInstMap(ctx.getPurchaseInstList(), purchaseMap.keySet(), InstTypeEnum.SELF_TEST);
        for (Map.Entry<Long, PurchaseInst> entry : instMap.entrySet()) {
            if (detail.getFinishTime() == null || detail.getFinishTime().isBefore(entry.getValue().getCreateOn())) {
                detail.setFinishTime(entry.getValue().getCreateOn());
            }
            ProcessProgressVo instanceDetail = processInstanceService.getProgressInstanceDetail(null, entry.getValue().getInstCode());
            ProcessInfoVO tenderProgress = new ProcessInfoVO();
            tenderProgress.setProcessName(purchaseMap.get(entry.getKey()).getBidName());
            tenderProgress.setProcessProgressVo(instanceDetail);
            detail.getProcessInfoList().add(tenderProgress);
        }
        StepStatusEnum stepStatus = getPurchaseStatus(detail.getProcessInfoList());
        if (!StepStatusEnum.ON_GOING.equals(stepStatus)) {
            detail.setStepStatus(stepStatus);
            ctx.getProcessDetails().add(detail);
        }
    }

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.SYSTEM_SELF_TEST;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
