package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.hz.pm.api.projectlib.model.enumeration.status.TenderXcfhxApplyStatus.*;

/**
 * <p>
 * XcfhxReviewHandle
 * </p>
 *
 * @author WendyYang
 * @since 10:43 2024/4/9
 */
@Component
@RequiredArgsConstructor
public class XcfhxReviewHandle extends AbstractProcessHandle {

    private final ProcessInstanceService processInstanceService;

    @Override
    public void handle(HandleParamContext ctx) {
        if (!ctx.getPreStepStatus().equals(StepStatusEnum.COMPLETED)) {
            return;
        }
        List<Purchase> purchases = ctx.getPurchases();
        Map<Long, Purchase> purchaseMap = purchases.stream()
                .filter(w -> XCFHX_APPLY_PASSED.eq(w.getXcfhxApplyStatus())
                        || XCFHX_APPLY_FAILED.eq(w.getXcfhxApplyStatus())
                        || ON_XCFHX_APPLY.eq(w.getXcfhxApplyStatus()))
                .collect(Collectors.toMap(Purchase::getId, Function.identity()));
        if (purchaseMap.isEmpty()) {
            return;
        }
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        detail.setProcessInfoList(new ArrayList<>());
        ctx.getProcessDetails().add(detail);
        Map<Long, PurchaseInst> instMap = getPurchaseInstMap(ctx.getPurchaseInstList(), purchaseMap.keySet(), InstTypeEnum.XCFHX_APPLY);
        instMap.forEach((bidId, inst) -> {
            if (detail.getFinishTime() == null || detail.getFinishTime().isBefore(inst.getCreateOn())) {
                detail.setFinishTime(inst.getCreateOn());
            }
            ProcessProgressVo instDetail = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
            ProcessInfoVO tenderProgress = new ProcessInfoVO();
            tenderProgress.setProcessName(purchaseMap.get(bidId).getBidName());
            tenderProgress.setProcessProgressVo(instDetail);
            detail.getProcessInfoList().add(tenderProgress);
        });
        StepStatusEnum stepStatus = getPurchaseStatus(detail.getProcessInfoList());
        detail.setStepStatus(stepStatus);
    }

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.XCFHX_REVIEW;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }
}
