package com.hz.pm.api.projectlib.helper;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ClassUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.projectdeclared.model.entity.ProjectChangeHistory;
import com.hz.pm.api.projectdeclared.model.dto.*;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.entity.PreInsAcceptancePerson;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.vo.PreInsAcceptancePersonVO;
import com.hz.pm.api.projectdeclared.service.*;
import com.hz.pm.api.projectdeclared.utils.ProjectDetailDiffUtil.FieldDiff;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.*;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * ProjectChangeStopHelper
 * </p>
 *
 * @author WendyYang
 * @since 14:37 2024/8/5
 */
@Component
@RequiredArgsConstructor
public class ProjectChangeStopHelper {

    private final IProjectChangeHistoryService projectChangeHistoryService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPreInsAcceptancePersonService preInsAcceptancePersonService;
    private final IPurchaseService purchaseService;
    private final IOperationService operationService;
    private final IContractService contractService;
    private final IProjectService projectService;
    private final ProjectLibManage projectLibManage;

    /**
     * 根据状态变更记录重置项目、标段状态
     *
     * @param project 项目
     * @param event   event
     */
    @Transactional(rollbackFor = Exception.class)
    public void changeProjectPurchaseStatus(Project project, ProjectStateChangeEvent event) {
        TenderStateChangeEvent tenderEvent = TenderStateChangeEvent.valueOf(event.name());
        ProjectStateChangeEvent beforeEvent;
        TenderStateChangeEvent tenderBeforeEvent;
        if (event.equals(ProjectStateChangeEvent.STOPPED_APPLY_FAILED)) {
            beforeEvent = ProjectStateChangeEvent.STOPPED_APPLY_SUBMIT;
            tenderBeforeEvent = TenderStateChangeEvent.STOPPED_APPLY_SUBMIT;
        } else if (event.equals(ProjectStateChangeEvent.CHANGE_APPLY_FAILED)
                || event.equals(ProjectStateChangeEvent.CHANGE_APPLY_PASS)) {
            beforeEvent = ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT;
            tenderBeforeEvent = TenderStateChangeEvent.CHANGE_APPLY_SUBMIT;
        } else if (event.equals(ProjectStateChangeEvent.STOPPED_APPLY_SUBMIT)
                || event.equals(ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT)
                || event.equals(ProjectStateChangeEvent.STOPPED_APPLY_PASS)
                || event.equals(ProjectStateChangeEvent.STOPPED_APPLY_AUTO_PASS)) {
            // 提交的时候不重置项目、标项状态
            beforeEvent = null;
            tenderBeforeEvent = null;
            ProjectStatus status;
            TenderMainStatus tenderStatus;
            if (event.equals(ProjectStateChangeEvent.STOPPED_APPLY_SUBMIT)) {
                status = ProjectStatus.ON_STOPPED_APPLY;
                tenderStatus = TenderMainStatus.ON_STOPPED_APPLY;
                project.setStoppedStatus(buildBizStatus(event).getCode());
            } else if (event.equals(ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT)) {
                status = ProjectStatus.ON_CHANGE_APPLY;
                tenderStatus = TenderMainStatus.ON_CHANGE_APPLY;
            } else {
                status = ProjectStatus.STOPPED_PASSED;
                tenderStatus = TenderMainStatus.STOPPED_PASSED;
                project.setStoppedStatus(buildBizStatus(event).getCode());
            }
            ProjectStatusChange projectStatusChange = ProjectStatusChange.builder()
                    .event(event.name())
                    .projectId(project.getId())
                    .event(event.name())
                    .projectCode(project.getProjectCode())
                    .beforeStatus(project.getStatus())
                    .afterStatus(status.getCode())
                    .build();
            projectStatusChangeService.save(projectStatusChange);
            project.setStatus(status.getCode());
            project.setStage(status.getStage());
            projectService.updateById(project);
            List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
            List<PurchaseStatusChange> purchaseStatusChanges = CollUtils.convert(purchases,
                    purchase -> PurchaseStatusChange.builder()
                            .bidId(purchase.getId())
                            .event(tenderEvent.name())
                            .projectId(project.getId())
                            .projectCode(project.getProjectCode())
                            .beforeStatus(purchase.getStatus())
                            .afterStatus(tenderStatus.getCode())
                            .build());
            purchaseStatusChangeService.saveBatch(purchaseStatusChanges);
        } else {
            throw new BizException("不支持的状态转换");
        }
        resetProjectPurchaseStatus(project.getId(), project.getProjectCode(), event, beforeEvent, tenderBeforeEvent, tenderEvent);
    }

    private void resetProjectPurchaseStatus(Long projectId, String projectCode,
                                            ProjectStateChangeEvent projEvent,
                                            ProjectStateChangeEvent projBeforeEvent,
                                            TenderStateChangeEvent tenderBeforeEvent,
                                            TenderStateChangeEvent tenderEvent) {
        LambdaUpdateWrapper<Project> projectWrapper = Wrappers.lambdaUpdate(Project.class)
                .eq(Project::getId, projectId);
        if (projBeforeEvent != null) {
            if (projEvent.equals(ProjectStateChangeEvent.STOPPED_APPLY_FAILED)) {
                projectWrapper.set(Project::getStoppedStatus, buildBizStatus(projEvent).getCode());
            } else {
                Wrapper<ProjectChangeHistory> changeHistoryWrapper = Wrappers.lambdaQuery(ProjectChangeHistory.class)
                        .eq(ProjectChangeHistory::getProjectCode, projectCode)
                        .orderByDesc(ProjectChangeHistory::getCreateOn)
                        .last(BizConst.LIMIT_1);
                ProjectChangeHistory history = projectChangeHistoryService.getOne(changeHistoryWrapper);
                history.setStatus(buildBizStatus(projEvent).getCode());
                projectChangeHistoryService.updateById(history);
                if (projEvent.equals(ProjectStateChangeEvent.CHANGE_APPLY_PASS)) {
                    EditProjectDetailDTO editProjectDetail = JSONUtil.toBean(history.getChangeContent(), EditProjectDetailDTO.class);
                    projectEditRewrite(projectCode, editProjectDetail);
                }
            }
            ProjectStatusChange change = projectStatusChangeService.getLastOne(projBeforeEvent, projectId);
            projectStatusChangeService.save(ProjectStatusChange.builder()
                    .event(projEvent.name())
                    .projectId(projectId)
                    .event(projEvent.name())
                    .projectCode(projectCode)
                    .beforeStatus(change.getAfterStatus())
                    .afterStatus(change.getBeforeStatus())
                    .build());
            Integer currStatus = change.getBeforeStatus();
            projectWrapper.set(Project::getStatus, currStatus);
            projectWrapper.set(Project::getStage, ProjectStatus.getNoNull(currStatus).getStage());
            projectService.update(projectWrapper);
            List<Purchase> purchases = purchaseService.listByProjectId(projectId);
            if (!purchases.isEmpty()) {
                Wrapper<PurchaseStatusChange> purchaseStatusChangeQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                        .eq(PurchaseStatusChange::getProjectId, projectId)
                        .eq(PurchaseStatusChange::getEvent, tenderBeforeEvent);
                List<PurchaseStatusChange> purchaseStatusChanges = purchaseStatusChangeService.list(purchaseStatusChangeQuery);
                Map<Long, PurchaseStatusChange> purchaseStatusChangeMap = BizUtils.groupFirstMap(purchaseStatusChanges,
                        PurchaseStatusChange::getBidId, Comparator.comparing(PurchaseStatusChange::getCreateOn).reversed());
                List<PurchaseStatusChange> changeArrayList = new ArrayList<>();
                purchases.forEach(purchase -> {
                    PurchaseStatusChange currChange = purchaseStatusChangeMap.get(purchase.getId());
                    purchase.setStatus(currChange.getBeforeStatus());
                    changeArrayList.add(PurchaseStatusChange.builder()
                            .afterStatus(currChange.getBeforeStatus())
                            .beforeStatus(currChange.getAfterStatus())
                            .bidId(purchase.getId())
                            .event(tenderEvent.name())
                            .projectId(projectId)
                            .projectCode(projectCode)
                            .build());
                });
                purchaseService.updateBatchById(purchases);
                purchaseStatusChangeService.saveBatch(changeArrayList);
            }
        }
    }

    private IStatus<Integer, String> buildBizStatus(ProjectStateChangeEvent event) {
        IStatus<Integer, String> bizStatus;
        switch (event) {
            case STOPPED_APPLY_PASS:
                bizStatus = ProjectStoppedStatus.STOPPED_PASSED;
                break;
            case STOPPED_APPLY_AUTO_PASS:
                bizStatus = ProjectStoppedStatus.STOPPED_AUTO_PASSED;
                break;
            case STOPPED_APPLY_SUBMIT:
                bizStatus = ProjectStoppedStatus.ON_STOPPED_APPLY;
                break;
            case STOPPED_APPLY_FAILED:
                bizStatus = ProjectStoppedStatus.STOPPED_FAILED;
                break;
            case CHANGE_APPLY_SUBMIT:
                bizStatus = ProjectChangeStatus.ON_CHANGE_APPLY;
                break;
            case CHANGE_APPLY_PASS:
                bizStatus = ProjectChangeStatus.CHANGE_PASSED;
                break;
            case CHANGE_APPLY_FAILED:
                bizStatus = ProjectChangeStatus.CHANGE_FAILED;
                break;
            default:
                bizStatus = null;
                break;
        }
        return bizStatus;
    }

    private void projectEditRewrite(String projectCode, EditProjectDetailDTO projectEdit) {
        List<FieldDiff> projectDiffs = projectEdit.getProjectDiffs();
        if (CollUtil.isNotEmpty(projectDiffs)) {
            EditProjectDTO projectDetail = projectEdit.getProject();
            FieldDiff systemReplaceInfos = CollUtil.findOne(projectDiffs,
                    w -> w.getFieldName().equals(EditProjectDTO.SYSTEM_REPLACE_INFOS));
            if (systemReplaceInfos != null) {
                projectLibManage.saveGovSystemReplaceInfos(projectDetail.getId(),
                        projectCode, projectDetail.getSystemReplaceInfos());
            }
            UpdateWrapper<Project> wrapper = buildWrapper(projectDiffs, Project.class);
            if (wrapper != null) {
                wrapper.eq("id", projectDetail.getId());
                projectService.update(wrapper);
            }
        }
        List<EditPurchaseDetailDTO> purchases = projectEdit.getPurchases();
        if (CollUtil.isNotEmpty(purchases)) {
            for (EditPurchaseDetailDTO purchaseEdit : purchases) {
                // 初验人员保存
                if (purchaseEdit.getFirstInspectedPersonsDiff() != null) {
                    Long purchaseId = purchaseEdit.getPurchase().getId();
                    preInsAcceptancePersonService.removeByBidId(purchaseId);
                    List<PreInsAcceptancePersonVO> firstInspectedPersons = purchaseEdit.getFirstInspectedPersons();
                    if (CollUtil.isNotEmpty(firstInspectedPersons)) {
                        List<PreInsAcceptancePerson> persons = firstInspectedPersons.stream().map(d -> {
                            PreInsAcceptancePerson person = new PreInsAcceptancePerson();
                            person.setPersonName(d.getPersonName());
                            person.setUnit(d.getUnit());
                            person.setBidId(purchaseId);
                            person.setProjectId(d.getProjectId());
                            return person;
                        }).collect(Collectors.toList());
                        preInsAcceptancePersonService.saveBatch(persons);
                    }
                }
                modifyPurchase(purchaseEdit);
                modifyContract(purchaseEdit);
                modifyOperation(purchaseEdit);
            }
        }
    }

    private void modifyPurchase(EditPurchaseDetailDTO purchaseEdit) {
        List<FieldDiff> purchaseDiffs = purchaseEdit.getPurchaseDiffs();
        if (CollUtil.isNotEmpty(purchaseDiffs)) {
            EditPurchaseDTO purchaseDetail = purchaseEdit.getPurchase();
            UpdateWrapper<Purchase> wrapper = buildWrapper(purchaseDiffs, Purchase.class);
            if (wrapper != null) {
                wrapper.eq("id", purchaseDetail.getId());
                purchaseService.update(wrapper);
            }
        }
    }

    private void modifyContract(EditPurchaseDetailDTO purchaseEdit) {
        List<FieldDiff> contractDiffs = purchaseEdit.getContractDiffs();
        if (CollUtil.isNotEmpty(contractDiffs)) {
            EditContractDTO contractDetail = purchaseEdit.getContract();
            UpdateWrapper<Contract> wrapper = buildWrapper(contractDiffs, Contract.class);
            if (wrapper != null) {
                wrapper.eq("id", contractDetail.getId());
                contractService.update(wrapper);
            }
        }
    }

    private void modifyOperation(EditPurchaseDetailDTO purchaseEdit) {
        List<FieldDiff> operationDiffs = purchaseEdit.getOperationDiffs();
        if (CollUtil.isNotEmpty(operationDiffs)) {
            EditOperationDTO operationDetail = purchaseEdit.getOperation();
            UpdateWrapper<Operation> wrapper = buildWrapper(operationDiffs, Operation.class);
            if (wrapper != null) {
                wrapper.eq("id", operationDetail.getId());
                operationService.update(wrapper);
            }
        }
    }

    private <T> UpdateWrapper<T> buildWrapper(List<FieldDiff> diffs, Class<T> clazz) {
        List<String> fields = Arrays.stream(ClassUtil.getDeclaredFields(clazz))
                .map(Field::getName)
                .collect(Collectors.toList());
        UpdateWrapper<T> wrapper = Wrappers.update();
        boolean hasFieldChanged = false;
        for (FieldDiff fieldDiff : diffs) {
            String fieldName = fieldDiff.getFieldName();
            Object newValue = fieldDiff.getNewValue();
            if (fields.contains(fieldName)) {
                hasFieldChanged = true;
                wrapper.set(StrUtil.toUnderlineCase(fieldName), newValue);
            }
        }
        return hasFieldChanged ? wrapper : null;
    }

}
