package com.hz.pm.api.projectlib.model.enumeration;

import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.Objects;
import java.util.Optional;

/**
 * 实例类型枚举
 *
 * @author CMM
 * @since 2023/02/24 16:14
 */
@Getter
@NoArgsConstructor
@AllArgsConstructor
public enum InstTypeEnum {
    /**
     * 流程实例类型
     */
    UNIT_INNER_AUDIT(1, "单位内部审批流程"),
    PRELIMINARY_PREVIEW(2, "项目预审审批流程"),
    DEPT_UNITED_REVIEW(3, "部门联合审批流程"),
    CONSTRUCTION_PLAN_REVIEW(4, "建设方案审批流程"),
    PROJECT_FINAL_INSPECTION(5, "验收申报审批流程"),

    APPLY_DELAY(6, "申请延期审批流程"),
    DECLARED_RECORD(8, "立项备案审批流程"),
    XCFHX_APPLY(9, "信创审查审批流程"),

    APPLY_BORROW(7, "申请借阅审批流程"),
    SELF_TEST(10, "系统自测审批流程"),
    ADAPTION(11, "开工文件审批流程"),
    TEST_VALID(12, "监理核实验证审批流程"),
    COMPLIANCE_REVIEW(13, "合规性审查流程"),
    PROJECT_REVIEW(14, "项目评审流程"),
    PROJECT_STOPPED(15, "项目终止流程"),
    PROJECT_CHANGE(16, "项目变更流程");

    private Integer code;
    private String desc;

    public static String getDesc(Integer code) {
        Optional<InstTypeEnum> instType = get(code);
        return instType.flatMap(w -> Optional.of(w.getDesc())).orElse(StringUtils.EMPTY);
    }

    private static Optional<InstTypeEnum> get(Integer code) {
        return Arrays.stream(values())
                .filter(w -> w.eq(code))
                .findFirst();
    }

    public static InstTypeEnum getNoNull(Integer code) {
        return Arrays.stream(values())
                .filter(w -> w.eq(code))
                .findFirst()
                .orElseThrow(() -> BizException.wrap("未找到对应的实例类型"));
    }

    public boolean eq(Integer code) {
        return this.getCode().equals(code);
    }

}
