package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * SelfTestStatusEnum
 * </p>
 *
 * @author WendyYang
 * @since 10:16 2024/3/27
 */
@Getter
@AllArgsConstructor
public enum TenderAdaptStatus implements ITenderStatus {

    WITHOUT_ADAPT_INFO(null, 2100, "待上传开工文件"),
    ADAPT_INFO_AUDIT(null, 2101, "开工文件审核中"),
    ADAPT_INFO_FAILED(null, 2102, "开工文件审核失败"),
    ADAPT_INFO_PASSED(null, 2103, "开工文件审核通过");

    private final ProjectStatus projectStatus;
    private final Integer code;
    private final String desc;

    private static Optional<TenderAdaptStatus> get(Integer code) {
        return Arrays.stream(values()).filter(w -> w.eq(code)).findFirst();
    }

    public static TenderAdaptStatus getNoNull(Integer code) {
        return get(code).orElseThrow(() -> BizException.wrap("无效的开工文件状态：%s", code));
    }

    public static String getDesc(Integer code) {
        return get(code).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }

}
