package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 标段状态
 * </p>
 *
 * @author WendyYang
 * @since 15:49 2024/2/23
 */
@Getter
@AllArgsConstructor
public enum TenderMainStatus implements ITenderStatus {

    TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO(ProjectStatus.ON_PURCHASING, 101, "待填写采购&合同信息"),
    PURCHASE_FINISHED(ProjectStatus.ON_PURCHASING, 106, "采购完成"),
    TO_BE_SUBMIT_OPERATION_PLAN(ProjectStatus.ON_ADAPTING, 103, "待填写实施计划"),
    // 衔接状态
    WAIT_ORG_CONFIRM(ProjectStatus.ON_ADAPTING, 104, "待单位确认"),
    ORG_CONFIRM_PASSED(ProjectStatus.ON_ADAPTING, 105, "单位已确认"),
    TO_BE_SUBMIT_FIRST_INSPECTED_INFO(ProjectStatus.TO_BE_FIRST_INSPECTED, 201, "待上传初验材料"),
    SUBMITTED_FIRST_INSPECTED_INFO(ProjectStatus.TO_BE_FIRST_INSPECTED, 202, "已上传初验材料"),
    TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY(ProjectStatus.ON_PILOT_RUNNING, 401, "待终验申报"),
    ON_FINALLY_INSPECTED_APPLY(ProjectStatus.ON_FINALLY_INSPECTED, 501, "终验申报中"),
    FINALLY_INSPECTED_FAILED(ProjectStatus.ON_FINALLY_INSPECTED, 502, "终验申报不通过"),
    FINALLY_INSPECTED_PASSED(ProjectStatus.ACCEPTED, 601, "终验完成"),
    ON_CHANGE_APPLY(ProjectStatus.ON_CHANGE_APPLY, 701, "项目变更中"),
    ON_STOPPED_APPLY(ProjectStatus.ON_STOPPED_APPLY, 901, "项目终止审核中"),
    STOPPED_PASSED(ProjectStatus.STOPPED, 902, "项目终止");

    private final ProjectStatus projectStatus;
    private final Integer code;
    private final String desc;


    private static Optional<ITenderStatus> get(Integer status) {
        return Optional.ofNullable(TENDER_STATUS_MAP.get(status));
    }

    public static ITenderStatus getNoNull(Integer status) {
        return get(status).orElseThrow(() -> BizException.wrap("无效的标段状态：%s", status));
    }

    public static String getDesc(Integer status) {
        return get(status).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }

    private static final Map<Integer, ITenderStatus> TENDER_STATUS_MAP;

    static {
        List<ITenderStatus> mainStatusList = Arrays.asList(TenderMainStatus.values());
        List<ITenderStatus> adaptStatusList = Arrays.asList(TenderAdaptStatus.values());
        List<ITenderStatus> selfTestStatusList = Arrays.asList(TenderSelfTestStatus.values());
        List<ITenderStatus> testValidStatusList = Arrays.asList(TenderTestValidStatus.values());
        TENDER_STATUS_MAP = Stream.of(mainStatusList, adaptStatusList, selfTestStatusList, testValidStatusList)
                .flatMap(List::stream)
                .collect(Collectors.toMap(IStatus::getCode, w -> w));

    }

    public static void main(String[] args) {
        TENDER_STATUS_MAP.forEach((k, v) -> System.out.println(k + ":" + v));
    }

}
