package com.hz.pm.api.projectlib.model.req;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.DecimalMax;
import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * <p>
 * 立项批复参数类
 * </p>
 *
 * @author WendyYang
 * @since 20:04 2023/2/12
 */
@Data
@ApiModel("立项批复参数类")
public class ProjectApprovedReq {

    @ApiModelProperty("项目ID")
    @NotNull(message = "项目ID不能为空")
    private Long projectId;

    @ApiModelProperty("批复总投资")
    @NotNull(message = "批复总投资不能为空")
    @DecimalMax(value = "999999999.99", message = "批复总投资无效")
    @DecimalMin(value = "0.01", message = "批复总投资无效")
    private BigDecimal approvedAmount;

    @ApiModelProperty("批复总预算")
    @NotNull(message = "批复总预算不能为空")
    @DecimalMax(value = "999999999.99", message = "批复总预算无效")
    @DecimalMin(value = "0.01", message = "批复总预算无效")
    private BigDecimal approvalBudget;


    @NotNull(message = "批复时间不能为空")
    @JsonFormat(pattern = "yyyy-MM-dd")
    @ApiModelProperty("批复时间")
    private LocalDateTime approvedDate;

    @ApiModelProperty("建设周期")
    @NotNull(message = "建设周期不能为空")
    @Range(min = 1, max = 99999, message = "建设周期无效")
    private Integer buildCycle;

    @ApiModelProperty("批复文件ID")
    @NotNull(message = "批复文件不能为空")
    private Long approvedFileId;

    @ApiModelProperty("建设方案文件ID")
    private Long buildPlanFileId;

}
