package com.hz.pm.api.projectlib.model.req;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.hz.pm.api.common.enumeration.ExportOptionEnum;
import com.ningdatech.basic.model.PagePo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;

/**
 * <p>
 * ProjectLibListReq
 * </p>
 *
 * @author WendyYang
 * @since 14:35 2023/2/1
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@ApiModel("项目列表查询")
@EqualsAndHashCode(callSuper = true)
public class ProjectListReq extends PagePo {

    private Long id;

    @ApiModelProperty("项目名称")
    private String projectName;

    @ApiModelProperty("申报单位")
    private String buildOrg;

    @ApiModelProperty("申报单位code")
    private String buildOrgCode;

    @ApiModelProperty("申报单位所属领域")
    private Integer unitStrip;

    @ApiModelProperty("申报单位名")
    private String buildOrgName;

    @ApiModelProperty("主管单位code")
    private String superOrgCode;

    @ApiModelProperty("项目类型 老的 1建设2运维")
    private Integer oldProjectType;

    @ApiModelProperty("项目类型")
    private String projectType;

    @ApiModelProperty("预算年度")
    private Integer projectYear;

    @ApiModelProperty("项目阶段")
    private Integer stage;

    @ApiModelProperty("项目状态")
    private Integer status;

    @ApiModelProperty("申报金额")
    private BigDecimal declareAmountMin;

    @ApiModelProperty("申报金额")
    private BigDecimal declareAmountMax;

    @ApiModelProperty("批复金额")
    private BigDecimal approveAmountMin;

    @ApiModelProperty("批复金额")
    private BigDecimal approveAmountMax;

    @ApiModelProperty("创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    private LocalDateTime createOnMin;

    @ApiModelProperty("创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    private LocalDateTime createOnMax;

    @ApiModelProperty("用户ID")
    private Long userId;

    @ApiModelProperty("是否临时增补 0:否 1:是")
    private Integer isTemporaryAugment;

    @ApiModelProperty("项目阶段 多个")
    private List<Integer> stageList;

    @ApiModelProperty("项目状态 多个")
    private List<Integer> statusList;

    @ApiModelProperty("实例code 多个")
    private List<String> instCodes;

    @ApiModelProperty("项目code 多个")
    private List<String> projectCodes;

    @ApiModelProperty("导出选项")
    private List<ExportOptionEnum> exportOptionList;

    private String userType;
    private String userValue;

    private Collection<String> regionCodes;

    @ApiModelProperty("区域编码")
    private String userRegionCode;

    @ApiModelProperty("评审金额最小值")
    private BigDecimal reviewAmountMin;

    @ApiModelProperty("评审金额最大值")
    private BigDecimal reviewAmountMax;

    private Boolean isStartDeclaredProject;

    @ApiModelProperty("是否资金已下达")
    private Boolean isAmountApproval;

    private Long declaredUnitId;

    private List<Long> declaredUnitIds;

    @ApiModelProperty("标项状态")
    private Integer bidStatus;

    @ApiModelProperty("tab状态")
    private Integer tabStatus;

    public Long clearBuildOrgCode() {
        String tmpBuildOrgCode = this.getBuildOrgCode();
        if (StrUtil.isNotBlank(tmpBuildOrgCode)) {
            this.setBuildOrgCode(null);
        }
        return NumberUtil.parseLong(tmpBuildOrgCode, null);
    }

}
