package com.hz.pm.api.projectlib.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.BooleanBit;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.mapper.ProjectMapper;
import com.hz.pm.api.projectlib.model.dto.ProjectChangeListDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.po.ProjectPO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.service.IProjectService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author WendyYang
 * @since 2023-02-03
 */
@Service
@RequiredArgsConstructor
public class ProjectServiceImpl extends ServiceImpl<ProjectMapper, Project> implements IProjectService {

    @Override
    public Project getNewProject(Long projectId) {
        return getById(ProjectIdCodeCacheUtil.newest(projectId));
    }

    @Override
    public Project getNewestNoNull(Long projectId) {
        return Objects.requireNonNull(getNewProject(projectId), "项目不存在");
    }

    @Override
    public Project getProjectByCode(String projectCode) {
        return this.getOne(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectCode, projectCode)
                .eq(Project::getNewest, BooleanBit.TRUE)
                .last(BizConst.LIMIT_1));
    }

    @Override
    public Project getNoNull(String projectCode) {
        return Objects.requireNonNull(getProjectByCode(projectCode), "项目不存在");
    }

    @Override
    public Page<ProjectPO> pageAllWithPermission(Page<ProjectPO> page, ProjectListReq req) {
        return baseMapper.pageAllWithPermission(page, req);
    }

    @Override
    public Page<ProjectPO> pageLibWithPermission(Page<ProjectPO> page, ProjectListReq req) {
        return baseMapper.pageLibWithPermission(page, req);
    }

    @Override
    public Page<ProjectChangeListDTO> pageChangeList(Wrapper<Project> wrapper, Page<ProjectChangeListDTO> page) {
        return baseMapper.pageChangeList(page, wrapper);
    }

}
