package com.hz.pm.api.scheduler.listener;

import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.helper.WflowHelper;
import com.hz.pm.api.common.util.ThreadPoolUtil;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.model.dto.MhZwddWorkNoticeDTO;
import com.hz.pm.api.external.todo.MHTodoClient;
import com.hz.pm.api.external.todo.dto.MhTodoExtraParamDTO;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.model.entity.Xinchuang;
import com.hz.pm.api.projectdeclared.model.entity.XinchuangInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IXinchuangInstService;
import com.hz.pm.api.projectdeclared.service.IXinchuangService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModelHistorys;
import com.wflow.workflow.service.ProcessModelHistorysService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.flowable.common.engine.api.delegate.event.FlowableEngineEntityEvent;
import org.flowable.common.engine.api.delegate.event.FlowableEngineEventType;
import org.flowable.common.engine.api.delegate.event.FlowableEvent;
import org.flowable.common.engine.api.delegate.event.FlowableEventListener;
import org.flowable.task.api.history.HistoricTaskInstance;
import org.flowable.task.service.impl.persistence.entity.TaskEntity;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * TaskCreatedListener
 * </p>
 *
 * @author WendyYang
 * @since 10:38 2024/3/12
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class TaskCreatedListener implements FlowableEventListener {

    private final IProjectService projectService;
    private final IProjectInstService projectInstService;
    private final ProcessModelHistorysService processModelHistorysService;
    private final MHTodoClient mhTodoClient;
    private final MhApiClient mhApiClient;
    private final UserInfoHelper userInfoHelper;
    private final IPurchaseInstService purchaseInstService;
    private final IXinchuangInstService xinchuangInstService;
    private final IXinchuangService xinchuangService;

    @Override
    public void onEvent(FlowableEvent event) {
        Instant instant = Instant.now().plusSeconds(5);
        ThreadPoolUtil.SCHEDULER.schedule(() -> handleEvent(event), instant);
    }

    private void handleEvent(FlowableEvent event) {
        if (event.getType().equals(FlowableEngineEventType.TASK_CREATED)) {
            FlowableEngineEntityEvent entityEvent = (FlowableEngineEntityEvent) event;
            String procDefId = entityEvent.getProcessDefinitionId();
            String procInstId = entityEvent.getProcessInstanceId();
            WflowModelHistorys models = processModelHistorysService.getByProcessDefId(procDefId);
            ProjectProcessType procType = ProjectProcessType.getNoNull(models.getProcessType());
            TaskEntity taskEntity = (TaskEntity) entityEvent.getEntity();
            UserFullInfoDTO assignee = userInfoHelper.getUserFullInfo(taskEntity.getAssignee());
            if (assignee == null || StrUtil.isBlank(assignee.getMhUserId())) {
                return;
            }
            switch (procType) {
                case XC_APPROVAL_PROCESS:
                case PROJECT_RECORD_APPROVAL_PROCESS:
                case ACCEPTANCE_DECLARATION_APPROVAL_PROCESS:
                case ADAPTION:
                case SELF_TEST:
                case TEST_VALID:
                case PROJECT_CHANGE:
                case PROJECT_STOPPED:
                case COMPLIANCE_REVIEW:
                case PROJECT_REVIEW:
                    // 发送信产平台待办
                    MHTodoTypeEnum todoType = getMhTodoTypeEnum(procType);
                    MhTodoExtraParamDTO paramObj = MhTodoExtraParamDTO.builder()
                            .taskId(taskEntity.getId())
                            .instanceId(taskEntity.getProcessInstanceId())
                            .nodeId(taskEntity.getTaskDefinitionId())
                            .userId(assignee.getUserId())
                            .userName(assignee.getRealName())
                            .path("/toDoCenter/handleDuringExamine")
                            .build();
                    if (procType.equals(ProjectProcessType.ACCEPTANCE_DECLARATION_APPROVAL_PROCESS)
                            || procType.equals(ProjectProcessType.ADAPTION)
                            || procType.equals(ProjectProcessType.XC_APPROVAL_PROCESS)
                            || procType.equals(ProjectProcessType.TEST_VALID)
                            || procType.equals(ProjectProcessType.SELF_TEST)) {
                        PurchaseInst purchaseInst = purchaseInstService.getByProjectIdAndInstCode(paramObj.getProjectId(), procInstId);
                        paramObj.setBidId(purchaseInst.getBidId());
                    }
                    String projectName;
                    // 上边查不到ID的时候说明是手动新增的信创项目
                    if (paramObj.getBidId() == null) {
                        XinchuangInst xinchuangInst = xinchuangInstService.getByInstCode(procInstId);
                        if (xinchuangInst == null) {
                            return;
                        }
                        Xinchuang xinchuang = xinchuangService.getById(xinchuangInst.getXinchuangId());
                        paramObj.setXinchuangId(xinchuangInst.getId());
                        projectName = xinchuang.getProjectName();
                    } else {
                        ProjectInst projectInst = projectInstService.getByInstCode(procInstId);
                        if (projectInst == null) {
                            return;
                        }
                        Project project = projectService.getById(projectInst.getProjectId());
                        projectName = project.getProjectName();
                        paramObj.setProjectId(project.getId());
                        paramObj.setProjectCode(project.getProjectCode());
                    }
                    String content = String.format("【%s】的【%s】需要您审核，请及时处理。",
                            projectName, procType.getDesc());
                    mhTodoClient.addTodo(paramObj, assignee, todoType, taskEntity.getId(), content);
                    // 发送浙政钉工作通知
                    if (assignee.getAccountId() != null) {
                        MhZwddWorkNoticeDTO notice = MhZwddWorkNoticeDTO.builder()
                                .content(content)
                                .title(todoType.getVal())
                                .targetUser(assignee.getMhUserId())
                                .build();
                        mhApiClient.sendZwddWorkNotice(notice);
                    }
                    break;
                default:
                    break;
            }
        } else if (event.getType().equals(FlowableEngineEventType.TASK_COMPLETED)) {
            FlowableEngineEntityEvent entityEvent = (FlowableEngineEntityEvent) event;
            String procDefId = entityEvent.getProcessDefinitionId();
            WflowModelHistorys models = processModelHistorysService.getByProcessDefId(procDefId);
            ProjectProcessType procType = ProjectProcessType.getNoNull(models.getProcessType());
            TaskEntity taskEntity = (TaskEntity) entityEvent.getEntity();
            WflowHelper wflowHelper = SpringUtil.getBean(WflowHelper.class);
            String taskDefKey = taskEntity.getTaskDefinitionKey();
            boolean orUserTask = wflowHelper.isOrUserTask(procDefId, taskDefKey);
            switch (procType) {
                case XC_APPROVAL_PROCESS:
                case PROJECT_RECORD_APPROVAL_PROCESS:
                case ACCEPTANCE_DECLARATION_APPROVAL_PROCESS:
                case ADAPTION:
                case SELF_TEST:
                case TEST_VALID:
                case PROJECT_CHANGE:
                case PROJECT_STOPPED:
                case COMPLIANCE_REVIEW:
                case PROJECT_REVIEW:
                    MHTodoTypeEnum todoType = getMhTodoTypeEnum(procType);
                    if (orUserTask) {
                        List<HistoricTaskInstance> tasks = wflowHelper.listFinishedTasks(taskDefKey);
                        Set<String> taskIds = CollUtils.fieldSet(tasks, HistoricTaskInstance::getId);
                        taskIds.add(taskEntity.getId());
                        mhTodoClient.completeTodo(todoType, taskIds);
                    } else {
                        mhTodoClient.completeTodo(todoType, taskEntity.getId());
                    }
                    break;
                default:
                    break;
            }
        }
    }

    private MHTodoTypeEnum getMhTodoTypeEnum(ProjectProcessType procType) {
        MHTodoTypeEnum todoType;
        switch (procType) {
            case XC_APPROVAL_PROCESS:
                todoType = MHTodoTypeEnum.XCFHX_INSPECT_AUDIT;
                break;
            case PROJECT_RECORD_APPROVAL_PROCESS:
                todoType = MHTodoTypeEnum.DECLARED_RECORD_AUDIT;
                break;
            case PROJECT_REVIEW:
                todoType = MHTodoTypeEnum.PROJECT_REVIEW_AUDIT;
                break;
            case COMPLIANCE_REVIEW:
                todoType = MHTodoTypeEnum.COMPLIANCE_REVIEW_AUDIT;
                break;
            case PROJECT_STOPPED:
                todoType = MHTodoTypeEnum.PROJECT_STOPPED_AUDIT;
                break;
            case TEST_VALID:
                todoType = MHTodoTypeEnum.TEST_VALID_AUDIT;
                break;
            case PROJECT_CHANGE:
                todoType = MHTodoTypeEnum.PROJECT_CHANGE_AUDIT;
                break;
            case ADAPTION:
                todoType = MHTodoTypeEnum.ADAPTION_AUDIT;
                break;
            case SELF_TEST:
                todoType = MHTodoTypeEnum.SELF_TEST_AUDIT;
                break;
            case ACCEPTANCE_DECLARATION_APPROVAL_PROCESS:
                todoType = MHTodoTypeEnum.FINAL_INSPECT_AUDIT;
                break;
            default:
                throw BizException.wrap("未知待办类型：" + procType);
        }
        return todoType;
    }

    @Override
    public boolean isFailOnException() {
        return false;
    }

    @Override
    public boolean isFireOnTransactionLifecycleEvent() {
        return false;
    }

    @Override
    public String getOnTransaction() {
        return null;
    }
}
