package com.hz.pm.api.scheduler.task;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.wflow.bean.dto.MajorProjectRuleDTO;
import com.wflow.bean.entity.WflowEarlyWarning;
import com.wflow.enums.WarningRuleTypeEnum;
import com.wflow.service.IEarlyWarningService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

/**
 * <p>
 * MajorProjectEvalTask
 * </p>
 *
 * @author WendyYang
 * @since 11:29 2024/9/10
 */
@Slf4j
@Component
@ConditionalOnExpression("${major-project-eval.open:true}")
@RequiredArgsConstructor
public class MajorProjectEvalTask {

    private final IEarlyWarningService earlyWarningService;
    private final IProjectService projectService;
    private final MhUnitCache mhUnitCache;

    @Scheduled(cron = "${major-project-eval.cron:0 0/3 * * * ?}")
    public void doMajorProjectEval() {
        Wrapper<WflowEarlyWarning> query = Wrappers.lambdaQuery(WflowEarlyWarning.class)
                .eq(WflowEarlyWarning::getRuleType, WarningRuleTypeEnum.MAJOR_PROJECT.getCode())
                .eq(WflowEarlyWarning::getIsOpen, Boolean.TRUE);
        List<WflowEarlyWarning> earlyWarnings = earlyWarningService.list(query);
        if (earlyWarnings.isEmpty()) {
            return;
        }
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getReviewAmount, Project::getBuildOrgCode)
                .isNotNull(Project::getReviewAmount)
                .gt(Project::getReviewAmount, 0)
                .eq(Project::getNewest, Boolean.TRUE));
        List<Project> projectsUpdate = new ArrayList<>();
        if (!projects.isEmpty()) {
            for (Project project : projects) {
                long unitId = Long.parseLong(project.getBuildOrgCode());
                List<Long> unitIdPaths = mhUnitCache.getUnitIdPaths(unitId);
                Optional<WflowEarlyWarning> earlyWarning = earlyWarnings.stream()
                        .filter(warning -> unitIdPaths.contains(warning.getMhUnitId()))
                        .max(Comparator.comparing(w -> unitIdPaths.indexOf(w.getMhUnitId())));
                earlyWarning.ifPresent(warning -> {
                    MajorProjectRuleDTO rule = JSONUtil.toBean(warning.getRule(), MajorProjectRuleDTO.class);
                    boolean isMajorProject = project.getReviewAmount().compareTo(rule.getMinimumReviewAmount()) >= 0;
                    project.setIsMajorProject(isMajorProject);
                    projectsUpdate.add(project);
                });
            }
            projectService.updateBatchById(projectsUpdate);
        }
    }

}
