package com.hz.pm.api.scheduler.task;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.staging.contants.StagingContant;
import com.hz.pm.api.staging.model.entity.WorkNoticeStaging;
import com.hz.pm.api.staging.service.INdWorkNoticeStagingService;
import com.hz.pm.api.todocenter.bean.entity.WorkNoticeInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 发送工作通知定时任务
 *
 * @author CMM
 * @since 2023/02/28 21:23
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class WorkNoticeFlowTask {

    private final INdWorkNoticeStagingService workNoticeStagingService;

    @Value("${hostname:}")
    private String HOST_NAME;

    @Scheduled(cron = "0 */1 * * * ?")
    public void statusFlow() throws UnknownHostException {
        //测试暂时用自己电脑HOST
        if (HOST_NAME.equals(InetAddress.getLocalHost().getHostName())) {
            //1. 定时取 工作通知暂存表的数据进行发送
            List<WorkNoticeStaging> stagingList = workNoticeStagingService.list(Wrappers.lambdaQuery(WorkNoticeStaging.class)
                    .eq(WorkNoticeStaging::getDead, Boolean.FALSE)
                    .le(WorkNoticeStaging::getNextTime, LocalDateTime.now())
                    .le(WorkNoticeStaging::getRetryTimes, StagingContant.Retry.MAX_RETRY_TIMES)
                    .orderByAsc(WorkNoticeStaging::getId));

            log.info("需要发送的工作通知 size:{} :{}", stagingList.size(), JSON.toJSONString(stagingList));
            if (CollUtil.isEmpty(stagingList)) {
                log.info("没有需要发送的工作通知!");
                return;
            }
            //遍历
            for (WorkNoticeStaging workNoticeStaging : stagingList) {
                try {
                    WorkNoticeInfo workNoticeInfo = new WorkNoticeInfo();
                    BeanUtils.copyProperties(workNoticeStaging, workNoticeInfo);
                } catch (Exception e) {
                    log.error("发送工作通知 异常 bizMsgId:【" + workNoticeStaging.getBizMsgId() + "】 异常内容:" + e);
                } finally {
                    //增加重试的次数 和下次扫描时间
                    workNoticeStagingService.addRetryTimes(workNoticeStaging);
                }
            }
        }
    }
}
