package com.hz.pm.api.sys.manage;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.organization.constant.ProcDefCont;
import com.hz.pm.api.user.model.entity.MhUnit;
import com.hz.pm.api.user.service.IMhUnitService;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.StrPool;
import com.wflow.bean.entity.WflowOrgModelHistorys;
import com.wflow.bean.entity.WflowOrgModels;
import com.wflow.enums.ProcessDefTypeEnum;
import com.wflow.service.OrgProcdefHistoryService;
import com.wflow.service.OrgProcdefService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * OrgProcDefManage
 * </p>
 *
 * @author WendyYang
 * @since 21:45 2024/3/6
 */
@Slf4j
@Component
@AllArgsConstructor
public class OrgProcDefManage {

    private final IMhUnitService mhUnitService;
    private final OrgProcdefService orgProcdefService;
    private final OrgProcdefHistoryService orgProcdefHistoryService;

    /**
     * 初始化 单位配置
     *
     * @return \
     */
    @Transactional(rollbackFor = Exception.class)
    public String initAllUnits() {
        List<MhUnit> units = mhUnitService.list();
        log.info("需要处理初始化单位配置的企业 数量为{}", units.size());
        for (MhUnit unit : units) {
            log.info("当前是 {}", unit.getName());
            String orgCode = String.valueOf(unit.getId());
            Wrapper<WflowOrgModels> womQuery = Wrappers.lambdaQuery(WflowOrgModels.class)
                    .select(WflowOrgModels::getType)
                    .eq(WflowOrgModels::getOrgCode, orgCode);
            List<WflowOrgModels> models = orgProcdefService.list(womQuery);
            List<String> existsType = CollUtils.fieldList(models, WflowOrgModels::getType);
            log.info("已经有的单位流程 {}", JSON.toJSONString(existsType));

            //默认流程
            if (!existsType.contains(ProcessDefTypeEnum.DEFAULT.name())) {
                saveDef(unit, ProcessDefTypeEnum.DEFAULT);
            }
            //盖章流程
            if (!existsType.contains(ProcessDefTypeEnum.SEAL.name())) {
                saveDef(unit, ProcessDefTypeEnum.SEAL);
            }
            //联审流程
            if (!existsType.contains(ProcessDefTypeEnum.JOINT_REVIEW.name())) {
                saveDef(unit, ProcessDefTypeEnum.JOINT_REVIEW);
            }
        }
        return "初始化成功";
    }

    private void saveDef(MhUnit unit, ProcessDefTypeEnum defType) {
        String procDefId = ProcDefCont.MODELS_ID_PREFIX + IdUtil.objectId();
        WflowOrgModels defaultModel = new WflowOrgModels();
        defaultModel.setProcessDefId(procDefId);
        defaultModel.setIsStop(Boolean.FALSE);
        defaultModel.setProcess(StrPool.BRACE);
        defaultModel.setCreated(LocalDateTime.now());
        defaultModel.setIsDelete(Boolean.FALSE);
        defaultModel.setOrgCode(unit.getId().toString());
        defaultModel.setOrgName(unit.getName());
        defaultModel.setProcessDefName(defType.getDesc());
        defaultModel.setType(defType.name());
        defaultModel.setSort(1);
        defaultModel.setVersion(1);
        orgProcdefService.save(defaultModel);

        WflowOrgModelHistorys defaultHistory = new WflowOrgModelHistorys();
        defaultHistory.setProcessDefId(procDefId);
        defaultHistory.setProcess(StrPool.BRACE);
        defaultHistory.setCreated(LocalDateTime.now());
        defaultHistory.setOrgCode(unit.getId().toString());
        defaultHistory.setOrgName(unit.getName());
        defaultHistory.setProcessDefName(defType.getDesc());
        defaultHistory.setType(defType.name());
        defaultHistory.setVersion(0);
        orgProcdefHistoryService.save(defaultHistory);
    }

}
