package com.hz.pm.api.sys.manage;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.sys.contants.UserGuidanceContant;
import com.hz.pm.api.sys.model.vo.UserGuidanceDetailVO;
import com.hz.pm.api.sys.model.vo.UserGuidanceVO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.wflow.bean.entity.WflowOrgModels;
import com.wflow.enums.ProcessDefTypeEnum;
import com.wflow.service.OrgProcdefService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.Objects;

/**
 * <p>
 * UserGuidanceManage
 * </p>
 *
 * @author WendyYang
 * @since 12:19 2024/4/11
 */
@Slf4j
@Component
@AllArgsConstructor
public class UserGuidanceManage {

    private final OrgProcdefService orgProcdefService;

    /**
     * 查询用户引导 详情
     *
     * @return \
     */
    public UserGuidanceVO userGuidanceDetail() {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        UserGuidanceVO vo = UserGuidanceVO.builder()
                .orgCode(user.getMhUnitId())
                .username(user.getUsername())
                .userId(user.getUserId())
                .build();
        //查询引导详情
        UserGuidanceDetailVO orgModelGuidance = new UserGuidanceDetailVO();
        orgModelGuidance.setName(UserGuidanceContant.OrgModel.GUIDANCE_NAME);
        orgModelGuidance.setRemark(UserGuidanceContant.OrgModel.GUIDANCE_REMARK);

        int finishedNum = 0;
        //1.去查询 用户所在单位 有没有单位默认流程配置 并且process 不能为空
        WflowOrgModels orgModel = orgProcdefService.getOne(Wrappers.lambdaQuery(WflowOrgModels.class)
                .eq(WflowOrgModels::getOrgCode, user.getMhUnitIdStr())
                .eq(WflowOrgModels::getType, ProcessDefTypeEnum.DEFAULT.name())
                .eq(WflowOrgModels::getIsDelete, Boolean.FALSE)
                .last(BizConst.LIMIT_1));
        if (Objects.isNull(orgModel)) {
            orgModelGuidance.setPath(UserGuidanceContant.OrgModel.GUIDANCE_PATH1);
            orgModelGuidance.setIsFinish(Boolean.FALSE);
        } else {
            orgModelGuidance.setPath(UserGuidanceContant.OrgModel.GUIDANCE_PATH2 + orgModel.getProcessDefId());
            //判断process是否为空
            if (StringUtils.isNotBlank(orgModel.getProcess()) &&
                    !UserGuidanceContant.OrgModel.EMPTY_PROCESS.equals(orgModel.getProcess())) {
                orgModelGuidance.setIsFinish(Boolean.TRUE);
                finishedNum++;
            } else {
                orgModelGuidance.setIsFinish(Boolean.FALSE);
            }
        }
        if (UserGuidanceContant.ALL_FINISHED_NUM == finishedNum) {
            vo.setIsAllFinished(Boolean.TRUE);
        }
        vo.setDetails(Collections.singletonList(orgModelGuidance));
        return vo;
    }

}
