package com.hz.pm.api.sys.service.impl;

import cn.hutool.http.HttpStatus;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ningdatech.basic.exception.BaseUncheckedException;
import com.ningdatech.basic.util.CollUtils;
import com.hz.pm.api.sys.mapper.RoleMapper;
import com.hz.pm.api.sys.mapper.UserRoleMapper;
import com.hz.pm.api.sys.model.entity.Role;
import com.hz.pm.api.sys.model.entity.UserRole;
import com.hz.pm.api.sys.service.IUserRoleService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 业务实现类
 * 角色分配
 * 账号角色绑定
 * </p>
 *
 * @author PoffyZhang
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class UserRoleServiceImpl extends ServiceImpl<UserRoleMapper, UserRole> implements IUserRoleService {

    private static final String INIT_ROLE_CODE = "SUPER_ADMIN";

    private final RoleMapper roleMapper;

    private LambdaQueryWrapper<UserRole> roleIdsQuery(Collection<Long> roleIds) {
        return Wrappers.lambdaQuery(UserRole.class).in(UserRole::getRoleId,roleIds);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean initAdmin(Long userId) {
        Role role = roleMapper.selectOne(Wrappers.lambdaQuery(Role.class)
                .eq(Role::getName, INIT_ROLE_CODE));
        if (role == null) {
            throw new BaseUncheckedException(HttpStatus.HTTP_OK, String.format("初始化用户角色失败, 无法查询到内置角色:%s", INIT_ROLE_CODE));
        }
        UserRole userRole = UserRole.builder()
                .userId(userId).roleId(role.getId())
                .build();

        return super.save(userRole);
    }

    @Override
    public List<Long> listUserIdByRoleId(Long roleId) {
        return listUserIdByRoleIds(Collections.singletonList(roleId));
    }

    @Override
    public List<Long> listUserIdByRoleIds(Collection<Long> roleIds) {
        LambdaQueryWrapper<UserRole> wrapper = roleIdsQuery(roleIds)
                .select(UserRole::getUserId);
        List<UserRole> userRoles = this.list(wrapper);
        return CollUtils.fieldList(userRoles, UserRole::getUserId);
    }

}
