package com.hz.pm.api.todocenter.controller;


import com.hz.pm.api.todocenter.manage.TodoCenterManage;
import com.hz.pm.api.todocenter.model.dto.AdjustHandleDTO;
import com.hz.pm.api.todocenter.model.dto.SealInfoDTO;
import com.hz.pm.api.todocenter.model.req.ProcessDetailReq;
import com.hz.pm.api.todocenter.model.req.ToBeProcessedExportReq;
import com.hz.pm.api.todocenter.model.req.ToBeProcessedReq;
import com.hz.pm.api.todocenter.model.vo.ProcessProgressDetailVO;
import com.hz.pm.api.todocenter.model.vo.ResToBeProcessedVO;
import com.hz.pm.api.todocenter.model.vo.TodoNumVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import com.wflow.workflow.bean.dto.ReqAuditOpinionSaveDTO;
import com.wflow.workflow.bean.dto.ReqProcessHandlerDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * <p>
 * 待办中心 前端控制器
 * </p>
 *
 * @author CMM
 * @since 2023-01-12
 */
@RestController
@AllArgsConstructor
@Api(tags = "待办中心控制器")
@RequestMapping("/api/v1/todo-center")
public class TodoCenterController {

    private final TodoCenterManage todoCenterManage;

    /**
     * 实时取当前操作人 某个工作流的下一个操作taskId
     */
    @GetMapping("/todo/taskId/{instanceId}")
    public String todoTaskId(@PathVariable String instanceId) {
        return todoCenterManage.todoTaskId(instanceId);
    }

    /**
     * 待办中心-待我处理项目列表查询
     *
     * @param param
     * @return
     */
    @GetMapping("/todo-list")
    public PageVo<ResToBeProcessedVO> todoList(@Valid @ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.todoProjectList(param);
    }

    @GetMapping("/todo-list-xinchuang")
    public PageVo<ResToBeProcessedVO> todoListXinchuang(@Valid @ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.todoXinchuangList(param);
    }

    /**
     * 待办中心-待我处理项目列表导出
     *
     * @param param
     * @param response
     * @return void
     */
    @PostMapping("/todo-list/export")
    @WebLog("待办中心-待我处理项目列表导出")
    public void exportPendingProjectList(@Valid @RequestBody ToBeProcessedExportReq param,
                                         HttpServletResponse response) {
        todoCenterManage.exportTodoList(response, param);
    }

    /**
     * 查询流程表单数据及审批的进度步骤
     *
     * @param request 请求参数
     * @return 流程进度及表单详情
     */
    @GetMapping("/progress/detail")
    public ProcessProgressDetailVO getProcessDetail(@Valid @ModelAttribute ProcessDetailReq request) {
        return todoCenterManage.getProcessDetail(request);
    }

    @GetMapping("/progress/xinchuang-detail")
    public ProcessProgressDetailVO getProcessXinchuangDetail(@Valid @ModelAttribute ProcessDetailReq request) {
        return todoCenterManage.getProcessXinchuangDetail(request);
    }

    /**
     * 审核通过，盖章并通过、退回、撤回、驳回等操作
     *
     * @param param 操作参数
     * @return 操作结果
     */
    @PostMapping("/handler")
    @WebLog("审核通过，盖章并通过、退回、撤回、驳回等操作")
    public Object handler(@Valid @RequestBody ReqProcessHandlerDTO param) {
        return todoCenterManage.handler(param);
    }

    @PostMapping("/handler-xinchuang")
    @WebLog("信创审核通过，盖章并通过、退回、撤回、驳回等操作")
    public Object handlerXinchuang(@Valid @RequestBody ReqProcessHandlerDTO param) {
        return todoCenterManage.handlerXinchuang(param);
    }

    /**
     * 保存盖章审核提交的审核意见
     *
     * @param param 操作参数
     * @return 操作结果
     */
    @PostMapping("/saveAuditOpinion")
    @WebLog("保存盖章审核提交的审核意见")
    public Object saveAuditOpinion(@Valid @RequestBody ReqAuditOpinionSaveDTO param) {
        return todoCenterManage.saveAuditOpinion(param);
    }

    /**
     * 待办中心-我已处理项目列表查询
     *
     * @param param
     * @return
     */
    @GetMapping("/ido-list")
    public PageVo<ResToBeProcessedVO> idoList(@ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.idoList(param);
    }

    @GetMapping("/ido-list-xinchuang")
    public PageVo<ResToBeProcessedVO> idoListXinchuang(@ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.idoListXinchuang(param);
    }

    /**
     * 待办中心-我已处理项目列表导出
     *
     * @param param
     * @param response
     * @return void
     */
    @PostMapping("/ido-list/export")
    public void exportHandledProjectList(@RequestBody ToBeProcessedExportReq param,
                                         HttpServletResponse response) {
        todoCenterManage.idoExport(response, param);
    }

    /**
     * 待办中心-我发起的项目列表查询
     *
     * @param param \
     */
    @GetMapping("/my-submitted-list")
    public PageVo<ResToBeProcessedVO> mySubmittedList(@ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.mySubmittedList(param);
    }

    @GetMapping("/my-submitted-xinchuang")
    public PageVo<ResToBeProcessedVO> mySubmittedXinchuang(@ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.mySubmittedXinchuang(param);
    }

    /**
     * 待办中心-我发起的项目列表导出
     *
     * @param param    \
     * @param response \
     */
    @PostMapping("/my-submitted-list/export")
    public void exportMySubmittedList(@RequestBody ToBeProcessedExportReq param,
                                      HttpServletResponse response) {
        todoCenterManage.exportMySubmittedList(response, param);
    }

    /**
     * 待办中心-抄送我的项目列表查询
     *
     * @param param \
     */
    @GetMapping("/ccme-list")
    public PageVo<ResToBeProcessedVO> ccmeList(@Valid @ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.ccmeList(param);
    }

    @GetMapping("/ccme-list-xinchuang")
    public PageVo<ResToBeProcessedVO> ccmeListXinchuang(@Valid @ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.ccmeListXinchuang(param);
    }

    /**
     * 待办中心-抄送我的项目列表导出
     *
     * @param param    \
     * @param response \
     */
    @PostMapping("/ccme-list/export")
    public void exportCcMeProjectList(@Valid @RequestBody ToBeProcessedExportReq param, HttpServletResponse response) {
        todoCenterManage.exportCcMeProjectList(response, param);
    }

    @ApiOperation(value = "被退回项目内容调整并通过", notes = "被退回项目内容调整并通过")
    @PostMapping("/adjustAndHandle")
    @WebLog("被退回项目内容调整并通过")
    public synchronized String startTheProcess(@Validated @RequestBody AdjustHandleDTO dto) {
        String instanceId = todoCenterManage.adjustmentAndHandle(dto);
        return "退回项目内容调整并且重新通过 【" + instanceId + "】 成功";
    }

    @ApiOperation("获取签章后的pdf文件")
    @WebLog("获取签章后的pdf文件")
    @PostMapping("/getSealedPdf")
    public Long getSealedPdf(@Valid @RequestBody SealInfoDTO req) {
        return todoCenterManage.getSealedPdf(req);
    }

    @ApiOperation("待办中心待我处理数量统计")
    @GetMapping("/getTodoNums")
    public TodoNumVO getTodoNums(@Valid @ModelAttribute ToBeProcessedReq param) {
        return todoCenterManage.getTodoNums(param);
    }
}
