package com.hz.pm.api.todocenter.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Sets;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.WflowHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.TypeReferenceConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.*;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.filemanage.model.entity.ProjectApplyBorrow;
import com.hz.pm.api.filemanage.service.INdProjectApplyBorrowService;
import com.hz.pm.api.projectdeclared.manage.DeclaredProjectManage;
import com.hz.pm.api.projectdeclared.model.dto.ProjectDraftSaveDTO;
import com.hz.pm.api.projectdeclared.model.entity.ProjectConstructionSuggestions;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.model.entity.Xinchuang;
import com.hz.pm.api.projectdeclared.model.enumerization.ProjectDraftTypeEnum;
import com.hz.pm.api.projectdeclared.service.IConstructionSuggestionsService;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.service.IXinchuangService;
import com.hz.pm.api.projectlib.helper.ProjectChangeStopHelper;
import com.hz.pm.api.projectlib.model.dto.ProjectApplicationDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectApplication;
import com.hz.pm.api.projectlib.model.entity.ProjectDelayApply;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderXcfhxApplyStatus;
import com.hz.pm.api.projectlib.service.INdProjectDelayApplyService;
import com.hz.pm.api.projectlib.service.IProjectApplicationService;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.staging.enums.MsgTypeEnum;
import com.hz.pm.api.sys.manage.NoticeManage;
import com.hz.pm.api.todocenter.constant.TodoCenterConst;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.wflow.contants.HisProInsEndActId;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.process.enums.NodeTypeEnum;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.ProcessHandlerEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.HistoryService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.engine.task.Comment;
import org.flowable.variable.api.history.HistoricVariableInstance;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static com.hz.pm.api.todocenter.constant.WorkNoticeConst.*;

/**
 * @Classname HandlerManage
 * @Description
 * @Date 2023/4/2 14:25
 * @Author PoffyZhang
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class HandlerManage {

    private final RuntimeService runtimeService;
    private final TaskService taskService;
    private final HistoryService historyService;
    private final IProjectService projectService;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final ProcessInstanceService processInstanceService;
    private final IProjectApplicationService projectApplicationService;
    private final IProjectInstService projectInstService;
    private final NoticeManage noticeManage;
    private final DeclaredProjectManage declaredProjectManage;
    private final INdProjectDelayApplyService projectDelayApplyService;
    private final INdProjectApplyBorrowService projectApplyBorrowService;
    private final IConstructionSuggestionsService constructionSuggestionsService;
    private final IPurchaseService purchaseService;
    private final IPurchaseInstService purchaseInstService;
    private final TenderStateMachineUtil tenderStateMachineUtil;
    private final SelfTestStateMachineUtil selfTestStateMachineUtil;
    private final AdaptStateMachineUtil adaptStateMachineUtil;
    private final TestValidStateMachineUtil testValidStateMachineUtil;
    private final XcfhxStateMachineUtil xcfhxStateMachineUtil;
    private final WflowHelper wflowHelper;
    private final ProjectChangeStopHelper projectChangeStopHelper;

    private final IXinchuangService xinchuangService;

    @Value("${spring.profiles.active}")
    private String active;


    /**
     * 审核通过后 所处理的逻辑
     *
     * @param project  \
     * @param instance \
     */
    public void afterPassTodo(Project project, HistoricProcessInstance instance) {
        Long userId = LoginUserUtil.getUserId();
        // 获取流程通过后的流程实例
        HistoricProcessInstance newInstance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(instance.getId())
                .singleResult();
        // 获取当前流程实例类型
        String instanceId = newInstance.getId();
        ProjectInst projectInst = projectInstService.getByInstCode(instanceId);
        Integer instType = projectInst.getInstType();
        InstTypeEnum instTypeEnum = InstTypeEnum.getNoNull(instType);
        // 获取流程通过后当前流程详情
        ProcessProgressVo newInstDetail = processInstanceService.getProgressInstanceDetail(null, instance.getId());
        // 获取流程通过后当前审核人信息，向其发送工作通知
        // 会签/或签会有多个审核人
        Set<String> currUserIdList = Sets.newHashSet();
        List<ProgressNode> newProgressInfo = newInstDetail.getProgressInfo();
        ProgressNode currNode = newProgressInfo.get(newProgressInfo.size() - 1);

        // 说明当前节点是子流程节点
        if (currNode.getNodeType().name().equals(NodeTypeEnum.SUB.name())) {
            List<ProgressNode> children = currNode.getChildren();
            // 获取子流程当前审核人节点
            ProgressNode subCurrNode = CollUtil.isEmpty(children) ? new ProgressNode() : children.get(children.size() - 1);
            // 获取节点ID相同地审核节点
            children.forEach(c -> {
                if (subCurrNode.getNodeId().equals(c.getNodeId())) {
                    currUserIdList.add(c.getUserId());
                }
            });
        } else {
            newProgressInfo.forEach(w -> {
                if (currNode.getNodeId().equals(w.getNodeId())) {
                    currUserIdList.add(w.getUserId());
                }
            });
        }

        // 流程通过后，判断当前登录用户是不是最后一个审核人
        // 若当前登录用户是最后一个审批人，需更新流程状态为审核完成，项目状态到下个状态
        // 并向流程发起人发送浙政钉工作通知：【项目名称】已通过【流程名称】，请及时开始下一步操作。
        if (HisProInsEndActId.END.equals(newInstance.getEndActivityId())) {
            // 如果是申请延期和申请借阅审批流程，不走状态机
            if (InstTypeEnum.APPLY_DELAY.eq(instType)
                    || InstTypeEnum.APPLY_BORROW.eq(instType)
                    || InstTypeEnum.TEST_VALID.eq(instType)
                    || InstTypeEnum.SELF_TEST.eq(instType)
                    || InstTypeEnum.ADAPTION.eq(instType)
                    || InstTypeEnum.XCFHX_APPLY.eq(instType)) {
                switch (instTypeEnum) {
                    case APPLY_DELAY:
                        // 保存延期申请记录,更新项目建设周期和计划验收时间
                        updateProjectDelayApplyInfo(project, instanceId);
                        break;
                    case APPLY_BORROW:
                        // 更新申请借阅状态为成功
                        updateProjectApplyBorrowInfo(project, instanceId);
                        break;
                    case ADAPTION:
                    case SELF_TEST:
                    case TEST_VALID:
                    case XCFHX_APPLY:
                        purchaseOperationPassedCallback(project.getId(), instanceId, instTypeEnum);
                        break;
                    default:
                        throw BizException.wrap("传入实例类型错误: %s", instTypeEnum);
                }
            } else {
                ProjectStatus projectStatus = ProjectStatus.getNoNull(project.getStatus());
                switch (projectStatus) {
                    //  立项备案审批
                    case DECLARED_APPROVED_RECORD_AUDITING:
                    case ON_COMPLIANCE_REVIEW:
                    case ON_PROJECT_REVIEW:
                        updatePassProjectStatus(userId, project);
                        break;
                    case ON_PURCHASING:
                    case TO_BE_FIRST_INSPECTED:
                    case ON_PILOT_RUNNING:
                    case ON_FINALLY_INSPECTED:
                        ProjectProcessType processStage = ProjectProcessType.getNoNull(instTypeEnum.getCode());
                        purchasePassedCallback(project, instanceId, processStage);
                        break;
                    case ON_CHANGE_APPLY:
                    case ON_STOPPED_APPLY:
                        stoppedChaneCallbackAfterPass(project, instTypeEnum);
                        break;
                    default:
                        throw BizException.wrap("传入项目状态错误： %s", project.getStatus());
                }
            }
            //发送消息
            noticeManage.sendNotice(Long.parseLong(instance.getStartUserId()), project,
                    instance.getProcessDefinitionName(), PASS_MSG_TEMPLATE2, MsgTypeEnum.PROJECT_REVIEW_PASS);
        } else {
            // 若有下一个审核人(当前节点的用户)，会签/或签会有多个
            // 向其发送浙政钉工作通知：标题：审核任务 内容：【单位名称】的【项目名称】需要您审核。
            // 获取发送浙政钉工作通知必要信息
            for (String currUserId : currUserIdList) {
                if (!NumberUtil.isNumber(currUserId)) {
                    log.warn("审核人信息不存在：{}", currUserId);
                } else {
                    //发送消息
                    noticeManage.sendNotice(Long.parseLong(currUserId), project, instance.getProcessDefinitionName(),
                            PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);
                }
            }
        }
    }

    public void afterPassTodo(Xinchuang xinchuang, HistoricProcessInstance instance) {
        Long userId = LoginUserUtil.getUserId();
        // 获取流程通过后的流程实例
        HistoricProcessInstance newInstance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(instance.getId())
                .singleResult();
        // 获取当前流程实例类型
        String instanceId = newInstance.getId();
        // 获取流程通过后当前流程详情
        ProcessProgressVo newInstDetail = processInstanceService.getProgressInstanceDetail(null, instance.getId());
        // 获取流程通过后当前审核人信息，向其发送工作通知
        // 会签/或签会有多个审核人
        Set<String> currUserIdList = Sets.newHashSet();
        List<ProgressNode> newProgressInfo = newInstDetail.getProgressInfo();
        ProgressNode currNode = newProgressInfo.get(newProgressInfo.size() - 1);

        // 说明当前节点是子流程节点
        if (currNode.getNodeType().name().equals(NodeTypeEnum.SUB.name())) {
            List<ProgressNode> children = currNode.getChildren();
            // 获取子流程当前审核人节点
            ProgressNode subCurrNode = CollUtil.isEmpty(children) ? new ProgressNode() : children.get(children.size() - 1);
            // 获取节点ID相同地审核节点
            children.forEach(c -> {
                if (subCurrNode.getNodeId().equals(c.getNodeId())) {
                    currUserIdList.add(c.getUserId());
                }
            });
        } else {
            newProgressInfo.forEach(w -> {
                if (currNode.getNodeId().equals(w.getNodeId())) {
                    currUserIdList.add(w.getUserId());
                }
            });
        }

        // 流程通过后，判断当前登录用户是不是最后一个审核人
        // 若当前登录用户是最后一个审批人，需更新流程状态为审核完成，项目状态到下个状态
        // 并向流程发起人发送浙政钉工作通知：【项目名称】已通过【流程名称】，请及时开始下一步操作。
        if (HisProInsEndActId.END.equals(newInstance.getEndActivityId())) {
            xinchuangPassedCallback(xinchuang);
        }
    }

    private void xinchuangPassedCallback(Xinchuang xinchuang) {
        xinchuang.setStatus(TenderXcfhxApplyStatus.XCFHX_APPLY_PASSED.getCode());
        xinchuangService.updateById(xinchuang);
    }

    private void purchaseOperationPassedCallback(Long projectId, String instanceId, InstTypeEnum instType) {
        PurchaseInst purchaseInst = purchaseInstService.getByProjectIdAndInstCode(projectId, instanceId);
        Purchase purchase = purchaseService.getById(purchaseInst.getBidId());
        switch (instType) {
            case ADAPTION:
                adaptStateMachineUtil.pass(purchase);
                break;
            case SELF_TEST:
                selfTestStateMachineUtil.pass(purchase);
                break;
            case TEST_VALID:
                testValidStateMachineUtil.pass(purchase);
                break;
            case XCFHX_APPLY:
                xcfhxStateMachineUtil.pass(purchase);
                break;
            default:
                throw BizException.wrap("无效的流程类型");
        }
        purchaseService.updateById(purchase);
    }

    private void purchaseOperationRejectedCallback(Long projectId, String instanceId, InstTypeEnum instType) {
        PurchaseInst purchaseInst = purchaseInstService.getByProjectIdAndInstCode(projectId, instanceId);
        Purchase purchase = purchaseService.getById(purchaseInst.getBidId());
        switch (instType) {
            case ADAPTION:
                adaptStateMachineUtil.reject(purchase);
                break;
            case SELF_TEST:
                selfTestStateMachineUtil.reject(purchase);
                break;
            case TEST_VALID:
                testValidStateMachineUtil.reject(purchase);
                break;
            case XCFHX_APPLY:
                xcfhxStateMachineUtil.reject(purchase);
                break;
            default:
                throw BizException.wrap("无效的流程类型");
        }
        purchaseService.updateById(purchase);
    }

    private void purchasePassedCallback(Project project, String instanceId, ProjectProcessType processStage) {
        // 查询所有的标段
        List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
        Map<Long, Purchase> purchaseMap = CollUtils.listToMap(purchases, Purchase::getId);
        // 查询所有标段对应的最新的信创审批流程编码
        List<PurchaseInst> instList = purchaseInstService.list(Wrappers.lambdaQuery(PurchaseInst.class)
                .in(PurchaseInst::getBidId, purchaseMap.keySet())
                .eq(PurchaseInst::getInstType, processStage.getCode()));

        Purchase currPurchase = instList.stream()
                .filter(w -> w.getInstCode().equals(instanceId))
                .findFirst()
                .flatMap(w -> Optional.ofNullable(purchaseMap.get(w.getBidId())))
                .orElseThrow(() -> BizException.wrap("当前审批的标段不存在"));

        tenderStateMachineUtil.pass(currPurchase);
        purchaseService.updateById(currPurchase);

        Collection<PurchaseInst> lastInstList = BizUtils.groupFirst(instList, PurchaseInst::getBidId,
                Comparator.comparing(PurchaseInst::getCreateOn).reversed());
        Set<String> instCodes = CollUtils.fieldSet(lastInstList, PurchaseInst::getInstCode);
        if (instCodes.size() != purchases.size()) {
            return;
        }
        if (wflowHelper.allInstancesFinished(instCodes)) {
            projectStateMachineUtil.pass(project);
            projectService.updateById(project);
        }
    }

    private void purchaseRejectedCallback(Project project, String instanceId) {
        PurchaseInst purchaseInst = purchaseInstService.getByProjectIdAndInstCode(project.getId(), instanceId);
        Assert.notNull(purchaseInst, "该标段对应的流程不存在");
        Purchase purchase = purchaseService.getById(purchaseInst.getBidId());
        Assert.notNull(purchase, "标段信息不存在");
        tenderStateMachineUtil.reject(purchase);
        purchaseService.updateById(purchase);
    }

    /**
     * 去保存 建设建议信息
     *
     * @param instanceId \
     * @param project    \
     */
    private void saveConstructionSuggestInfo(String instanceId, Project project) {
        Wrapper<ProjectConstructionSuggestions> query = Wrappers.lambdaQuery(ProjectConstructionSuggestions.class)
                .eq(ProjectConstructionSuggestions::getProjectCode, project.getProjectCode())
                .eq(ProjectConstructionSuggestions::getInstanceId, instanceId)
                .last(BizConst.LIMIT_1);
        ProjectConstructionSuggestions pcs = constructionSuggestionsService.getOne(query);
        if (Objects.nonNull(pcs)) {
            project.setProposeTotalInvest(pcs.getProposeTotalInvest());
            project.setProposeAnnualBudget(pcs.getProposeAnnualBudget());
            project.setProposeAttachFiles(pcs.getProposeAttachFiles());
            projectService.updateById(project);
        }
    }

    public void updateProjectApplyBorrowInfo(Project project, String instanceId) {
        // 获取申请借阅信息
        Wrapper<ProjectApplyBorrow> query = Wrappers.lambdaQuery(ProjectApplyBorrow.class)
                .eq(ProjectApplyBorrow::getProjectId, project.getId())
                .eq(ProjectApplyBorrow::getInstanceId, instanceId);
        ProjectApplyBorrow projectApplyBorrow = projectApplyBorrowService.getOne(query);
        // 更新项目借阅信息为成功
        projectApplyBorrow.setIsSuccess(Boolean.TRUE);
        projectApplyBorrow.setUpdateOn(LocalDateTime.now());
        projectApplyBorrow.setUpdateBy(LoginUserUtil.getUserId());
        projectApplyBorrowService.updateById(projectApplyBorrow);
    }

    public void updateProjectDelayApplyInfo(Project declaredProject, String instanceId) {
        // 获取延期申请填写的相关信息
        ProjectDelayApply delayApply = projectDelayApplyService.getOne(Wrappers.lambdaQuery(ProjectDelayApply.class)
                .eq(ProjectDelayApply::getProjectId, declaredProject.getId())
                .eq(ProjectDelayApply::getInstanceId, instanceId));
        delayApply.setSuccess(Boolean.TRUE);
        delayApply.setUpdateOn(LocalDateTime.now());
        projectDelayApplyService.updateById(delayApply);
        // 获取项目的立项批复时间和建设周期
        LocalDateTime approvalDate = declaredProject.getApprovalDate();
        String buildCycle = declaredProject.getBuildCycle();
        // 重新计算建设周期、验收时间
        String constructCycle = String.valueOf(Integer.parseInt(buildCycle) + delayApply.getDelayTime());
        declaredProject.setBuildCycle(constructCycle);
        long constructMonth = Long.parseLong(constructCycle);
        String acceptTime = NdDateUtils.format(approvalDate.plusMonths(constructMonth), DatePattern.NORM_DATETIME_MINUTE_PATTERN);
        LocalDateTime time = LocalDateTime.parse(acceptTime, DateTimeFormatter.ofPattern(DatePattern.NORM_DATETIME_MINUTE_PATTERN));
        declaredProject.setPlanAcceptanceTime(time);
        declaredProject.setInstCode(instanceId);
        declaredProject.setApplyDelayFile(delayApply.getDelayApplyFile());
        declaredProject.setApplyDelayReason(delayApply.getDelayApplyReason());
        //增加总延期的月份
        declaredProject.setApplyDelayMonths(Objects.nonNull(declaredProject.getApplyDelayMonths()) ?
                declaredProject.getApplyDelayMonths() + delayApply.getDelayTime() : delayApply.getDelayTime());
        projectService.updateById(declaredProject);
    }


    /**
     * 驳回后 所处理的逻辑
     *
     * @param project \
     * @param hiInst  \
     */
    public void afterRejectTodo(Project project, HistoricProcessInstance hiInst) {
        Long userId = LoginUserUtil.getUserId();
        // 获取当前流程实例类型
        String instanceId = hiInst.getId();
        ProjectInst projectInst = projectInstService.getByInstCode(instanceId);
        Integer instType = projectInst.getInstType();
        // 审批流程不是申请延期和申请借阅,需调用状态机
        InstTypeEnum instTypeEnum = InstTypeEnum.getNoNull(instType);
        switch (instTypeEnum) {
            case APPLY_BORROW:
            case APPLY_DELAY:
                break;
            case PROJECT_FINAL_INSPECTION:
                purchaseRejectedCallback(project, instanceId);
                break;
            case XCFHX_APPLY:
            case TEST_VALID:
            case SELF_TEST:
            case ADAPTION:
                purchaseOperationRejectedCallback(project.getId(), instanceId, instTypeEnum);
                break;
            case PROJECT_CHANGE:
            case PROJECT_STOPPED:
                stoppedChaneCallbackAfterReject(project, instTypeEnum);
                break;
            default:
                updateRejectProjectStatus(userId, project, instType);
                break;

        }
        //发送消息
        noticeManage.sendNotice(userId, project, hiInst.getProcessDefinitionName(),
                REJECT_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW_REJECT);
    }

    public void stoppedChaneCallbackAfterPass(Project project,
                                              InstTypeEnum instType) {
        ProjectStateChangeEvent event;
        if (InstTypeEnum.PROJECT_STOPPED.equals(instType)) {
            event = ProjectStateChangeEvent.STOPPED_APPLY_PASS;
        } else if (InstTypeEnum.PROJECT_CHANGE.equals(instType)) {
            event = ProjectStateChangeEvent.CHANGE_APPLY_PASS;
        } else {
            throw BizException.wrap("不支持的流程类型");
        }
        projectChangeStopHelper.changeProjectPurchaseStatus(project, event);
    }

    public void stoppedChaneCallbackAfterReject(Project project,
                                                InstTypeEnum instType) {
        ProjectStateChangeEvent event;
        if (InstTypeEnum.PROJECT_STOPPED.equals(instType)) {
            event = ProjectStateChangeEvent.STOPPED_APPLY_FAILED;
        } else if (InstTypeEnum.PROJECT_CHANGE.equals(instType)) {
            event = ProjectStateChangeEvent.CHANGE_APPLY_FAILED;
        } else {
            throw BizException.wrap("不支持的流程类型");
        }
        projectChangeStopHelper.changeProjectPurchaseStatus(project, event);
    }

    public void afterRejectTodo(Xinchuang xinchuang, HistoricProcessInstance hiInst) {
        xinchuang.setStatus(TenderXcfhxApplyStatus.XCFHX_APPLY_FAILED.getCode());
        xinchuangService.updateById(xinchuang);
    }

    /**
     * 发起人 撤回逻辑
     */
    public void rootWithDraw(Project project) {
        /*Long userId = LoginUserUtil.getUserId();
        // 若是流程发起人点击撤回，项目回到上一个状态，需调用状态机更新项目状态,流程状态更新为审核通过
        switch (ProjectStatus.getNoNull(project.getStatus())) {
            // 当前项目状态是单位内部审核中
            case UNDER_INTERNAL_AUDIT:
                // 更新项目状态为待申报
                updateWithdrawProjectStatus(userId, project);
                // 保存到草稿箱中
                saveToDraft(project);
                //删除所有项目相关的信息
                deleteProjectRelated(project);
                break;
                // 当前项目状态是部门联审中
            case DEPARTMENT_JOINT_REVIEW:
                updateWithdrawProjectStatus(userId, project);
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + project.getStatus());
        }*/
    }

    // 保存到草稿箱中
    private void saveToDraft(Project req) {
        ProjectDraftSaveDTO projectDraft = new ProjectDraftSaveDTO();
        ProjectDTO projectInfo = new ProjectDTO();
        BeanUtils.copyProperties(req, projectInfo);
        // 查询出项目关联的应用信息
        List<ProjectApplication> applications = projectApplicationService.listByProjectId(req.getId());
        List<ProjectApplicationDTO> applicationDTOList = applications.stream().map(a -> {
            ProjectApplicationDTO applicationDTO = new ProjectApplicationDTO();
            BeanUtils.copyProperties(a, applicationDTO);
            return applicationDTO;
        }).collect(Collectors.toList());
        projectInfo.setApplicationList(applicationDTOList);
        Map<String, Object> dynamicMap = JSON.parseObject(req.getDynamicForm(), TypeReferenceConst.STR_OBJ_MAP);
        projectInfo.setDynamicForm(dynamicMap);
        projectDraft.setProjectInfo(projectInfo);
        declaredProjectManage.saveToDraft(projectDraft, ProjectDraftTypeEnum.PROJECT_DECLARED);
    }

    /**
     * 删除项目的所有信息
     *
     * @param project \
     */
    private void deleteProjectRelated(Project project) {
        //删除项目库中该项目信息
        projectService.removeById(project);
        //删除app信息
        projectApplicationService.remove(Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(ProjectApplication::getProjectId, project.getId()));
    }

    /**
     * 退回审核后 所处理的逻辑
     *
     * @param project  \
     * @param instance \
     */
    public void afterBackTodo(Project project, HistoricProcessInstance instance) {
        Long userId = LoginUserUtil.getUserId();
        // 获取当前流程实例类型
        String instCode = project.getInstCode();
        ProjectInst projectInst = projectInstService.getByInstCode(instCode);
        Integer instType = projectInst.getInstType();
        LambdaUpdateWrapper<Project> updateWrapper = Wrappers.lambdaUpdate();
        // 如果是预审审批或建设方案退回,需要重新盖章，原来盖过章的文件要清空
        if (InstTypeEnum.PRELIMINARY_PREVIEW.eq(instType)) {
            updateWrapper.set(Project::getPretrialFileId, null)
                    .set(Project::getPretrialFileName, null)
                    .eq(Project::getId, project.getId());
            projectService.update(updateWrapper);
        } else if (InstTypeEnum.CONSTRUCTION_PLAN_REVIEW.eq(instType)) {
            updateWrapper.set(Project::getConstructFileId, null)
                    .set(Project::getConstructFileName, null)
                    .eq(Project::getId, project.getId());
            projectService.update(updateWrapper);
        }
        // 给项目创建人、流程发起人发送浙政钉工作通知：【项目名称】的【流程名称】被退回，请及时处理。
        //发送消息
        noticeManage.sendNotice(userId, project, instance.getProcessDefinitionName(),
                BACK_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW_BACK);
    }

    /**
     * 当为通过操作时，更新项目表中项目状态
     *
     * @param userId  \
     * @param project \
     * @author CMM
     * @since 2023/02/08
     */
    public void updatePassProjectStatus(Long userId, Project project) {
        try {
            projectStateMachineUtil.pass(project);
            project.setUpdateOn(LocalDateTime.now());
            project.setUpdateBy(userId);
            projectService.updateById(project);
        } catch (Exception e) {
            log.error("状态机执行失败", e);
            throw new BizException("状态机执行失败!");
        }
    }

    /**
     * 当为驳回操作时，更新项目表中的项目状态
     *
     * @param userId   \
     * @param project  \
     * @param instType \
     * @author CMM
     * @since 2023/02/08
     */
    private void updateRejectProjectStatus(Long userId, Project project, Integer instType) {
        projectStateMachineUtil.reject(project);
        project.setUpdateOn(LocalDateTime.now());
        project.setUpdateBy(userId);
        projectService.updateById(project);
        // 如果是预审审批驳回,需要重新盖章，原来盖过章的文件要清空
        LambdaUpdateWrapper<Project> updateWrapper;
        if (InstTypeEnum.PRELIMINARY_PREVIEW.eq(instType)) {
            updateWrapper = Wrappers.lambdaUpdate(Project.class)
                    .set(Project::getPretrialFileId, null)
                    .set(Project::getPretrialFileName, null)
                    .eq(Project::getId, project.getId());
        } else if (InstTypeEnum.CONSTRUCTION_PLAN_REVIEW.eq(instType)) {
            updateWrapper = Wrappers.lambdaUpdate(Project.class)
                    .set(Project::getConstructFileId, null)
                    .set(Project::getConstructFileName, null)
                    .eq(Project::getId, project.getId());
        } else {
            return;
        }
        projectService.update(updateWrapper);
    }

    /**
     * 当为撤回操作时，更新项目表中的项目状态为前一个状态
     *
     * @param userId  \
     * @param project \
     * @author CMM
     * @since 2023/02/08
     */
    public void updateWithdrawProjectStatus(Long userId, Project project) {
        // 删除关联表信息
        projectInstService.remove(Wrappers.lambdaQuery(ProjectInst.class)
                .eq(ProjectInst::getProjectId, project.getId())
                .eq(ProjectInst::getInstCode, project.getInstCode()));
        //删除项目的实例信息
        historyService.deleteHistoricProcessInstance(project.getInstCode());

        projectStateMachineUtil.withDraw(project);
        project.setUpdateOn(LocalDateTime.now());
        project.setUpdateBy(userId);
        project.setInstCode(TodoCenterConst.Declared.NULL_INST_CODE);
        projectService.updateById(project);
    }

    public void deleteBackComments(List<HistoricVariableInstance> approves) {
        if (CollUtil.isNotEmpty(approves)) {
            for (HistoricVariableInstance approve : approves) {
                if (approve.getValue() instanceof ProcessHandlerEnum
                        && (ProcessHandlerEnum.BACK.equals(ProcessHandlerEnum.getEnumByName(String.valueOf(approve.getValue()))))) {
                    runtimeService.removeVariable(approve.getProcessInstanceId(), approve.getVariableName());

                    Optional<Comment> first = taskService.getProcessInstanceComments(approve.getProcessInstanceId())
                            .stream()
                            .filter(c -> c.getTaskId().equals(approve.getVariableName().replace("approve_", StringUtils.EMPTY)))
                            .findFirst();
                    first.ifPresent(comment -> taskService.deleteComment(comment.getId()));
                    break;
                }
            }
        }
    }

}
