package com.hz.pm.api.user.helper;

import com.hz.pm.api.common.model.constant.MhUnitConst;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 * MhUnitQueryHelper
 * </p>
 *
 * @author WendyYang
 * @since 16:31 2024/8/31
 */
@Component
@RequiredArgsConstructor
public class MhUnitQueryAuthHelper {

    private final MhUnitCache unitCache;

    @Data
    public static class UnitQueryState {

        private boolean state = true;

        private List<Long> unitIds;

    }

    /**
     * 获取可查看的子部门ID列表
     *
     * @param unitId 当前单位
     * @param user   用户信息
     * @return 可查看的子部门ID列表
     */
    public UnitQueryState listCanViewUnitIds(Long unitId, UserInfoDetails user) {
        UnitQueryState state = new UnitQueryState();
        if (unitId == null) {
            if (!user.hasSuperAdmin() && MhUnitConst.ROOT_UNIT_ID != user.getMhUnitId()) {
                state.setUnitIds(unitCache.getViewChildIdsRecursion(user.getMhUnitId()));
            }
        } else {
            if (!user.hasSuperAdmin() && !unitId.equals(user.getMhUnitId())) {
                List<Long> unitIdPaths = unitCache.getUnitIdPaths(unitId);
                if (!unitIdPaths.contains(user.getMhUnitId())) {
                    state.setState(false);
                    return state;
                }
            }
            if (!unitId.equals(MhUnitConst.ROOT_UNIT_ID)) {
                state.setUnitIds(unitCache.getViewChildIdsRecursion(unitId));
            }
        }
        return state;
    }

    public UnitQueryState listCanViewUnitIds(Long unitId) {
        return listCanViewUnitIds(unitId, LoginUserUtil.loginUserDetail());
    }

}
