package com.hz.pm.api.user.security.handler;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.hz.pm.api.user.security.errorcode.AuthErrorCodeEnum;
import com.ningdatech.basic.model.ApiResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.web.session.SessionInformationExpiredEvent;
import org.springframework.security.web.session.SessionInformationExpiredStrategy;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * <p>
 * DefaultExpiredSessionStrategy
 * </p>
 *
 * @author WendyYang
 * @since 09:33 2024/9/6
 */
@Component
@RequiredArgsConstructor
public class DefaultExpiredSessionStrategy implements SessionInformationExpiredStrategy {

    private final ObjectMapper objectMapper;
    private static final ApiResponse<Void> SESSION_EXPIRED = ApiResponse.of(AuthErrorCodeEnum.SESSION_EXPIRED.getCode(),
            AuthErrorCodeEnum.SESSION_EXPIRED.getMsg());


    @Override
    public void onExpiredSessionDetected(SessionInformationExpiredEvent event)
            throws IOException {
        HttpServletResponse response = event.getResponse();
        response.setStatus(HttpStatus.UNAUTHORIZED.value());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.getWriter().write(objectMapper.writeValueAsString(SESSION_EXPIRED));
    }

}
