package com.hz.pm.api.workbench.controller;

import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.SystemReplaceInfoVO;
import com.hz.pm.api.workbench.manage.QxWorkbenchManage;
import com.hz.pm.api.workbench.manage.WorkbenchManage;
import com.hz.pm.api.workbench.model.WorkbenchProjectLibReq;
import com.hz.pm.api.workbench.model.WorkbenchReq;
import com.hz.pm.api.workbench.model.vo.*;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * WorkbenchController
 * </p>
 *
 * @author WendyYang
 * @since 18:59 2024/3/23
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "工作台控制器")
@RequestMapping("/api/v1/workbench")
public class WorkbenchController {

    private final WorkbenchManage workbenchManage;
    private final QxWorkbenchManage qxWorkbenchManage;

    /**
     * @param year \
     * @return \
     */
    @GetMapping
    public WorkbenchVO workbench(@RequestParam(required = false) Integer year) {
        if (year == null) {
            year = LocalDate.now().getYear();
        }
        return workbenchManage.getWorkbenchData(year);
    }

    @ApiOperation(value = "催办", notes = "催办")
    @PostMapping("/urging/{recordId}")
    @WebLog("催办")
    public String urging(@PathVariable Long recordId) {
        return workbenchManage.urging(recordId);
    }

    @GetMapping("/projectTotalView")
    @ApiOperation("获取项目总览")
    public ProjectTotalViewVO projectTotalView(WorkbenchReq req) {
        return workbenchManage.projectTotalView(req);
    }

    @GetMapping("/projectProcessStageStatistics")
    @ApiOperation("各阶段项目数据统计")
    public List<ProjectProcessStatVO> projectProcessStageStatistics(WorkbenchReq req) {
        return workbenchManage.projectProcessStageStatistics(req);
    }

    @GetMapping("/pageProjectLib")
    @ApiOperation("各阶段项目列表")
    public PageVo<ProjectLibListItemVO> pageProjectLib(WorkbenchProjectLibReq req) {
        return workbenchManage.pageProjectLib(req);
    }

    @GetMapping("/systemReplaceInfos")
    @ApiOperation("获取项目替代情况")
    public SystemReplaceInfoVO systemReplaceInfos(WorkbenchProjectLibReq req) {
        return workbenchManage.listSystemReplaceInfos(req);
    }

    @GetMapping("/projectEarlyWarnings")
    @ApiOperation("获取项目预警")
    public WorkbenchProjectEarlyWarnVO projectEarlyWarnings(WorkbenchReq req) {
        return workbenchManage.projectEarlyWarnings(req);
    }

    @GetMapping("/qx/projectStatistics")
    @ApiOperation("区县项目统计")
    public QxProjectStatisticsVO qxProjectStatistics(WorkbenchReq req) {
        return qxWorkbenchManage.projectStatistics(req);
    }

    @GetMapping("/projectMonitorEarlyWarnings")
    @ApiOperation("项目监测预警")
    public List<WorkbenchProjectMonitorEarlyWarnVO> projectMonitorEarlyWarnings(WorkbenchReq req) {
        return workbenchManage.projectMonitorEarlyWarnings(req);
    }

    @GetMapping("/pageMajorProjects")
    @ApiOperation("获取重大项目动态")
    public PageVo<ProjectLibListItemVO> listMajorProjects(WorkbenchProjectLibReq req) {
        return workbenchManage.pageMajorProjects(req);
    }

}
