package com.hz.pm.api.workbench.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.MhUnitConst;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.projectlib.entity.QxProject;
import com.hz.pm.api.projectlib.entity.QxProjectSystemReplaceInfos;
import com.hz.pm.api.projectlib.entity.QxPurchase;
import com.hz.pm.api.projectlib.service.IQxProjectService;
import com.hz.pm.api.projectlib.service.IQxProjectSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IQxPurchaseService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.workbench.model.WorkbenchReq;
import com.hz.pm.api.workbench.model.vo.QxProjectStatisticsVO;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * QxWorkbenchManage
 * </p>
 *
 * @author WendyYang
 * @since 14:15 2024/8/31
 */
@Component
@RequiredArgsConstructor
public class QxWorkbenchManage {

    private final IQxProjectService qxProjectService;
    private final MhUnitCache mhUnitCache;
    private final IQxPurchaseService qxPurchaseService;
    private final IQxProjectSystemReplaceInfosService qxProjectSystemReplaceInfosService;


    public QxProjectStatisticsVO projectStatistics(WorkbenchReq req) {
        LambdaQueryWrapper<QxProject> query = Wrappers.lambdaQuery(QxProject.class)
                .eq(req.getProjectYear() != null, QxProject::getProjectYear, req.getProjectYear())
                .eq(req.getUnitStrip() != null, QxProject::getUnitStrip, req.getUnitStrip())
                .select(QxProject::getId);
        if (req.getUnitId() != null && req.getUnitId() != MhUnitConst.ROOT_UNIT_ID) {
            query.in(QxProject::getBuildOrgCode, mhUnitCache.getViewChildIdsRecursion(req.getUnitId()));
        }
        List<QxProject> projects = qxProjectService.list(query);
        if (projects.isEmpty()) {
            return null;
        }
        List<Long> projectIds = CollUtils.fieldList(projects, QxProject::getId);
        QxProjectStatisticsVO stat = new QxProjectStatisticsVO();
        stat.setDeclaredCount(projects.size());
        Wrapper<QxPurchase> purchaseQuery = Wrappers.lambdaQuery(QxPurchase.class)
                .select(QxPurchase::getPurchaseBudget, QxPurchase::getContractAmount,
                        QxPurchase::getFinalAcceptDate)
                .in(QxPurchase::getProjectId, projectIds);
        List<QxPurchase> purchases = qxPurchaseService.list(purchaseQuery);
        if (CollUtil.isNotEmpty(purchases)) {
            stat.setPurchasedCount(purchases.size());
            stat.setAcceptedCount(CollUtil.count(purchases, purchase -> purchase.getFinalAcceptDate() != null));
            stat.setTotalBudget(DecimalUtil.sum(purchases, QxPurchase::getPurchaseBudget));
            stat.setTotalContractAmount(DecimalUtil.sum(purchases, QxPurchase::getContractAmount));
            QueryWrapper<QxProjectSystemReplaceInfos> wrapper = new QueryWrapper<>();
            wrapper.select("count(distinct project_id) \"adaptCount\"");
            wrapper.in("project_id", projectIds);
            wrapper.isNotNull("has_security_eval");
            Map<String, Object> map = qxProjectSystemReplaceInfosService.getMap(wrapper);
            stat.setUpgradedCount(MapUtil.getInt(map, "adaptCount"));
        }
        return stat;
    }


}
