package com.hz.pm.api.workbench.model.vo;

import cn.hutool.core.util.ObjectUtil;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.sys.model.vo.NoticeListItemVO;
import com.hz.pm.api.sys.model.vo.WflowEarlyWarningRecordsVO;
import com.hz.pm.api.todocenter.model.vo.TodoCenterStatisticsVO;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Classname WorkbenchVO
 * @Description
 * @Date 2023/3/20 9:33
 * @Author PoffyZhang
 */
@Data
public class WorkbenchVO {


    @ApiModelProperty("项目动态")
    public List<ProjectLibListItemVO> projects;

    @ApiModelProperty("公告列表")
    public Map<String, List<NoticeListItemVO>> noticeList;

    private TodoCenterStatisticsVO todoCerter;

    @ApiModelProperty("项目预警记录")
    public WorkbenchProjectEarlyWarnVO workbenchProjectEarlyWarnVO;


    @Data
    public static class EarlyWarning {
        @ApiModelProperty("类型")
        private Integer type;

        @ApiModelProperty("类型名称")
        private String typeName;

        @ApiModelProperty("总数")
        private Integer total;

        @ApiModelProperty("预警记录")
        private List<WflowEarlyWarningRecordsVO> records;
    }

    @Data
    public static class DeclaredStatistics {
        @ApiModelProperty("总项目数")
        private Integer totalNum = 0;
        @ApiModelProperty("建设项目数")
        private Integer buildNum = 0;
        @ApiModelProperty("运维项目数")
        private Integer operationNum = 0;

        @ApiModelProperty("待立项数")
        private Integer notApprovedNum = 0;
        @ApiModelProperty("已立项数")
        private Integer approvedNum = 0;
        @ApiModelProperty("建设中数")
        private Integer constructionNum = 0;
        @ApiModelProperty("待验收数")
        private Integer tobeInspectedNum = 0;
        @ApiModelProperty("已归档数")
        private Integer archivedNum = 0;

        @ApiModelProperty("申报总金额")
        private BigDecimal declaredAmount = BigDecimal.ZERO;
        @ApiModelProperty("立项总金额")
        private BigDecimal approvalAmount = BigDecimal.ZERO;


        @ApiModelProperty("申报总金额-去年")
        private BigDecimal declaredAmountLastYear = BigDecimal.ZERO;
        @ApiModelProperty("立项总金额-去年")
        private BigDecimal approvalAmountLastYear = BigDecimal.ZERO;
        @ApiModelProperty("申报总金额-较去年 增长")
        private BigDecimal declaredAmountGrowthRate = BigDecimal.ZERO;
        @ApiModelProperty("立项总金额-较去年 增长")
        private BigDecimal approvalAmountGrowthRate = BigDecimal.ZERO;

        public BigDecimal getDeclaredAmountGrowthRate() {
            if (Objects.isNull(declaredAmount)) {
                this.declaredAmount = BigDecimal.ZERO;
            }

            if (!ObjectUtil.hasNull(declaredAmountLastYear, declaredAmountLastYear)
                    && declaredAmountLastYear.compareTo(BigDecimal.ZERO) > 0) {
                return declaredAmount.subtract(declaredAmountLastYear).multiply(BigDecimal.valueOf(100))
                        .divide(declaredAmountLastYear, RoundingMode.CEILING)
                        .setScale(BigDecimal.ROUND_CEILING, RoundingMode.CEILING);
            }
            return BigDecimal.ZERO;
        }

        public BigDecimal getApprovalAmountGrowthRate() {
            if (Objects.isNull(approvalAmount)) {
                this.approvalAmount = BigDecimal.ZERO;
            }

            if (!ObjectUtil.hasNull(declaredAmountLastYear, approvalAmountLastYear)
                    && approvalAmountLastYear.compareTo(BigDecimal.ZERO) > 0) {
                return approvalAmount.subtract(approvalAmountLastYear).multiply(BigDecimal.valueOf(100))
                        .divide(approvalAmountLastYear, RoundingMode.CEILING)
                        .setScale(BigDecimal.ROUND_CEILING, RoundingMode.CEILING);
            }
            return BigDecimal.ZERO;
        }
    }
}
