package com.hz.pm.api.todocenter;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.AppTests;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.MhFileClient;
import com.hz.pm.api.external.model.dto.MhZwddWorkNoticeDTO;
import com.hz.pm.api.external.todo.MHTodoClient;
import com.hz.pm.api.external.todo.dto.MhTodoExtraParamDTO;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.model.entity.MhProject;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.service.IMhProjectService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.ningdatech.file.service.FileService;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * MhApiClientTest
 * </p>
 *
 * @author WendyYang
 * @since 10:06 2024/3/13
 */
public class MhApiClientTest extends AppTests {

    @Autowired
    private MHTodoClient mhTodoClient;
    @Autowired
    private MhApiClient mhApiClient;
    @Autowired
    private MhFileClient mhFileClient;
    @Autowired
    private FileService fileService;

    @Test
    public void sendTodo() {
        UserFullInfoDTO assignee = new UserFullInfoDTO();
        assignee.setUserId(150L);
        assignee.setMhUserId("62697f4ed80d823fa1f26c5a082d989e");
        MhTodoExtraParamDTO paramObj = MhTodoExtraParamDTO.builder()
                .bidId(117L)
                .path("/toDoCenter/handleDuringExamine")
                .projectId(2333L)
                .userName("杨云迪")
                .nodeId("node_65f010952418b30a56467594")
                .taskId("a733d3fb-e049-11ee-aa3b-024410885fae")
                .instanceId("8edc9888-e049-11ee-aa3b-024410885fae")
                .build();
        mhTodoClient.addTodo(paramObj, assignee, MHTodoTypeEnum.XCFHX_INSPECT_AUDIT, "1212121", null);
    }

    @Test
    public void test() {
        MhZwddWorkNoticeDTO notice = MhZwddWorkNoticeDTO.builder()
                .title(MHTodoTypeEnum.DECLARED_RECORD_AUDIT.getVal())
                .targetUser("ce44b47503147319fc3b5c826cb51fd2")
                .content(String.format("【%s】的【%s】需要您审核，请及时处理。",
                        "市住保房管局深化信创项目", "立项备案审批"))
                .build();
        mhApiClient.sendZwddWorkNotice(notice);
    }

    @Autowired
    private IProjectService projectService;
    @Autowired
    private IProjectStatusChangeService projectStatusChangeService;
    @Autowired
    private IMhProjectService mhProjectService;


    @Test
    public void test1() {
        List<MhProject> projects = mhProjectService.list();
        for (MhProject project : projects) {
            if (project.getProjectCode() == null) {
                continue;
            }
            projectStatusChangeService.remove(Wrappers.lambdaQuery(ProjectStatusChange.class)
                    .in(ProjectStatusChange::getEvent,
                            ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT,
                            ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS,
                            ProjectStateChangeEvent.PROJECT_REVIEW_SUBMIT,
                            ProjectStateChangeEvent.PROJECT_REVIEW_PASS)
                    .eq(ProjectStatusChange::getProjectCode, project.getProjectCode()));
            List<ProjectStatusChange> changes = new ArrayList<>();
            changes.add(new ProjectStatusChange() {{
                setAfterStatus(ProjectStatus.ON_COMPLIANCE_REVIEW.getCode());
                setBeforeStatus(ProjectStatus.TO_BE_DECLARED.getCode());
                setEvent(ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT.name());
                setCreateOn(LocalDateTime.now());
                setProjectCode(project.getProjectCode());
                setProjectId(ProjectIdCodeCacheUtil.newest(project.getProjectCode()));
            }});
            changes.add(new ProjectStatusChange() {{
                setAfterStatus(ProjectStatus.WITHOUT_PROJECT_REVIEW.getCode());
                setBeforeStatus(ProjectStatus.ON_COMPLIANCE_REVIEW.getCode());
                setEvent(ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS.name());
                setCreateOn(LocalDateTime.now().plusMinutes(1));
                setProjectCode(project.getProjectCode());
                setProjectId(ProjectIdCodeCacheUtil.newest(project.getProjectCode()));
            }});
            changes.add(new ProjectStatusChange() {{
                setAfterStatus(ProjectStatus.ON_PROJECT_REVIEW.getCode());
                setBeforeStatus(ProjectStatus.WITHOUT_PROJECT_REVIEW.getCode());
                setEvent(ProjectStateChangeEvent.PROJECT_REVIEW_SUBMIT.name());
                setCreateOn(LocalDateTime.now().plusMinutes(2));
                setProjectCode(project.getProjectCode());
                setProjectId(ProjectIdCodeCacheUtil.newest(project.getProjectCode()));
            }});
            changes.add(new ProjectStatusChange() {{
                setAfterStatus(ProjectStatus.ON_ANNUAL_PLAN.getCode());
                setBeforeStatus(ProjectStatus.ON_PROJECT_REVIEW.getCode());
                setEvent(ProjectStateChangeEvent.PROJECT_REVIEW_PASS.name());
                setCreateOn(LocalDateTime.now().plusMinutes(3));
                setProjectCode(project.getProjectCode());
                setProjectId(ProjectIdCodeCacheUtil.newest(project.getProjectCode()));
            }});
            projectStatusChangeService.saveBatch(changes);
        }
    }

}
