package com.hz.pm.api.warning;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.StopWatch;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.util.EnvironmentUtil;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.WarningFlowTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.WarningNoticeTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.scheduler.contants.TaskConstant;
import com.hz.pm.api.sys.enumeration.ProjectEarlyWarningStatusEnum;
import com.hz.pm.api.sys.manage.EarlyWarningManage;
import com.hz.pm.api.sys.model.entity.ProjectEarlyWarning;
import com.hz.pm.api.sys.model.entity.WflowEarlyWarningRecords;
import com.hz.pm.api.sys.service.IEarlyWarningRecordsService;
import com.hz.pm.api.sys.service.IProjectEarlyWarningService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.entity.UserInfo;
import com.hz.pm.api.user.model.enumeration.RoleEnum;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.StrPool;
import com.wflow.bean.entity.WflowEarlyWarning;
import com.wflow.bean.entity.WflowModels;
import com.wflow.enums.WarningRuleTypeEnum;
import com.wflow.service.IEarlyWarningService;
import com.wflow.workflow.service.ProcessModelService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.junit.Test;

import com.alibaba.fastjson.JSON;
import com.hz.pm.api.AppTests;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Classname DraftTest
 * @Description
 * @Date 2023/2/7 10:56
 * @Author PoffyZhang
 */
@Slf4j
public class DelayWarningTest extends AppTests {

    @Autowired
    private IEarlyWarningService earlyWarningService;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private EarlyWarningManage earlyWarningManage;
    @Autowired
    private IProjectEarlyWarningService projectEarlyWarningService;
    @Autowired
    private IProjectStatusChangeService projectStatusChangeService;
    @Autowired
    private UserInfoHelper userInfoHelper;
    @Autowired
    private EnvironmentUtil environmentUtil;
    @Autowired
    private IEarlyWarningRecordsService earlyWarningRecordsService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private IPurchaseStatusChangeService purchaseStatusChangeService;
    @Autowired
    private MhUnitCache mhUnitCache;
    @Autowired
    private ProcessModelService processModelService;

    private ChronoUnit getTimeUnit() {
        if (environmentUtil.isDevEnv()) {
            return ChronoUnit.MINUTES;
        } else {
            return ChronoUnit.DAYS;
        }
    }

    @Test
    public void one(){
        log.info("=========== 滞后预警任务开始 ===========");
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        // 获取项目库所有建设单位不为空的项目
        LambdaQueryWrapper<Project> queryWrapper = Wrappers.lambdaQuery(Project.class)
                .isNotNull(Project::getBuildOrgCode)
                .eq(Project::getNewest, Boolean.TRUE);
        List<Project> projects = projectService.list(queryWrapper);
         projects = projects.stream().filter(project -> project.getProjectCode().equals("330100000202700003")).collect(Collectors.toList());
        List<ProjectEarlyWarning> projectEarlyWarnings = Lists.newArrayList();
        List<WflowEarlyWarningRecords> earlyWarningRecords = Lists.newArrayList();
        List<ProjectEarlyWarning> needRemoved = Lists.newArrayList();
        // 查询当前预警表中 已存在的的滞后预警数据
        Wrapper<ProjectEarlyWarning> warningQuery = Wrappers.lambdaQuery(ProjectEarlyWarning.class)
                .eq(ProjectEarlyWarning::getRuleType, WarningRuleTypeEnum.DELAY_WARNING.getCode());
        List<ProjectEarlyWarning> earlyWarnings = projectEarlyWarningService.list(warningQuery);
        Map<String, ProjectEarlyWarning> projectEarlyWarningMap = CollUtils.listToMap(earlyWarnings, c -> c.getProjectCode() + StrPool.DASH + c.getStepName());
        // 遍历项目 获取对应申报单位的滞后预警规则
        for (Project project : projects) {
            String buildOrgCode = project.getBuildOrgCode();
            String projectCode = project.getProjectCode();
            Long mhUnitId = Long.valueOf(buildOrgCode);
            List<Long> unitIdPaths = mhUnitCache.getUnitIdPaths(mhUnitId);
            if (unitIdPaths.isEmpty()) {
                log.error("该单位无法进行滞后预警：{}", mhUnitId);
                continue;
            }
            List<String> unitIdPathsStr = CollUtils.convert(unitIdPaths, String::valueOf);
            String orderSql = String.format(" order by field(mh_unit_id, %s) desc limit 1",
                    CollUtils.join(unitIdPaths, w -> "'" + w + "'", StrUtil.COMMA));
            log.info("单位ID路径：{}", unitIdPathsStr);
            // 从当前建设单位开始向上获取滞后预警规则
            WflowEarlyWarning warning = earlyWarningService.getOne(Wrappers.lambdaQuery(WflowEarlyWarning.class)
                    .eq(WflowEarlyWarning::getRuleType, WarningRuleTypeEnum.DELAY_WARNING.getCode())
                    .eq(WflowEarlyWarning::getIsOpen, true)
                    .in(WflowEarlyWarning::getMhUnitId, unitIdPathsStr)
                    .last(orderSql));
            if (Objects.isNull(warning)){
                log.info("未找到对应单位：{} 的滞后预警规则", mhUnitId);
                continue;
            }
            String rule = warning.getRule();
            if (StringUtils.isNotBlank(rule)) {
                JSONArray ruleArray = JSON.parseArray(rule);
                // 对JSONArray 根据"biz"字段从小到大排序
                ruleArray.sort((o1, o2) -> {
                    JSONObject o1Json = JSON.parseObject(JSON.toJSONString(o1));
                    JSONObject o2Json = JSON.parseObject(JSON.toJSONString(o2));
                    Integer o1Biz = o1Json.getInteger("biz");
                    Integer o2Biz = o2Json.getInteger("biz");
                    return o1Biz.compareTo(o2Biz);
                });
                if (CollUtil.isNotEmpty(ruleArray)) {
                    ruleArray.forEach(r -> {
                        JSONObject rJson = JSON.parseObject(JSON.toJSONString(r));
                        Integer biz = rJson.getInteger("biz");
                        Integer delayTime = rJson.getInteger("delayTime");
                        String notice = rJson.getString("notice");
                        WarningFlowTypeEnum flowType = WarningFlowTypeEnum.getByCode(biz);
                        if (Objects.isNull(delayTime) || Objects.isNull(biz) || flowType == null) {
                            log.warn("规则数据 错误 :{}", rJson);
                            return;
                        }
                        String path = flowType.getPath();
                        Map<String, ProjectStatusChange> projectStatusChangeMap = MapUtil.newHashMap();
                        String stepName = null;
                        List<String> projectCodes = Lists.newArrayList();
                        projectCodes.add(project.getProjectCode());
                        // 是否进行采购&合同备案
                        boolean isPurchase = false;
                        // 适配改造是否完成单位确认
                        boolean isModified = false;
                        // 终验申请是否提交
                        boolean isFinalInspection = false;
                        switch (flowType) {
                            // 已立项未采购 获取 立项时间
                            case APPROVED_PROJECT_NOT_PURCHASED:{
                                stepName = TaskConstant.StepName.PURCHASE;
                                if (CollUtil.isNotEmpty(projectCodes)) {
                                    Map<String, ProjectStatusChange> approvedProjectNotPurchasedMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                            ProjectStateChangeEvent.DECLARED_RECORD_PASS);
                                    List<ProjectStateChangeEvent> events = Lists.newArrayList();
                                    events.add(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                                    ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                                    // 说明进行了采购&合同信息备案
                                    if (Objects.nonNull(projectStatusChange)) {
                                       isPurchase = true;
                                    }
                                    if (CollUtil.isNotEmpty(approvedProjectNotPurchasedMap)) {
                                        projectStatusChangeMap.putAll(approvedProjectNotPurchasedMap);
                                    }
                                }
                            }
                            break;
                            // 已采购未改造完成 获取合同签订时间
                            case PURCHASED_UNMODIFIED:{
                                stepName = TaskConstant.StepName.ADAPT_MODIFY;
                                if (CollUtil.isNotEmpty(projectCodes)) {
                                    Map<String, ProjectStatusChange> purchasedUnmodifiedMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                            ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                                    if (CollUtil.isNotEmpty(purchasedUnmodifiedMap)) {
                                        projectStatusChangeMap.putAll(purchasedUnmodifiedMap);
                                    }
                                    List<TenderStateChangeEvent> events = Lists.newArrayList();
                                    events.add(TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                                    PurchaseStatusChange purchaseStatusChange = purchaseStatusChangeService.getLastOne(projectCode, events);
                                    // 说明进行了单位确认
                                    if (Objects.nonNull(purchaseStatusChange)) {
                                        isModified = true;
                                    }
                                }
                            }
                            break;
                            // 已改造完成未终验 获取单位确认时间
                            case MODIFIED_NOT_FINAL_INSPECTION:{
                                stepName = TaskConstant.StepName.FINAL;
                                if (CollUtil.isNotEmpty(projectCodes)) {
                                    Map<String, ProjectStatusChange> modifiedNotFinalInspectionMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                            ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                                    if (CollUtil.isNotEmpty(modifiedNotFinalInspectionMap)) {
                                        projectStatusChangeMap.putAll(modifiedNotFinalInspectionMap);
                                    }
                                }
                                List<ProjectStateChangeEvent> events = Lists.newArrayList();
                                events.add(ProjectStateChangeEvent.FINAL_ACCEPTANCE_APPLICATION);
                                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                                // 说明提交了终验申请
                                if (Objects.nonNull(projectStatusChange)) {
                                    isFinalInspection = true;
                                }
                            }
                            break;
                            default:
                                log.error("Unexpected value: {}", flowType);
                        }
                        // 需要首页监测预警展示的项目
                        // 获取项目 滞后(历时)时间
                        HashMap<String, LocalDateTime> touchTimeMap = MapUtil.newHashMap();
                        String finalStepName = stepName;
                        ProjectStatusChange change = projectStatusChangeMap.get(projectCode);
                        LocalDateTime touchTime = change == null ? null : change.getCreateOn();
                        // 是否进行预警
                        boolean isTouch = isTouch(flowType, isPurchase, isModified, isFinalInspection);
                        if (touchTime != null) {
                            touchTimeMap.put(projectCode + StrPool.DASH + biz, touchTime);
                            long duration = getDurationTime(touchTime,LocalDateTime.now(),getTimeUnit());
                            // 如果对应节点超期 且未提交至下一步
                            if (duration > delayTime && isTouch) {
                                List<ProjectEarlyWarning> projectEarlyWarningList = Lists.newArrayList();
                                List<WflowEarlyWarningRecords> earlyWarningRecordsList = Lists.newArrayList();
                                // 项目 滞后预警
                                projectEarlyWarningList = projectEarlyWarningService.earlyWarning(project, WarningRuleTypeEnum.DELAY_WARNING.getCode(), com.wflow.enums.WarningNoticeTypeEnum.DELAY.getCode(), finalStepName, projectEarlyWarningList);
                                projectEarlyWarnings.addAll(projectEarlyWarningList);

                                // 目前没有通知
                                // 如果去预警通知人 1.超级管理员 2.单位管理员 如果都包含 都要发
                                List<String> batchEmployees = Lists.newArrayList();
                                if (StringUtils.isNotBlank(notice) && notice.contains(WarningNoticeTypeEnum.SUPER_ADMIN.getCode().toString())) {
                                    // 获取超级管理员用户信息
                                    List<UserInfo> superAdminUsers = userInfoHelper.listUsersByRole(RoleEnum.SUPER_ADMIN);
                                    List<String> superAdminEmployeeCodes = superAdminUsers.stream().map(UserInfo::getEmployeeCode).collect(Collectors.toList());
                                    batchEmployees.addAll(superAdminEmployeeCodes);
                                }
                                if (StringUtils.isNotBlank(notice) && notice.contains(WarningNoticeTypeEnum.UNIT_ADMIN.getCode().toString())) {
                                    // 获取单位管理员用户信息
                                    List<UserInfo> unitAdminUsers = userInfoHelper.listUsersByRole(RoleEnum.COMPANY_MANAGER);
                                    // 过滤出当前项目所属单位的单位管理员
                                    List<String> unitAdminEmployeeCodes = unitAdminUsers.stream().filter(user -> Objects.equals(user.getMhUnitId().toString(), project.getBuildOrgCode()))
                                            .map(UserInfo::getEmployeeCode).collect(Collectors.toList());
                                    batchEmployees.addAll(unitAdminEmployeeCodes);
                                }
                                // 保存到预警记录中
                                earlyWarningRecordsList = earlyWarningManage.getEarlyWarningDelayRecords(warning.getNoticeMethod(), biz, project,
                                        WarningRuleTypeEnum.DELAY_WARNING.getCode(), ProjectEarlyWarningStatusEnum.DELAY_TIME.getCode(),
                                        path, touchTimeMap, earlyWarningRecordsList);
                                earlyWarningRecords.addAll(earlyWarningRecordsList);
                            }else {
                                // 判断是否已经存在对应节点预警 如果有 从预警信息表中移除
                                if (projectEarlyWarningMap.containsKey(project.getProjectCode() + StrPool.DASH + finalStepName)) {
                                    ProjectEarlyWarning projectEarlyWarning = projectEarlyWarningMap.get(projectCode + StrPool.DASH + finalStepName);
                                    needRemoved.add(projectEarlyWarning);
                                }
                            }
                        }
                    });
                }
            }
        }
        // 每个规则按通知节点顺序 更新保存一次
        if (CollUtil.isNotEmpty(needRemoved)){
            projectEarlyWarningService.removeBatchByIds(needRemoved);
        }
        if (CollUtil.isNotEmpty(projectEarlyWarnings)){
            log.info("需要滞后预警通知的项目数：{}", projectEarlyWarnings.size());
            projectEarlyWarningService.saveOrUpdateBatch(projectEarlyWarnings);
        }
        if (CollUtil.isNotEmpty(earlyWarningRecords)){
            log.info("需要滞后预警通知记录数：{}", earlyWarningRecords.size());
            earlyWarningRecordsService.saveOrUpdateBatch(earlyWarningRecords);
        }
        stopWatch.stop();
        log.info("=========== 滞后预警任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
    }

    private boolean isTouch(WarningFlowTypeEnum flowType, Boolean isPurchase, Boolean isModified, Boolean isFinalInspection) {
        switch (flowType) {
            case APPROVED_PROJECT_NOT_PURCHASED:
                return !isPurchase;
            case PURCHASED_UNMODIFIED:
                return !isModified;
            case MODIFIED_NOT_FINAL_INSPECTION:
                return !isFinalInspection;
            default:
                return false;
        }
    }

    @Test
    public void two(){
        log.info("=========== 预警项目状态校验任务开始 ===========");
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE));

        if (CollUtil.isEmpty(projects)) {
            return;
        }

        Set<String> projectSet = Sets.newHashSet();
        Map<String, Project> projectMap = projects.stream().filter(p -> projectSet.add(p.getProjectCode()))
                .collect(Collectors.toMap(Project::getProjectCode, p -> p));
        List<ProjectEarlyWarning> warnings = projectEarlyWarningService.list();

        // 删除 已经不存在的项目
        List<ProjectEarlyWarning> toRemove = warnings.stream()
                .filter(w -> !projectMap.containsKey(w.getProjectCode()))
                .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(toRemove)) {
            List<Long> removeIds = toRemove.stream().map(ProjectEarlyWarning::getId)
                    .collect(Collectors.toList());
            projectEarlyWarningService.removeBatchByIds(removeIds);
        }

        //  去查询 各个报警 是否已经正常
        List<ProjectEarlyWarning> trueWarnings = warnings.stream().filter(w -> Objects.nonNull(w.getNormal()) && !w.getNormal()).collect(Collectors.toList());
        if (CollUtil.isEmpty(trueWarnings)) {
            return;
        }
        checkNormal(trueWarnings);
        stopWatch.stop();
        log.info("=========== 预警项目状态校验任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
    }

    @Test
    public void three(){
        List<Long> unitIdPaths = mhUnitCache.getUnitIdPaths(32L);
        System.out.println(unitIdPaths);
        List<String> unitIdPathsStr = CollUtils.convert(unitIdPaths, String::valueOf);
        String orderSql = String.format(" order by field(region_code, %s) desc limit 1",
                CollUtils.join(unitIdPaths, w -> "'" + w + "'", StrUtil.COMMA));
        System.out.println("单位ID路径:" + unitIdPathsStr);
        WflowModels one = processModelService.getOne(Wrappers.lambdaQuery(WflowModels.class)
                .eq(WflowModels::getProcessType, ProjectProcessType.PROJECT_RECORD_APPROVAL_PROCESS.getCode())
                .in(WflowModels::getRegionCode, unitIdPathsStr)
                .last(orderSql));
        System.out.println(one);
    }

    private long getDurationTime(LocalDateTime touchTime, LocalDateTime now, ChronoUnit timeUnit) {
        return timeUnit.between(touchTime, now);
    }

    // 3.去查询 各个报警 是否已经正常
    private void checkNormal(List<ProjectEarlyWarning> trueWarnings) {
        // 滞后预警 如果正常了 要改回正常
        List<ProjectEarlyWarning> delayWarning = trueWarnings.stream().filter(w ->
                        Objects.nonNull(w.getDelayWarning())
                                && w.getDelayWarning())
                .collect(Collectors.toList());
        checkDelayWarning(delayWarning);
    }

    private void checkDelayWarning(List<ProjectEarlyWarning> delayWarning) {
        if (CollUtil.isEmpty(delayWarning)) {
            return;
        }
        for (ProjectEarlyWarning delay : delayWarning) {
            String projectCode = delay.getProjectCode();
            Project project = projectService.getProjectByCode(projectCode);
            if (Objects.isNull(project)) {
                continue;
            }
            WflowEarlyWarningRecords record = earlyWarningRecordsService.getOne(Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .eq(WflowEarlyWarningRecords::getProjectCode, projectCode)
                    .eq(WflowEarlyWarningRecords::getRuleType, WarningRuleTypeEnum.DELAY_WARNING.getCode())
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime)
                    .last(BizConst.LIMIT_1));
            if (Objects.isNull(record)) {
                continue;
            }
            Integer biz = record.getBiz();
            WarningFlowTypeEnum flowTypeEnum = WarningFlowTypeEnum.getByCode(biz);
            if (Objects.isNull(flowTypeEnum)) {
                continue;
            }
            ProjectStatus projectStatus = flowTypeEnum.getProjectStatus();
            // 已经不是这个状态了 说明已经被提交了
            if (!projectStatus.eq(project.getStatus())) {
                checkCommited(delay, flowTypeEnum, projectCode);
                // 如果滞后预警正常了 改回正常
                if (Boolean.FALSE.equals(delay.getDelayWarning())){
                    delay.setNormal(Boolean.TRUE);
                    delay.setStatus(ProjectEarlyWarningStatusEnum.NORMAL.name());
                }
                projectEarlyWarningService.updateById(delay);
            }
        }
    }

    private void checkDeclaredWarning(List<ProjectEarlyWarning> declaredWarnings) {
        if (CollUtil.isEmpty(declaredWarnings)) {
            return;
        }
        for (ProjectEarlyWarning declaredWarning : declaredWarnings) {
            String projectCode = declaredWarning.getProjectCode();
            Project project = projectService.getProjectByCode(projectCode);
            if (Objects.isNull(project)) {
                continue;
            }
            WflowEarlyWarningRecords record = earlyWarningRecordsService.getOne(Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .eq(WflowEarlyWarningRecords::getProjectCode, projectCode)
                    .eq(WflowEarlyWarningRecords::getRuleType, WarningRuleTypeEnum.DECLARED_WARNING.getCode())
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime)
                    .last(BizConst.LIMIT_1));
            if (Objects.isNull(record)) {
                continue;
            }
            Integer biz = record.getBiz();
            WarningFlowTypeEnum flowTypeEnum = WarningFlowTypeEnum.getByCode(biz);
            if (Objects.isNull(flowTypeEnum)) {
                continue;
            }
            ProjectStatus projectStatus = flowTypeEnum.getProjectStatus();
            // 已经不是这个状态了 根据选择的节点判断是否完成了提交
            if (!projectStatus.eq(project.getStatus())) {
                checkCommited(declaredWarning, flowTypeEnum, projectCode);
                // 其它几种 都没有 说明已经正常了
                if (Boolean.FALSE.equals(declaredWarning.getProcessWarning()) &&
                        Boolean.FALSE.equals(declaredWarning.getDelayWarning())) {
                    declaredWarning.setNormal(Boolean.TRUE);
                    declaredWarning.setStatus(ProjectEarlyWarningStatusEnum.NORMAL.name());
                }
                projectEarlyWarningService.updateById(declaredWarning);
            }
        }
    }

    private void checkCommited(ProjectEarlyWarning warning, WarningFlowTypeEnum flowTypeEnum, String projectCode) {
        switch (flowTypeEnum){
            // 已立项未采购 当用户采购&合同备案完成 说明进度正常
            case APPROVED_PROJECT_NOT_PURCHASED: {
                List<ProjectStateChangeEvent> events = Lists.newArrayList();
                events.add(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                // 说明进行了采购&合同信息备案
                if (Objects.nonNull(projectStatusChange)) {
                    if (WarningRuleTypeEnum.DECLARED_WARNING.getCode().equals(warning.getRuleType())) {
                        warning.setDeclaredWarning(Boolean.FALSE);
                    }else if (WarningRuleTypeEnum.DELAY_WARNING.getCode().equals(warning.getRuleType())){
                        warning.setDelayWarning(Boolean.FALSE);
                    }
                }
            }
            break;
            // 已采购未改造完成 当用户适配改造完成已完成单位确认 说明进度正常
            case PURCHASED_UNMODIFIED:{
                List<TenderStateChangeEvent> events = Lists.newArrayList();
                events.add(TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                PurchaseStatusChange purchaseStatusChange = purchaseStatusChangeService.getLastOne(projectCode, events);
                // 说明进行了单位确认
                if (Objects.nonNull(purchaseStatusChange)) {
                    if (WarningRuleTypeEnum.DECLARED_WARNING.getCode().equals(warning.getRuleType())) {
                        warning.setDeclaredWarning(Boolean.FALSE);
                    }else if (WarningRuleTypeEnum.DELAY_WARNING.getCode().equals(warning.getRuleType())){
                        warning.setDelayWarning(Boolean.FALSE);
                    }
                }

            }
            break;
            // 已改造完成未终验 当用户提交终验申请 说明进度正常
            case MODIFIED_NOT_FINAL_INSPECTION:{
                List<ProjectStateChangeEvent> events = Lists.newArrayList();
                events.add(ProjectStateChangeEvent.FINAL_ACCEPTANCE_APPLICATION);
                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                // 说明提交了终验申请
                if (Objects.nonNull(projectStatusChange)) {
                    if (WarningRuleTypeEnum.DECLARED_WARNING.getCode().equals(warning.getRuleType())) {
                        warning.setDeclaredWarning(Boolean.FALSE);
                    }else if (WarningRuleTypeEnum.DELAY_WARNING.getCode().equals(warning.getRuleType())){
                        warning.setDelayWarning(Boolean.FALSE);
                    }
                }
            }
            break;
        }
    }

    /**
     * 流程 查看 是否已经处理过
     *
     * @param processWarnings
     */
    private void checkProcessWarning(List<ProjectEarlyWarning> processWarnings) {
        if (CollUtil.isEmpty(processWarnings)) {
            return;
        }
        for (ProjectEarlyWarning processWarning : processWarnings) {
            String projectCode = processWarning.getProjectCode();
            WflowEarlyWarningRecords record = earlyWarningRecordsService.getOne(Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .eq(WflowEarlyWarningRecords::getProjectCode, projectCode)
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime)
                    .last(BizConst.LIMIT_1));
            if (Objects.isNull(record)) {
                continue;
            }
            String nodeId = record.getNodeId();
            if (StringUtils.isBlank(nodeId)) {
                continue;
            }
            List<Task> tasks = taskService.createTaskQuery()
                    .taskDefinitionKey(nodeId)
                    .list();
            if (CollUtil.isEmpty(tasks)) {
                //说明 已经审批通过了
                processWarning.setProcessWarning(Boolean.FALSE);
                //其它三种 都没有 说明已经正常了
                if (Boolean.FALSE.equals(processWarning.getDeclaredWarning()) &&
                        Boolean.FALSE.equals(processWarning.getDelayWarning())) {
                    processWarning.setNormal(Boolean.TRUE);
                    processWarning.setStatus(ProjectEarlyWarningStatusEnum.NORMAL.name());
                }
                projectEarlyWarningService.updateById(processWarning);
            }
        }

    }

}
